@@ -41,6 +41,7 @@
 #include <time.h>
 #include <string.h>
 #include <ctype.h>
+#include <sys/stat.h>
 
 #include "abcparse.h"
 #include "abc2ps.h"
@@ -63,6 +64,9 @@ int epsf;			/* for EPSF postscript output */
 char outf[STRL1];		/* output file name */
 int  file_initialized;		/* for output file */
 FILE *fout;			/* output file */
+char *in_fname;			/* current input file name */
+char *styd = DEFAULT_FDIR;	/* format search directory */
+time_t mtime;			/* last modification time of the input file */
 
 int s_argc;			/* command line arguments */
 char **s_argv;
@@ -89,7 +93,6 @@ static int flatbeams = -1;	/* flat beams when bagpipe */
 static int graceslurs = -1;	/* slurs in grace notes */
 static float scalefac = 0;	/* scale factor for symbol size */
 static float staffsep = 0;	/* staff separation */
-static char *styd = DEFAULT_FDIR; /* format search directory */
 static int def_fmt_done = 0;	/* default format read */
 static float swidth = 0;	/* staff width */
 static int measurenb = -1;	/* measure numbering (-1: none, 0: on the left, or every n bars) */
@@ -97,7 +100,6 @@ static int measurebox = -1;	/* display measure numbers in a box */
 static int measurefirst = -1;	/* first measure number */
 static int printtempo = -1;	/* print the tempo indications */
 static char *sel;		/* current input file selector */
-char *in_fname;			/* current input file name */
 static struct SYMBOL notitle;
 
 /* memory arena (for clrarena, lvlarena & getarena) */
@@ -351,7 +353,7 @@ int main(int argc,
 						break;
 					case 'F':
 						read_def_format();
-						if (read_fmt_file(aaa, styd) < 0) {
+						if (read_fmt_file(aaa) < 0) {
 							fprintf(stderr,
 								"++++ Cannot open format file: %s\n",
 								aaa);
@@ -415,7 +417,7 @@ int main(int argc,
 
 		if (strstr(p, ".fmt")) {	/* implicit -F */
 			read_def_format();
-			if (read_fmt_file(p, styd) < 0) {
+			if (read_fmt_file(p) < 0) {
 				fprintf(stderr,
 					"++++ Cannot open format file: %s\n",
 					p);
@@ -612,7 +614,7 @@ static void read_def_format(void)
 	if (def_fmt_done)
 		return;
 	def_fmt_done = 1;
-	read_fmt_file("default.fmt", styd);
+	read_fmt_file("default.fmt");
 }
 
 /* -- read an input file -- */
@@ -640,7 +642,10 @@ static char *read_file(void)
 			free(file);
 			return 0;
 		}
+		time(&mtime);
 	} else {
+		struct stat sbuf;
+
 		if ((fin = fopen(in_fname, "rb")) == 0) {
 			char new_file[256];
 
@@ -667,6 +672,8 @@ static char *read_file(void)
 			return 0;
 		}
 		file[fsize] = '\0';
+		fstat(fileno(fin), &sbuf);
+		memcpy(&mtime, &sbuf.st_mtime, sizeof mtime);
 		fclose(fin);
 	}
 	return file;
@@ -768,7 +775,7 @@ static void whistle_parse(char *p)
 			pitch++;
 		else	pitch--;
 	}
-	if ((q = strchr(notes_tb, *p)) == 0) {
+	if (*p == '\0' || (q = strchr(notes_tb, *p)) == 0) {
 		fprintf(stderr, "++++ Bad note name in '-W'\n");
 		return;
 	}
