@@ -44,15 +44,16 @@ char *deco_tb[128];		/* decoration names */
 unsigned short micro_tb[MAXMICRO]; /* microtone values [ (n-1) | (d-1) ] */
 int severity;			/* error severity */
 
-static int abc_state;		/* parse state */
-static int gulen, ulen;		/* unit note length set by M: or L: */
+static short abc_state;		/* parse state */
+static short abc_vers, abc_vers_g;	/* current and global versions */
+static short gulen, ulen;	/* unit note length set by M: or L: */
+static short meter;		/* upper value of time sig for n-plets */
 static char *gchord;		/* guitar chord */
-static int meter;		/* upper value of time sig for n-plets */
 static struct deco dc;		/* decorations */
 static int lyric_started;	/* lyric started */
-static struct abcsym *deco_start;	/* 1st note of the line for d: / s: */
-static struct abcsym *deco_cont;	/* current symbol when d: / s: continuation */
-static int vover_bar;		/* in a simple voice overlay sequence */
+static struct abcsym *deco_start; /* 1st note of the line for d: / s: */
+static struct abcsym *deco_cont; /* current symbol when d: / s: continuation */
+static int vover;		/* voice overlay (1: single bar, -1: multi-bar */
 
 #define VOICE_NAME_SZ 64	/* max size of a voice name */
 
@@ -61,14 +62,13 @@ static int linenum;		/* current source line number */
 static int colnum;		/* current source column number */
 static unsigned char *scratch_line;	/* parse line */
 static int scratch_length = 0;	/* allocated length */
-static int line_length;		/* current line length */
 
 static short nvoice;		/* number of voices (0..n-1) */
 static struct {			/* voice table and current pointer */
 	char name[32];			/* voice name */
 	struct abcsym *last_note;	/* last note or rest */
 	short ulen;			/* unit note length */
-	char slur;			/* number of slur starts */
+	unsigned char slur;		/* slurs starting on the note */
 	char dotted_slur, dotted_tie;	/* dotted slurs / ties */
 	signed char add_pitch;		/* key transpose */
 	unsigned char mvoice;		/* main voice when voice overlay */
@@ -88,13 +88,12 @@ static struct {			/* voice table and current pointer */
 #define CHAR_BAR 10
 #define CHAR_OPAR 11
 #define CHAR_VOV 12
-#define CHAR_VOVE 13
-#define CHAR_SPAC 14
-#define CHAR_MINUS 15
-#define CHAR_CPAR 16
-#define CHAR_BRHY 17
-#define CHAR_DECOS 18
-#define CHAR_SLASH 19
+#define CHAR_SPAC 13
+#define CHAR_MINUS 14
+#define CHAR_CPAR 15
+#define CHAR_BRHY 16
+#define CHAR_DECOS 17
+#define CHAR_SLASH 18
 static char char_tb[256] = {
 	0, 0, 0, 0, 0, 0, 0, 0, 0, CHAR_SPAC, 0, 0, 0, 0, 0, 0,	/* 00 - 0f */
 	0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,		/* 10 - 1f */
@@ -142,9 +141,9 @@ static unsigned char *parse_basic_note(unsigned char *p,
 				       int *length,
 				       int *accidental,
 				       int *stemless);
-static void parse_header(struct abctune *t,
-			 unsigned char *p,
-			 char *comment);
+static void parse_info(struct abctune *t,
+		       unsigned char *p,
+		       char *comment);
 static int parse_line(struct abctune *t,
 		      unsigned char *p);
 static unsigned char *parse_note(struct abctune *t,
@@ -278,7 +277,7 @@ void abc_insert(char *file_api,
 			break;			/* blank line --> done */
 /*fixme-insert: don't accept X: nor T:*/
 		/* parse the music line */
-		if (!parse_line(t, p))
+		if (parse_line(t, p))
 			break;
 	}
 }
@@ -364,7 +363,7 @@ struct abctune *abc_parse(char *file_api)
 		}
 
 		/* parse the music line */
-		if (!parse_line(t, p))
+		if (parse_line(t, p))
 			t = 0;
 	}
 	return first_tune;
@@ -392,7 +391,7 @@ static unsigned char *decomment_line(unsigned char *p)
 		i++;
 	}
 
-	/* remove trailing blanks */
+	/* remove the trailing blanks */
 	while (--i > 0) {
 		c = *--p;
 		if (!isspace(c)) {
@@ -495,8 +494,7 @@ static unsigned char *clef_skip(unsigned char *p,
 		    || strncmp(p, "treble", 6) == 0
 		    || strncmp(p, "alto", 4) == 0
 		    || strncmp(p, "tenor", 5) == 0
-		    || strncmp(p, "perc", 4) == 0
-		    || strncmp(p, "none", 4) == 0) {
+		    || strncmp(p, "perc", 4) == 0) {
 			if (*p_name != 0)
 				syntax("Double clef name", p);
 			*p_name = p;
@@ -675,11 +673,11 @@ static void parse_clef(struct clef_s *p_clef,
 		} else if (!strncmp(name, "perc", 4)) {
 			clef = PERC;
 			name += 4;
-		} else /*if (strncmp(name, "none", 4) == 0)*/ {
+		} else if (strncmp(name, "none", 4) == 0) {
 			clef = TREBLE;
 			p_clef->invis = 1;
 			name += 4;
-		}
+		} else	syntax("Unknown clef", name);
 	}
 
 	if (clef_name) {
@@ -729,8 +727,7 @@ static void parse_clef(struct clef_s *p_clef,
 
 	p_clef->type = clef;
 	p_clef->line = clef_line;
-	p_clef->transpose = transpose;
-	curvoice->add_pitch = transpose;
+	p_clef->transpose = curvoice->add_pitch = transpose;
 }
 
 /* -- parse a 'K:' -- */
@@ -742,6 +739,12 @@ static void parse_key(unsigned char *p,
 
 	clef_name = 0;
 	clef_middle = 0;
+	if (strncmp(p, "none", 4) == 0) {
+		s->u.key.empty = 2;
+		p += 4;
+		while (isspace(*p))
+			p++;
+	}
 	p = clef_skip(p, &clef_name, &clef_middle);
 	sf = 0;
 	switch (*p++) {
@@ -767,7 +770,8 @@ static void parse_key(unsigned char *p,
 		p--;			/* explicit accidentals */
 		break;
 	case '\0':
-		s->u.key.empty = 1;
+		if (s->u.key.empty == 0)
+			s->u.key.empty = 1;
 		p--;
 		break;
 	default:
@@ -855,21 +859,6 @@ static void parse_key(unsigned char *p,
 		case '=':
 			p = parse_acc(p, s);	/* explicit accidentals */
 			continue;
-		case '+':
-		case '-':
-			if (p[1] == '8') {
-				int i, dpit;
-
-				/* "+8" / "-8" (fixme: not standard) */
-				dpit = *p == '+' ? +7 : -7;
-				if (abc_state == ABC_S_HEAD) {
-					for (i = MAXVOICE; --i >= 0; )
-						voice_tb[i].add_pitch += dpit;
-				} else	curvoice->add_pitch += dpit;
-				p += 2;
-				continue;
-			}
-			goto unk;
 		default:
 		unk:
 			syntax("Unknown token in key specifier", p);
@@ -893,6 +882,12 @@ static void parse_key(unsigned char *p,
 		s = abc_new(s->tune, 0, 0);
 		s->type = ABC_T_CLEF;
 		parse_clef(&s->u.clef, clef_name, clef_middle);
+		if (abc_state == ABC_S_HEAD) {	/* if first K: */
+			int i;
+
+			for (i = MAXVOICE; --i >= 0; )
+				voice_tb[i].add_pitch = s->u.clef.transpose;
+		}
 	}
 }
 
@@ -931,30 +926,27 @@ static char *get_len(unsigned char *p,
 static unsigned char *get_line(void)
 {
 	int l;
-	unsigned char *p;
-	unsigned char *line;
+	unsigned char *p, *line;
 
 	p = file;
 	if (*p == '\0')
 		return 0;
-	line = p; 		/* (for syntax error) */
+	line = p;
+	linenum++;
 
 	/* memorize the beginning of the next line */
 	while (*p != '\0'
 	       && *p != '\r'
-	       && *p != '\n') {
+	       && *p != '\n')
 		p++;
-	}
 	l = p - line;
-	if (*p != '\0')
-		p++;
-	/* solve PC-DOS */
-	if (p[-1] == '\r' && *p == '\n')
+	if (*p != '\0') {
 		p++;
+		if (p[-1] == '\r' && *p == '\n')	/* (IBMPC) */
+			p++;
+	}
 	file = p;
 
-	linenum++;
-
 	/* allocate space for the line */
 	if (scratch_line != 0
 	    && l >= scratch_length) {
@@ -962,15 +954,66 @@ static unsigned char *get_line(void)
 		scratch_line = 0;
 	}
 	if (scratch_line == 0) {
-		scratch_line = malloc(l + 1);
-		scratch_length = l;
+		scratch_length = l + 1;
+		scratch_line = malloc(scratch_length);
 	}
 	p = scratch_line;
 	strncpy(p, line, l);
 	p[l] = '\0';
-	line_length = l;	/* for syntax error */
 
-	return p;
+	/* special case for continuation lines in versions >= 2 */
+	if (abc_vers >= 2) {
+		for (;;) {
+			int l0;
+
+			while (*p != '\0') {
+				if (*p == '\\'
+				    && p[1] != '%') {	/* (??) */
+					line = p + 1;
+					while (isspace(*line))
+						line++;
+					if (*line == '\0' || *line == '%')
+						break;
+				}
+				p++;
+			}
+			if (*p == '\0')
+				break;
+			*p = '\0';		/* remove the end of line */
+/*fixme:and lose the comment!*/
+
+			p = file;
+			if (*p == '\0')
+				break;
+			line = p;
+			linenum++;
+			while (*p != '\0'
+			       && *p != '\r'
+			       && *p != '\n')
+				p++;
+			l0 = p - line;
+			if (*p != '\0') {
+				p++;
+				if (p[-1] == '\r' && *p == '\n')
+					p++;
+			}
+			file = p;
+			p = scratch_line;
+			l = strlen(p);
+			if (l + l0 >= scratch_length) {
+				scratch_length = l + l0 + 1;
+				scratch_line = malloc(scratch_length);
+				strcpy(scratch_line, p);
+				free(p);
+				p = scratch_line;
+			}
+			p += l;
+			strncpy(p, line, l0);
+			p[l0] = '\0';
+		}
+	}
+
+	return scratch_line;
 }
 
 /* -- parse a 'M:' -- */
@@ -1086,7 +1129,7 @@ static void parse_staves(unsigned char *p,
 			staff->flags |= CLOSE_BRACKET;
 			break;
 		case '{':
-			if (flags & (OPEN_BRACKET | OPEN_BRACE | OPEN_PARENTH))
+			if (flags & (OPEN_BRACE | OPEN_PARENTH))
 				goto err;
 			flags |= OPEN_BRACE;
 			staff = 0;
@@ -1115,12 +1158,15 @@ static void parse_staves(unsigned char *p,
 		default:
 			if (!isalnum(*p) && *p != '_')
 				goto err;
+			if (voice >= MAXVOICE) {
+				syntax("Too many voices in %%staves", p);
+				return;
+			}
 			{
 				int v;
 
 				p = def_voice(p, &v);
-				staff = &s->u.staves[voice];
-				voice++;
+				staff = &s->u.staves[voice++];
 				staff->voice = v;
 				staff->name = alloc_f(strlen(voice_tb[v].name) + 1);
 				strcpy(staff->name, voice_tb[v].name);
@@ -1141,15 +1187,20 @@ static void parse_staves(unsigned char *p,
 	for (voice = 0, staff = s->u.staves;
 	     voice <= MAXVOICE && staff->name;
 	     voice++, staff++) {
-		if (staff->flags & flags)
-			goto err;
-		if (staff->flags & CLOSE_PARENTH)
-			flags = flags2;
+		if ((staff->flags & (OPEN_PARENTH | CLOSE_PARENTH))
+				== (OPEN_PARENTH | CLOSE_PARENTH))
+			;			/* voice alone in a staff */
+		else {
+			if (staff->flags & flags)
+				goto err;
+			if (staff->flags & CLOSE_PARENTH)
+				flags = flags2;
+		}
 		if (staff->flags & OPEN_BRACKET) {
 			flags &= ~CLOSE_BRACKET;
-			flags |= OPEN_BRACKET | OPEN_BRACE;
+			flags |= OPEN_BRACKET;
 		} else if (staff->flags & CLOSE_BRACKET) {
-			flags &= ~(OPEN_BRACKET | OPEN_BRACE);
+			flags &= ~OPEN_BRACKET;
 			flags |= CLOSE_BRACKET;
 		} else if (staff->flags & OPEN_BRACE) {
 			flags &= ~CLOSE_BRACE;
@@ -1158,9 +1209,11 @@ static void parse_staves(unsigned char *p,
 			flags &= ~(OPEN_BRACKET | OPEN_BRACE);
 			flags |= CLOSE_BRACE;
 		}
-		if (staff->flags & OPEN_PARENTH) {
+		if (staff->flags & OPEN_PARENTH
+		    && !(staff->flags & CLOSE_PARENTH)) {
 			flags2 = flags;
 			flags &= ~CLOSE_PARENTH;
+			flags |= OPEN_PARENTH;
 		}
 	}
 	return;
@@ -1179,7 +1232,6 @@ char *get_str(unsigned char *d,		/* destination */
 	maxlen--;		/* have place for the EOS */
 	while (isspace(*s))
 		s++;
-
 	if (*s == '"') {
 		sep = '"';
 		s++;
@@ -1192,11 +1244,9 @@ char *get_str(unsigned char *d,		/* destination */
 			break;
 		}
 		if (c == '\\'
-		   && (c == sep
-		       || (c == '\t' && sep == ' '))) {
-			s++;
-			continue;
-		}
+		    && (s[1] == sep
+		       || (s[1] == '\t' && sep == ' ')))
+			c = *++s;
 		if (--maxlen > 0)
 			*d++ = c;
 		s++;
@@ -1343,6 +1393,7 @@ static struct kw_s {
 	{"stem=", 5, 5},
 	{"gstem=", 6, 6},
 	{"auto", 4, 7},
+	{"dyn=", 4, 8},
 	{0, 0, 0}
 };
 	struct kw_s *kw;
@@ -1427,6 +1478,9 @@ static struct kw_s {
 		case 7:			/* auto */
 			*p_stem = 2;
 			break;
+		case 8:			/* dyn= */
+			p_stem = &s->u.voice.dyn;
+			break;
 		}
 	}
 	if (clef_name != 0 || clef_middle != 0) {
@@ -1479,9 +1533,9 @@ static char *parse_bar(struct abctune *t,
 		bar_type = (B_OBRA << 4) + B_CBRA;		/* [] */
 
 /*	curvoice->last_note = 0; */
-	if (vover_bar) {
+	if (vover > 0) {
 		curvoice = &voice_tb[curvoice->mvoice];
-		vover_bar = 0;
+		vover = 0;
 	}
 	s = abc_new(t, gchord, 0);
 	if (gchord) {
@@ -1495,7 +1549,7 @@ static char *parse_bar(struct abctune *t,
 
 	if (dc.n > 0) {
 		memcpy(&s->u.bar.dc, &dc, sizeof s->u.bar.dc);
-		dc.n = 0;
+		dc.n = dc.h = dc.s = 0;
 	}
 	if (!isdigit(*p)			/* if not a repeat bar */
 	    && (*p != '"' || p[-1] != '['))	/* ('["' only) */
@@ -1604,11 +1658,11 @@ static unsigned char *parse_basic_note(unsigned char *p,
 		p_n = strchr(all_notes, *p);
 		if (p_n == 0
 		    || p_n - all_notes >= 14) {
-			if (acc)
-				syntax("Missing note after accidental",
-				       p);
-			else	syntax("Not a note", p);
+			syntax(acc ? "Missing note after accidental"
+				   : "Not a note", p);
 			pit = 16 + 7;	/* 'c' */
+			if (*p == '\0')
+				p--;
 		} else	pit = p_n - all_notes + 16;
 		p++;
 	}
@@ -1639,7 +1693,7 @@ static unsigned char *parse_basic_note(unsigned char *p,
 	return p;
 }
 
-/* -- parse for decoration on note/bar -- */
+/* -- parse the decorations of notes and bars -- */
 unsigned char *parse_deco(unsigned char *p,
 			  struct deco *deco)
 {
@@ -1652,13 +1706,14 @@ unsigned char *parse_deco(unsigned char *p,
 		if (char_tb[c] != CHAR_DECO && char_tb[c] != CHAR_DECOS)
 			break;
 
-		if (char_tb[c] == CHAR_DECOS)
+		if (char_tb[c] == CHAR_DECOS) {
 			p = get_deco(p, &d);
-		else	d = c;
+			c = d;
+		}
 		if (n >= MAXDC)
 			syntax("Too many decorations for the note", p);
-		else if (d != 0)
-			deco->t[n++] = d;
+		else if (c != 0)
+			deco->t[n++] = c;
 	}
 	deco->n = n;
 	return p - 1;
@@ -1687,7 +1742,7 @@ static char *parse_decoline(unsigned char *p)
 					   || is->u.bar.type == B_OBRA))
 				is = is->next;
 			if (is == 0) {
-				syntax("Not enough bar lines for lyric line", p);
+				syntax("Not enough bar lines for deco line", p);
 				return 0;
 			}
 			is = is->next;
@@ -1697,7 +1752,7 @@ static char *parse_decoline(unsigned char *p)
 			while (is != 0 && is->type != ABC_T_NOTE)
 				is = is->next;
 			if (is == 0) {
-				syntax("Not enough notes for decoration line", p);
+				syntax("Not enough notes for deco line", p);
 				return 0;
 			}
 			is = is->next;
@@ -1706,7 +1761,7 @@ static char *parse_decoline(unsigned char *p)
 		case '\\':
 			if (p[1] == '\0') {
 				if (is == 0)
-					return "Not enough notes for decoration line";
+					return "Not enough notes for deco line";
 				deco_cont = is;
 				return 0;
 			}
@@ -1723,15 +1778,16 @@ static char *parse_decoline(unsigned char *p)
 		}
 
 		/* store the decoration in the next note */
-		while (is != 0 && is->type != ABC_T_NOTE)
+		while (is != 0 && (is->type != ABC_T_NOTE
+				   || is->u.note.grace))
 			is = is->next;
 		if (is == 0)
-			return "Not enough notes for decoration line";
+			return "Not enough notes for deco line";
 
 		n = is->u.note.dc.n;
 		if (n >= MAXDC)
 			syntax("Too many decorations for the note", p);
-		else {
+		else if (d != 0) {
 			is->u.note.dc.t[n] = d;
 			is->u.note.dc.n = n + 1;
 		}
@@ -1774,7 +1830,7 @@ static unsigned char *parse_len(unsigned char *p,
 }
 
 /* -- parse a music line -- */
-/* return 0 at end of tune */
+/* return 1 at end of tune */
 static int parse_line(struct abctune *t,
 		      unsigned char *p)
 {
@@ -1789,36 +1845,53 @@ static int parse_line(struct abctune *t,
 	colnum = 0;
 again:					/* for history */
 	switch (*p) {
-	case '\0':
+	case '\0':			/* blank line */
 		switch (abc_state) {
 		case ABC_S_GLOBAL:
-		case ABC_S_HEAD:	/*fixme: may have blank lines in headers*/
+		case ABC_S_HEAD:	/*fixme: may have blank lines in headers?*/
 			if (keep_comment) {
 				s = abc_new(t, 0, 0);
 				s->type = ABC_T_NULL;
 				s->state = abc_state;
 			}
-			return 1;
+			return 0;
 		}
 		abc_state = ABC_S_GLOBAL;
+		abc_vers = abc_vers_g;
 		if (level_f)
 			level_f(0);
-		return 0;
+		return 1;
 	case '%':
 		if (p[1] == '%') {
+			char begintype;
+
 			comment = decomment_line(p + 2);
 			s = abc_new(t, p, comment);
 			s->type = ABC_T_PSCOM;
 			s->state = abc_state;
 			p += 2;				/* skip '%%' */
-			if (strncasecmp(p, "fmt ", 4) == 0)
+			while (isspace(*p))
+				p++;
+			if (strncasecmp(p, "abc-version ", 12) == 0) {
+				abc_vers = atoi(p + 12);
+				return 0;
+			}
+			if (strncasecmp(p, "fmt ", 4) == 0) {
 				p += 4;			/* skip 'fmt' */
-			if (strncmp(p, "begintext", 9) == 0) {
+				while (isspace(*p))
+					p++;
+			}
+			begintype = '\0';
+			if (strncmp(p, "beginps", 7) == 0)
+				begintype = 'p';
+			else if (strncmp(p, "begintext", 9) == 0)
+				begintype = 't';
+			if (begintype != '\0') {
 				for (;;) {
 					if ((p = get_line()) == 0) {
-						syntax("EOF while parsing %%begintext pseudo-comment",
+						syntax("EOF while parsing %%beginxxx pseudo-comment",
 						       scratch_line);
-						return 0;
+						return 1;
 					}
 					s = abc_new(t, p, 0);
 					s->type = ABC_T_PSCOM;
@@ -1826,34 +1899,45 @@ again:					/* for history */
 					if (*p != '%' || p[1] != '%')
 						continue;
 					p += 2;
-					if (strncasecmp(p, "fmt ", 4) == 0)
+					while (isspace(*p))
+						p++;
+					if (strncasecmp(p, "fmt ", 4) == 0) {
 						p += 4;
-					if (strncmp(p, "endtext", 7) == 0)
-						return 1;
+						while (isspace(*p))
+							p++;
+					}
+					if (begintype == 'p') {
+						if (strncmp(p, "endps", 5) == 0)
+							return 0;
+					} else {
+						if (strncmp(p, "endtext", 7) == 0)
+							return 0;
+					}
 				}
 				/* not reached */
 			}
 			if (strncmp(p, "staves ", 7) == 0)
 				parse_staves(p + 7, s);
-			return 1;
+			return 0;
 		}
+		/* fall thru */
 	case '\\':			/* abc2mtex specific lines */
 		if (keep_comment) {
 			s = abc_new(t, p, 0);
 			s->type = ABC_T_NULL;
 			s->state = abc_state;
 		}
-		return 1;		/* skip */
+		return 0;		/* skip */
 	}
 	comment = decomment_line(p);
 
 	/* header fields */
 	if (p[1] == ':'
 	    && *p != '|' && *p != ':') {	/* not '|:' nor '::' */
-		parse_header(t, p, comment);
+		parse_info(t, p, comment);
 		if (*p == 'H') {
 
-			/* wait for an other 'x:' or any '%%' */
+			/* loop till any 'x:' or '%%' */
 			for (;;) {
 				if ((p = get_line()) == 0)
 					break;
@@ -1872,10 +1956,10 @@ again:					/* for history */
 		/* 'V:n <note line ending with a bar>' */
 		if (*p != 'V'
 		    || abc_state != ABC_S_TUNE)
-			return 1;
+			return 0;
 		c = p[strlen(p) - 1];
 		if (c != '|' && c != ']')
-			return 1;
+			return 0;
 		while (!isspace(*p) && *p != '\0')
 			p++;
 		while (isspace(*p))
@@ -1887,10 +1971,10 @@ again:					/* for history */
 			s->type = ABC_T_NULL;
 			s->state = abc_state;
 		}
-		return 1;
+		return 0;
 	}
 
-	if (scratch_line[0] == ' ' && curvoice->last_note != 0)
+	if (isspace(scratch_line[0]) && curvoice->last_note != 0)
 		curvoice->last_note->u.note.word_end = 1;
 
 	lyric_started = 0;
@@ -1917,11 +2001,11 @@ again:					/* for history */
 				int l2;
 				char *gch;
 
-				/* many guitar chord: concatenate with '\n' */
+				/* many guitar chord: concatenate with '\n' or ';' */
 				l2 = strlen(gchord);
 				gch = alloc_f(l2 + 1 + l + 1);
 				strcpy(gch, gchord);
-				gch[l2++] = '\n';
+				gch[l2++] = more_gch ? ';' : '\n';
 				strncpy(&gch[l2], q, l);
 				gch[l2 + l] = '\0';
 				if (free_f)
@@ -1938,7 +2022,7 @@ again:					/* for history */
 				if ((p = get_line()) == 0) {
 					syntax("EOF reached while parsing guitar chord",
 					       q);
-					return 0;
+					return 1;
 				}
 				goto gch_continue;
 			}
@@ -1988,6 +2072,7 @@ again:					/* for history */
 					break;
 			}
 			p = parse_deco(p - 1, &dc);
+			dc.h = dc.s = dc.n;
 			break;
 		case CHAR_ACC:
 		case CHAR_NOTE:
@@ -2005,6 +2090,7 @@ again:					/* for history */
 				q++;
 			if (char_tb[*q] != CHAR_BAR)
 				goto bad_char;
+			syntax("Obsolete measure repeat syntax", p - 1);
 			s = abc_new(t, 0, 0);
 			s->type = ABC_T_MREP;
 			s->state = ABC_S_TUNE;
@@ -2015,7 +2101,7 @@ again:					/* for history */
 		case CHAR_BSLASH:		/* '\\' */
 /*fixme: KO if in grace note sequence*/
 			if (*p == '\0')
-				return 1;
+				return 0;
 			syntax("'\\' ignored", p - 1);
 			break;
 		case CHAR_OBRA:			/* '[' */
@@ -2044,7 +2130,7 @@ again:					/* for history */
 				c = '\0';
 			} else	*p = '\0';
 			abc_state = ABC_S_EMBED;
-			parse_header(t, q, 0);
+			parse_info(t, q, 0);
 			abc_state = ABC_S_TUNE;
 			*p = c;
 			if (c != '\0')
@@ -2100,15 +2186,16 @@ again:					/* for history */
 				break;
 			}
 			if (*p == '&') {
+				p++;
+				if (vover != 0) {
+					syntax("Nested voice overlay", p - 1);
+					break;
+				}
 				s = abc_new(t, 0, 0);
 				s->type = ABC_T_V_OVER;
-				p++;
-				if (*p == '&') {
-					s->u.v_over.type = V_OVER_SD;
-					p++;
-				} else	s->u.v_over.type = V_OVER_SS;
+				s->u.v_over.type = V_OVER_S;
 				s->u.v_over.voice = curvoice - voice_tb;
-				char_tb[')'] = CHAR_VOVE;
+				vover = -1;		/* multi-bars */
 				break;
 			}
 			curvoice->slur <<= 2;
@@ -2137,23 +2224,23 @@ again:					/* for history */
 			t->last_sym->u.note.slur_end++;
 			break;
 		case CHAR_VOV:			/* '&' */
-			s = abc_new(t, 0, 0);
-			s->type = ABC_T_V_OVER;
-			if (*p == '&') {
-				s->u.v_over.type = V_OVER_D;
-				p++;
-			} /*else s->u.v_over.type = V_OVER_S; */
-			vover_new();
-			s->u.v_over.voice = curvoice - voice_tb;
-			if (char_tb[')'] != CHAR_VOVE)
-				vover_bar = 1;
-			break;
-		case CHAR_VOVE:			/* ')' after '(&' */
+			if (*p != ')'
+			    || vover == 0) {		/*??*/
+				s = abc_new(t, 0, 0);
+				s->type = ABC_T_V_OVER;
+				/*s->u.v_over.type = V_OVER_N; */
+				vover_new();
+				s->u.v_over.voice = curvoice - voice_tb;
+				if (vover == 0)
+					vover = 1;	/* single bar */
+				break;
+			}
+			p++;
+			vover = 0;
 			s = abc_new(t, 0, 0);
 			s->type = ABC_T_V_OVER;
 			s->u.v_over.type = V_OVER_E;
 			s->u.v_over.voice = curvoice->mvoice;
-			char_tb[')'] = CHAR_CPAR;
 			curvoice->last_note = 0;	/* ?? */
 			curvoice = &voice_tb[curvoice->mvoice];
 			break;
@@ -2229,7 +2316,7 @@ again:					/* for history */
 	s->type = ABC_T_EOLN;
 	s->state = abc_state;
 
-	return 1;
+	return 0;
 }
 
 /* -- parse a note or a rest -- */
@@ -2238,26 +2325,27 @@ static unsigned char *parse_note(struct abctune *t,
 {
 	struct abcsym *s;
 	unsigned char *q;
-	int pit, len, acc, nostem;
-	int chord, j, m;
+	int pit, len, acc, nostem, chord, j, m;
 
 	if (char_tb['{'] == CHAR_BAD) {	/* in a grace note sequence */
 		s = abc_new(t, 0, 0);
 		s->u.note.grace = 1;
 	} else {
 		s = abc_new(t, gchord, 0);
-		if (gchord && free_f)
-			free_f(gchord);
-		gchord = 0;
+		if (gchord) {
+			if (free_f)
+				free_f(gchord);
+			gchord = 0;
+		}
 	}
 	s->type = ABC_T_NOTE;
 	s->state = ABC_S_TUNE;
 
-#if 0
-/*fixme: problem in ABC: have deco on rests in s: lines?*/
-	if (!s->u.note.grace && deco_start == 0)
-			deco_start = s;
-#endif
+	if (*p != 'Z' && !lyric_started && !s->u.note.grace) {
+		lyric_started = 1;
+		s->u.note.lyric_start = 1;
+		deco_start = s;
+	}
 
 	/* rest */
 	switch (*p) {
@@ -2291,15 +2379,9 @@ static unsigned char *parse_note(struct abctune *t,
 		goto do_brhythm;
 	}
 
-	if (!s->u.note.grace && !lyric_started) {
-		lyric_started = 1;
-		s->u.note.lyric_start = 1;
-		deco_start = s;
-	}
-
 	chord = 0;
 	q = p;
-	if (*p == '[') {	/* accept only '[..]' for chord */
+	if (*p == '[') {	/* '[..]' = chord */
 		chord = 1;
 		p++;
 	}
@@ -2341,10 +2423,12 @@ static unsigned char *parse_note(struct abctune *t,
 				tmp = dc.n - 7;
 			}
 			s->u.note.decs[m] = (tmp << 3) + dc.n - tmp;
+			dc.s = dc.n;
 		}
-		if (strchr(all_notes, *p) == 0) {
+		if (*p == '\0' || strchr(all_notes, *p) == 0) {
 			syntax("Not a note", p);
-			p++;
+			if (*p != '\0')
+				p++;
 		} else {
 			p = parse_basic_note(p, &pit, &len, &acc, &tmp);
 			if (s->u.note.grace) {
@@ -2432,18 +2516,18 @@ add_slurs:
 add_deco:
 	if (dc.n > 0) {
 		memcpy(&s->u.note.dc, &dc, sizeof s->u.note.dc);
-		dc.n = 0;
+		dc.n = dc.h = dc.s = 0;
 	}
 	return p;
 }
 
-/* -- parse a header -- */
-static void parse_header(struct abctune *t,
-			 unsigned char *p,
-			 char *comment)
+/* -- parse an information field -- */
+static void parse_info(struct abctune *t,
+		       unsigned char *p,
+		       char *comment)
 {
 	struct abcsym *s;
-	unsigned char header_type = *p;
+	unsigned char info_type = *p;
 	char *error_txt = 0;
 
 	s = abc_new(t, p, comment);
@@ -2453,9 +2537,11 @@ static void parse_header(struct abctune *t,
 	p += 2;
 	while (isspace(*p))
 		p++;
-	switch (header_type) {
+	switch (info_type) {
 	case 'd':
 	case 's':
+		if (abc_state == ABC_S_GLOBAL)
+			break;
 		if (deco_start == 0) {
 			error_txt = "Erroneous 'd:'/'s:'";
 			break;
@@ -2511,6 +2597,7 @@ static void parse_header(struct abctune *t,
 		nvoice = 0;
 		curvoice = &voice_tb[0];
 		abc_state = ABC_S_HEAD;
+		abc_vers_g = abc_vers;
 		if (level_f)
 			level_f(1);
 		break;
@@ -2528,7 +2615,8 @@ static void syntax(char *msg,
 
 	severity = 1;
 	n = q - scratch_line;
-	if ((unsigned) n >= line_length)
+	len = strlen(scratch_line);
+	if ((unsigned) n >= len)
 		n = -1;
 	print_error(msg, n);
 	if (n < 0) {
@@ -2537,7 +2625,7 @@ static void syntax(char *msg,
 		return;
 	}
 	m1 = 0;
-	m2 = len = line_length - 1;
+	m2 = --len;
 	if (m2 > maxcol) {
 		if (n < maxcol)
 			m2 = maxcol;
