@@ -5,8 +5,8 @@
 
 #define MAXVOICE 32	/* max number of voices */
 
-#define MAXHD	8	/* max heads on one stem */
-#define MAXDC	31	/* max decorations */
+#define MAXHD	8	/* max heads in a chord */
+#define MAXDC	45	/* max decorations per note/chord/bar */
 
 #define BASE_LEN 1536	/* basic note length (semibreve or whole note - same as MIDI) */
 
@@ -32,8 +32,10 @@ enum accidentals {
 #define SL_AUTO 0x03
 
 /* note structure */
-struct deco {			/* describes decorations */
-	char n;			/* number of decorations */
+struct deco {		/* decorations */
+	char n;			/* whole number of decorations */
+	char h;			/* start of head decorations */
+	char s;			/* start of decorations from s: (d:) */
 	unsigned char t[MAXDC];	/* decoration type */
 };
 
@@ -41,8 +43,8 @@ struct note {		/* note or rest */
 	signed char pits[MAXHD]; /* pitches */
 	short lens[MAXHD];	/* note lengths (# pts in [1] if space) */
 	unsigned char accs[MAXHD]; /* code for accidentals & index in micro_tb */
-	char sl1[MAXHD];	/* which slur starts on this head */
-	char sl2[MAXHD];	/* which slur ends on this head */
+	unsigned char sl1[MAXHD]; /* slur start per head */
+	char sl2[MAXHD];	/* number of slur end per head */
 	char ti1[MAXHD];	/* flag to start tie here */
 	unsigned char decs[MAXHD]; /* head decorations (index: 5 bits, len: 3 bits) */
 	unsigned invis:1;	/* invisible rest (x / y) */
@@ -95,7 +97,7 @@ struct abcsym {
 			signed char sf;		/* sharp (> 0) flats (< 0) */
 			char bagpipe;		/* HP or Hp */
 			char minor;		/* major (0) / minor (1) */
-			char empty;		/* clef alone if 1 */
+			char empty;		/* clef alone if 1, 'none' if 2 */
 			char nacc;		/* explicit accidentals */
 			char pits[8];
 			char accs[8];
@@ -126,6 +128,7 @@ struct abcsym {
 			char merge;		/* merge with previous voice */
 			signed char stem;	/* have stems up or down (2 = auto) */
 			signed char gstem;	/* have grace stems up or down (2 = auto) */
+			signed char dyn;	/* have dynamic marks above or below the staff */
 		} voice;
 		struct {		/* bar, mrest or mrep */
 			struct deco dc;		/* decorations */
@@ -167,11 +170,9 @@ struct abcsym {
 		} staves[MAXVOICE];
 		struct {		/* voice overlay */
 			char type;
-#define V_OVER_S 0				/* single & */
-#define V_OVER_D 1				/* && */
-#define V_OVER_SS 2				/* (& */
-#define V_OVER_SD 3				/* (&& */
-#define V_OVER_E 4				/* )& */
+#define V_OVER_V 0				/* & */
+#define V_OVER_S 1				/* (& */
+#define V_OVER_E 2				/* &) */
 			unsigned char voice;
 		} v_over;
 		struct {		/* tuplet */
