@@ -30,7 +30,7 @@
 #include "abcparse.h"
 #include "abc2ps.h" 
 
-#define BUFFLN	100		/* max number of lines in output buffer */
+#define BUFFLN	80		/* max number of lines in output buffer */
 
 static int ln_num;		/* number of lines in buffer */
 static float ln_pos[BUFFLN];	/* vertical positions of buffered lines */
@@ -50,14 +50,20 @@ char *mbf;			/* where to PUTx() */
 int nbuf;			/* number of bytes buffered */
 int use_buffer;			/* 1 if lines are being accumulated */
 
-/*  subroutines for postscript output  */
+/* -- convert a date -- */
+static void cnv_date(time_t *ltime)
+{
+	unsigned char buf[TEX_BUF_SZ];
+
+	tex_str(cfmt.dateformat);
+	strcpy(buf, tex_buf);
+	strftime(tex_buf, TEX_BUF_SZ, buf, localtime(ltime));
+}
 
 /* -- initialize the postscript file -- */
-static void init_ps(char *str,
-		    int is_epsf)
+static void init_ps(char *str, int is_epsf)
 {
 	time_t ltime;
-	char tstr[41];
 	int i;
 
 	if (is_epsf) {
@@ -74,14 +80,9 @@ static void init_ps(char *str,
 
 	/* CreationDate */
 	time(&ltime);
-	strcpy(tstr, ctime(&ltime));
-/* Wed Jun 30 21:49:08 1993\n */
-	tstr[10]='\0';
-	tstr[16]='\0';
-	tstr[24]='\0';
+	strftime(tex_buf, TEX_BUF_SZ, "%b %e, %Y %H:%M", localtime(&ltime));
 	fprintf(fout, "%%%%Creator: abcm2ps-" VERSION "\n"
-		"%%%%CreationDate: %s, %s %s\n",
-		&tstr[4], &tstr[20], &tstr[11]);
+		"%%%%CreationDate: %s\n", tex_buf);
 	if (!is_epsf)
 		fprintf(fout, "%%%%Pages: (atend)\n");
 	fprintf(fout, "%%%%LanguageLevel: %d\n"
@@ -108,47 +109,36 @@ static void init_ps(char *str,
 		"/RM/rmoveto load def\n"
 		"/RL/rlineto load def\n"
 		"/RC/rcurveto load def\n"
-		"/dlw{0.7 setlinewidth}!\n");
+		"/SLW/setlinewidth load def\n"
+		"/defl 0 def\n"	/* decoration flags - see deco.c for values */
+		"/dlw{0.7 SLW}!\n");
 	if (cfmt.pslevel < 2)
 		/* (simple!) level2 emulation */
 	    fprintf(fout, 
-		"/cshow{exch dup {fh fh 4 index exec} forall pop pop}!\n"
-		"/glyphshow{ currentfont /Encoding .knownget not { {} } if\n"
-		"   0 1 2 index length 1 sub {	%% Stack: glyph encoding index\n"
-		"      2 copy get 3 index eq {exch pop exch pop null exit} if\n"
+		"/cshow{exch dup{fh fh 4 index exec}forall pop pop}!\n"
+/*		"/glyphshow{currentfont/Encoding .knownget not{{}} if\n" */
+		"/glyphshow{currentfont/Encoding get\n"
+		"   0 1 2 index length 1 sub{\n"	/* stack: glyph encoding index */
+		"      2 copy get 3 index eq{exch pop exch pop null exit}if\n"
 		"      pop\n"
 		"    }\n"
-		"   for null eq {(X) dup 0 4 -1 roll put show} {pop} ifelse\n"
+		"   for null eq{(X) dup 0 4 -1 roll put show}{pop}ifelse\n"
 		"}!\n"
 		"/rectstroke{\n"
-		"	4 2 roll M 1 index 0 RL 0 1 index RL exch neg 0 RL neg 0 exch RL\n"
+		"	4 2 roll M 1 index 0 RL 0 exch RL neg 0 RL closepath\n"
 		"	stroke}!\n"
 		"/rectfill{\n"
-		"	4 2 roll M 1 index 0 RL 0 1 index RL exch neg 0 RL neg 0 exch RL\n"
+		"	4 2 roll M 1 index 0 RL 0 exch RL neg 0 RL closepath\n"
 		"	fill}!\n"
 		"/selectfont{exch findfont exch dup\n"
-		"	type /arraytype eq {makefont}{scalefont} ifelse setfont}!\n");
+		"	type/arraytype eq{makefont}{scalefont}ifelse setfont}!\n");
 	define_symbols();
-	fprintf(fout, "0 setlinecap 0 setlinejoin\n");
 	user_ps_write();
 	define_fonts();
 	fprintf(fout, "%%%%EndSetup\n");
 	file_initialized = 1;
 }
 
-/* -- close a page -- */
-static void close_page(void)
-{
-	if (!in_page)
-		return;
-	in_page = 0;
-	fprintf(fout, "%%%%PageTrailer\n"
-		"grestore\n"
-		"showpage\n");
-	cur_lmarg = 0;
-	cur_scale = 1.0;
-}
-
 /* -- close_output_file -- */
 void close_output_file(void)
 {
@@ -166,22 +156,35 @@ void close_output_file(void)
 		m = ftell(fout);
 		fclose(fout);
 		fprintf(stderr,
-			"Output written on %s (%d page%s, %d title%s, %ld byte%s)\n",
+			"Output written on %s (%d page%s, %d title%s, %ld bytes)\n",
 			outfnam,
 			nbpages, nbpages == 1 ? "" : "s",
 			tunenum, tunenum == 1 ? "" : "s",
-			m, m == 1 ? "" : "s");
+			m);
 	}
 	fout = 0;
 	file_initialized = 0;
 	nbpages = tunenum = 0;
 }
 
+/* -- close the PS page -- */
+void close_page(void)
+{
+	if (!in_page)
+		return;
+	in_page = 0;
+	fprintf(fout, "%%%%PageTrailer\n"
+		"grestore\n"
+		"showpage\n");
+	cur_lmarg = 0;
+	cur_scale = 1.0;
+	defl = -1;
+}
+
 /* -- output a header/footer element -- */
 static void format_hf(char *p)
 {
 	char *q;
-	char t[512];
 	time_t ltime;
 
 	for (;;) {
@@ -194,14 +197,13 @@ static void format_hf(char *p)
 			break;
 		p = q + 1;
 		switch (*p) {
+		case 'd':
+			ltime = mtime;
+			goto dput;
 		case 'D':
 			time(&ltime);
-			strcpy(t, ctime(&ltime));
-/* Wed Jun 30 21:49:08 1993\n */
-			t[10]='\0';
-			t[16]='\0';
-			t[24]='\0';
-			fprintf(fout, "%s, %s %s", &t[4], &t[20], &t[11]);
+		dput:	cnv_date(&ltime);
+			fprintf(fout, "%s", tex_buf);
 			break;
 		case 'F':		/* ABC file name */
 #if DIRSEP!='\\'
@@ -212,8 +214,8 @@ static void format_hf(char *p)
 				char *r;
 
 				q = in_fname;
-				r = t;
-				i = sizeof t;
+				r = tex_buf;
+				i = TEX_BUF_SZ;
 				for (;;) {
 					if (--i <= 0 || *q == '\0')
 						break;
@@ -224,7 +226,7 @@ static void format_hf(char *p)
 				}
 				*r = '\0';
 			}
-			fprintf(fout, "%s", t);
+			fprintf(fout, "%s", tex_buf);
 #endif
 			break;
 		case 'P':		/* page number */
@@ -243,8 +245,8 @@ static void format_hf(char *p)
 			q = &info.title->as.text[2];
 			while (isspace(*q))
 				q++;
-			tex_str(t, q, sizeof t);
-			fprintf(fout, "%s", t);
+			tex_str(q);
+			fprintf(fout, "%s", tex_buf);
 			break;
 		case 'V':
 			fprintf(fout,"abcm2ps-"  VERSION);
@@ -261,28 +263,30 @@ static float headfooter(int header,
 			float pwidth,
 			float pheight)
 {
-	char str[256];
+	char str[512];
 	char *p, *q, *r;
 	float size, y, wsize;
-	int fnum;
+	int fnum, fi_sav;
+	struct FONTSPEC *f, f_sav;
+
+	memcpy(&f_sav, &cfmt.font_tb[0], sizeof f_sav);
+	fi_sav = strcf;
 
 	if (header) {
 		p = cfmt.header;
-/*		str_font(&cfmt.headerfont); */
-		size = cfmt.headerfont.size;
-		fnum = cfmt.headerfont.fnum;
+		f = &cfmt.headerfont;
+		size = f->size;
 		y = 2.;
-		wsize = 0;
 	} else {
 		p = cfmt.footer;
-/*		str_font(&cfmt.footerfont); */
-		size = cfmt.footerfont.size;
-		fnum = cfmt.footerfont.fnum;
+		f = &cfmt.footerfont;
+		size = f->size;
 		y = - (pheight - cfmt.topmargin - cfmt.botmargin)
 			- size + 2.;
-		wsize = 0;
 	}
-
+	wsize = 0;
+	str_font(f);
+	fnum = f->fnum;
 	fprintf(fout, "%.1f F%d ", size, fnum);
 
 	/* may have 2 lines */
@@ -294,17 +298,18 @@ static float headfooter(int header,
 	}
 
 	for (;;) {
-		tex_str(str, p, sizeof str);
+		tex_str(p);
+		strcpy(str, tex_buf);
 
 		/* left side */
 		p = str;
 		if ((q = strchr(p, '\t')) != 0) {
 			if (q != p) {
 				*q = '\0';
-				fprintf(fout, "%.1f %.1f M (",
+				fprintf(fout, "%.1f %.1f M(",
 					cfmt.leftmargin, y);
 				format_hf(p);
-				fprintf(fout, ") show\n");
+				fprintf(fout, ")show\n");
 			}
 			p = q + 1;
 		}
@@ -313,20 +318,20 @@ static float headfooter(int header,
 
 		/* center */
 		if (q != p) {
-			fprintf(fout, "%.1f %.1f M (",
+			fprintf(fout, "%.1f %.1f M(",
 				pwidth * 0.5, y);
 			format_hf(p);
-			fprintf(fout, ") showc\n");
+			fprintf(fout, ")showc\n");
 		}
 
 		/* right side */
 		if (q != 0) {
 			p = q + 1;
 			if (*p != '\0') {
-				fprintf(fout, "%.1f %.1f M (",
+				fprintf(fout, "%.1f %.1f M(",
 					pwidth - cfmt.rightmargin, y);
 				format_hf(p);
-				fprintf(fout, ") lshow\n");
+				fprintf(fout, ")showr\n");
 			}
 		}
 		if (r == 0)
@@ -337,6 +342,9 @@ static float headfooter(int header,
 		y -= size;
 	}
 
+	memcpy(&cfmt.font_tb[0], &f_sav, sizeof cfmt.font_tb[0]);
+	strcf = fi_sav;
+
 	return wsize;
 }
 
@@ -371,13 +379,17 @@ static void init_page(void)
 	posy = pheight - cfmt.topmargin - cfmt.botmargin;
 
 	/* output the header and footer */
-	if (cfmt.header == 0 && pagenumbers != 0) {
+	if (cfmt.header == 0) {
+		char *p = 0;
+
 		switch (pagenumbers) {
-		case 1: cfmt.header = "$P\t"; break;
-		case 2: cfmt.header = "\t\t$P"; break;
-		case 3: cfmt.header = "$P0\t\t$P1"; break;
-		case 4: cfmt.header = "$P1\t\t$P0"; break;
+		case 1: p = "$P\t"; break;
+		case 2: p = "\t\t$P"; break;
+		case 3: p = "$P0\t\t$P1"; break;
+		case 4: p = "$P1\t\t$P0"; break;
 		}
+		if (p != 0)
+			cfmt.header = strdup(p);
 	}
 	if (cfmt.header != 0) {
 		float dy;
@@ -391,6 +403,8 @@ static void init_page(void)
 	if (cfmt.footer != 0)
 		posy -= headfooter(0, pwidth, pheight);
 	pagenum++;
+	if (font_init != 0)
+		fprintf(fout,"%.1f F%d ", font_init->size, font_init->fnum);
 }
 
 /* -- open the output file -- */
@@ -502,12 +516,10 @@ void write_eps(void)
 }
 
 /* -- start a new page -- */
-void write_pagebreak()
+static void newpage(void)
 {
 	close_page();
 	init_page();
-	if (font_init != 0)
-		fprintf(fout,"%.1f F%d ", font_init->size, font_init->fnum);
 }
 
 /*  subroutines to handle output buffer  */
@@ -522,13 +534,11 @@ void a2b(void)
 		init_page();
 
 	l = strlen(mbf);
-
 	nbuf += l;
 	if (nbuf >= BUFFSZ - 500) {	/* must have place for 1 more line */
 		error(1, 0, "a2b: buffer full, BUFFSZ=%d", BUFFSZ);
 		exit(3);
 	}
-
 	mbf += l;
 }
 
@@ -542,15 +552,15 @@ void abskip(float h)
 /* -- translate down by 'h' scaled points in output buffer -- */
 void bskip(float h)
 {
-	PUT1("0 %.2f T\n", -h);
 	bposy -= h * cfmt.scale;
+	PUT1("0 %.2f T\n", -h);
 }
 
 /* -- clear_buffer -- */
 void clear_buffer(void)
 {
 	nbuf = 0;
-	bposy = 0.0;
+	bposy = 0;
 	ln_num = 0;
 	mbf = buf;
 }
@@ -563,7 +573,6 @@ void write_buffer(void)
 
 	if (nbuf == 0)
 		return;
-
 	i = 0;
 	p1 = 0;
 	for (l = 0; l < ln_num; l++) {
@@ -571,7 +580,7 @@ void write_buffer(void)
 		dp = ln_pos[l] - p1;
 		np = posy + dp < 0 && !epsf;
 		if (np)
-			write_pagebreak();
+			newpage();
 		if (ln_scale[l] != cur_scale) {
 			fprintf(fout, "%.2f dup scale\n",
 				ln_scale[l] / cur_scale);
@@ -591,9 +600,7 @@ void write_buffer(void)
 		posy += dp;
 		p1 = ln_pos[l];
 	}
-
 	fwrite(&buf[i], 1, nbuf - i, fout);
-
 	clear_buffer();
 }
 
@@ -602,37 +609,35 @@ void write_buffer(void)
    after page break and change buffer handling mode to pass though */
 void buffer_eob(void)
 {
-	if (ln_num >= BUFFLN) {
-		error(1, 0, "max number of buffer lines exceeded"
-			" -- check BUFFLN");
-		exit(3);
-	}
-
+	if (bposy == 0 && nbuf == 0)
+		return;
 	ln_buf[ln_num] = nbuf;
 	ln_pos[ln_num] = bposy;
 	ln_lmarg[ln_num] = cfmt.leftmargin;
 	ln_scale[ln_num] = cfmt.scale;
 	ln_num++;
-
 	if (!use_buffer) {
 		write_buffer();
 		return;
 	}
-
-	if ((posy + bposy < 0
-	     || cfmt.oneperpage)
+	if (ln_num >= BUFFLN && posy + bposy >= 0) {
+		error(1, 0, "max number of buffer lines exceeded"
+			" -- check BUFFLN");
+		bposy = -posy - 1;	/* force write_buffer() */
+	}
+	if ((posy + bposy < 0 || cfmt.oneperpage)
 	    && !epsf) {
 		if (tunenum != 1)
-			write_pagebreak();
+			newpage();
 		write_buffer();
 		use_buffer = 0;
 	}
 }
 
-/* -- dump buffer if less than nb bytes available -- */
+/* -- dump buffer if not enough place for a music line -- */
 void check_buffer(void)
 {
-	if (nbuf + BUFFSZ1 > BUFFSZ) {
+	if (nbuf > BUFFSZ - 5000) {	/* assume music line < 5000 bytes */
 		error(0, 0,
 		      "Possibly bad page breaks, BUFFSZ exceeded");
 		write_buffer();
@@ -645,27 +650,3 @@ float get_bposy(void)
 {
 	return posy + bposy;
 }
-
-/* -- set value in the output buffer -- */
-/* The values are flagged as "\x01vnnn.nn" where
- * - 'v' is the variable index ('0'..'z')
- * - 'nnn.nn' is the value to add to the variable
- * The variables are:
- * '0'..'O': staff offsets */
-/* this function must be called before buffer_eob() */
-void set_buffer(float *p_v)
-{
-	char *p;
-	int i;
-	float v;
-
-	if (ln_num == 0)
-		p = buf;
-	else	p = &buf[ln_buf[ln_num - 1]];
-	while ((p = strchr(p, '\x01')) != 0) {
-		i = p[1] - '0';
-		sscanf(p + 2, "%f", &v);
-		p += sprintf(p, "%7.1f", p_v[i] + v);
-		*p = ' ';
-	}
-}
