@@ -24,6 +24,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <time.h>
 #include <ctype.h>
 #include <math.h>
 
@@ -62,6 +63,8 @@ static struct format {
 	{"composerspace", FORMAT_U, 0, &cfmt.composerspace},
 	{"contbarnb", FORMAT_B, 0, &cfmt.contbarnb},
 	{"continueall", FORMAT_B, 0, &cfmt.continueall},
+	{"dateformat", FORMAT_S, 0, &cfmt.dateformat},
+	{"dynalign", FORMAT_B, 0, &cfmt.dynalign},
 	{"encoding", FORMAT_I, 1, &cfmt.encoding},
 	{"exprabove", FORMAT_B, 0, &cfmt.exprabove},
 	{"exprbelow", FORMAT_B, 0, &cfmt.exprbelow},
@@ -75,6 +78,7 @@ static struct format {
 	{"halfbeam", FORMAT_B, 0, &cfmt.halfbeam},
 	{"header", FORMAT_S, 0, &cfmt.header},
 	{"headerfont", FORMAT_F, 0, &cfmt.headerfont},
+	{"hyphencont", FORMAT_B, 0, &cfmt.hyphencont},
 	{"indent", FORMAT_U, 0, &cfmt.indent},
 	{"infofont", FORMAT_F, 0, &cfmt.infofont},
 	{"infoline", FORMAT_B, 0, &cfmt.infoline},
@@ -109,12 +113,16 @@ static struct format {
 	{"setfont-2", FORMAT_F, 0, &cfmt.font_tb[2]},
 	{"setfont-3", FORMAT_F, 0, &cfmt.font_tb[3]},
 	{"setfont-4", FORMAT_F, 0, &cfmt.font_tb[4]},
+#if DFONT_MIN!= 5
+#	error Bad start of dynamic fonts
+#endif
 	{"shifthnote", FORMAT_B, 0, &cfmt.shifthnote},
 	{"slurheight", FORMAT_R, 0, &cfmt.slurheight},
 	{"splittune", FORMAT_B, 0, &cfmt.splittune},
 	{"squarebreve", FORMAT_B, 0, &cfmt.squarebreve},
 	{"staffsep", FORMAT_U, 0, &cfmt.staffsep},
 	{"staffwidth", FORMAT_U, 1, &staffwidth},
+	{"stemheight", FORMAT_R, 0, &cfmt.stemheight},
 	{"straightflags", FORMAT_B, 0, &cfmt.straightflags},
 	{"stretchlast", FORMAT_B, 0, &cfmt.stretchlast},
 	{"stretchstaff", FORMAT_B, 0, &cfmt.stretchstaff},
@@ -123,6 +131,7 @@ static struct format {
 	{"sysstaffsep", FORMAT_U, 0, &cfmt.sysstaffsep},
 	{"tempofont", FORMAT_F, 0, &cfmt.tempofont},
 	{"textfont", FORMAT_F, 0, &cfmt.textfont},
+	{"textoption", FORMAT_I, 4, &cfmt.textoption},
 	{"textspace", FORMAT_U, 0, &cfmt.textspace},
 	{"titlecaps", FORMAT_B, 0, &cfmt.titlecaps},
 	{"titlefont", FORMAT_F, 0, &cfmt.titlefont},
@@ -174,13 +183,31 @@ static int add_font(char *fname, int encoding)
 		error(1, 0,
 		      "Cannot have a new font when the output file is opened");
 	fnum = nfontnames++;
-	fontnames[fnum] = getarena(strlen(fname) + 1);
-	strcpy(fontnames[fnum], fname);
+	fontnames[fnum] = strdup(fname);
 	font_enc[fnum] = encoding;
 	return fnum;
 }
 
-/* -- fontspec -- */
+/* -- set a dynamic font -- */
+static int dfont_set(struct FONTSPEC *f)
+{
+	int i;
+
+	for (i = DFONT_MIN; i < cfmt.ndfont; i++) {
+		if (cfmt.font_tb[i].fnum == f->fnum
+		    && cfmt.font_tb[i].size == f->size)
+			return i;
+	}
+	if (i >= DFONT_MAX - 1) {
+		error(1, 0, "Too many dynamic fonts");
+		return DFONT_MAX - 1;
+	}
+	memcpy(&cfmt.font_tb[i], f, sizeof cfmt.font_tb[0]);
+	cfmt.ndfont = i + 1;
+	return i;
+}
+
+/* -- define a font -- */
 static void fontspec(struct FONTSPEC *f,
 		     char *name,
 		     int encoding,
@@ -190,14 +217,24 @@ static void fontspec(struct FONTSPEC *f,
 		f->fnum = add_font(name, encoding);
 	else	name = fontnames[f->fnum];
 	f->size = size;
-	f->swfac = 1.0;
-	if (strcmp(name, "Times-Bold") == 0)
-		f->swfac = 1.05;
-	else if (strcmp(name, "Helvetica-Bold") == 0)
-		f->swfac = 1.15;
+	f->swfac = size;
+	if (strncmp(name, "Times", 5) == 0) {
+		if (strcmp(name, "Times-Bold") == 0)
+			f->swfac *= 1.05;
+	} else if (strcmp(name, "Helvetica-Bold") == 0)
+		f->swfac *= 1.15;
 	else if (strncmp(name, "Helvetica", 9) == 0
 		 || strncmp(name, "Palatino", 8) == 0)
-		f->swfac = 1.10;
+		f->swfac *= 1.10;
+	else if (strncmp(name, "Courier", 7) == 0)
+		f->swfac *= 1.35;
+	else	f->swfac *= 1.2;		/* unknown font */
+	if (f == &cfmt.gchordfont)
+		cfmt.gcf = dfont_set(f);
+	else if (f == &cfmt.annotationfont)
+		cfmt.anf = dfont_set(f);
+	else if (f == &cfmt.vocalfont)
+		cfmt.vof = dfont_set(f);
 }
 
 /* -- output the font definitions with their encodings -- */
@@ -254,40 +291,45 @@ void set_format(void)
 
 	f = &cfmt;
 	memset(f, 0, sizeof *f);
-	f->pageheight	= PAGEHEIGHT;
-	f->pagewidth	= PAGEWIDTH;
-	f->leftmargin	= MARGIN;
-	f->rightmargin	= MARGIN;
-	f->topmargin	= 1.0 * CM;
-	f->botmargin	= 1.0 * CM;
-	f->topspace	= 0.8 * CM;
-	f->titlespace 	= 0.2 * CM;
+	f->ndfont = DFONT_MIN;
+	f->pageheight = PAGEHEIGHT;
+	f->pagewidth = PAGEWIDTH;
+	f->leftmargin = MARGIN;
+	f->rightmargin = MARGIN;
+	f->topmargin = 1.0 * CM;
+	f->botmargin = 1.0 * CM;
+	f->topspace = 0.8 * CM;
+	f->titlespace = 0.2 * CM;
 	f->subtitlespace = 0.1 * CM;
 	f->composerspace = 0.2 * CM;
-	f->musicspace	= 0.2 * CM;
-	f->partsspace	= 0.3 * CM;
-	f->staffsep	= 46.0 * PT;
-	f->sysstaffsep	= 34.0 * PT;
-	f->maxstaffsep	= 800.0 * PT;
+	f->musicspace = 0.2 * CM;
+	f->partsspace = 0.3 * CM;
+	f->staffsep = 46.0 * PT;
+	f->sysstaffsep = 34.0 * PT;
+	f->maxstaffsep = 800.0 * PT;
 	f->maxsysstaffsep = 800.0 * PT;
-	f->vocalspace	= 23.0 * PT;
-	f->textspace	= 0.5 * CM;
-	f->scale	= 0.75;
-	f->slurheight	= 1.0;
-	f->maxshrink	= 0.65;
-	f->stretchstaff	= 1;
-	f->graceslurs	= 1;
-	f->halfbeam	= 1;
-	f->lineskipfac	= 1.1;
-	f->parskipfac	= 0.4;
+	f->vocalspace = 23.0 * PT;
+	f->textspace = 0.5 * CM;
+	f->scale = 0.75;
+	f->slurheight = 1.0;
+	f->maxshrink = 0.65;
+	f->stretchstaff = 1;
+	f->graceslurs = 1;
+	f->halfbeam = 1;
+	f->lineskipfac = 1.1;
+	f->parskipfac = 0.4;
 	f->measurenb = -1;
 	f->measurefirst = 1;
 	f->printparts = 1;
 	f->printtempo = 1;
 	f->autoclef = 1;
 	f->aligncomposer = A_RIGHT;
+	f->dynalign = 1;
 	f->notespacingfactor = 1.414;
 	f->pslevel = 2;
+	f->stemheight = STEM;
+	f->dateformat = strdup("\\%b \\%e, \\%Y \\%H:\\%M");
+	f->textoption = OBEYLINES;
 	fontspec(&f->titlefont, "Times-Roman", 0, 20.0);
 	fontspec(&f->subtitlefont, "Times-Roman", 0, 16.0);
 	fontspec(&f->composerfont, "Times-Italic", 0, 14.0);
@@ -363,9 +405,7 @@ static char yn[2][5]={"no","yes"};
 			printf("%s\n", yn[*((int *) fd->v)]);
 			break;
 		case FORMAT_S:
-			if ((char *) fd->v != 0)
-				printf("\"%s\"\n", (char *) fd->v);
-			else	printf("\"\"\n");
+			printf("\"%s\"\n", *((char **) fd->v) != 0 ? *((char **) fd->v) : "");
 			break;
 		}
 	}
@@ -401,6 +441,29 @@ static int get_encoding(char *p)
 	return e - enc_name;
 }
 
+/* -- get the option for text -- */
+int get_textopt(unsigned char *p)
+{
+	int option;
+
+	option = OBEYLINES;
+	if (*p == '\0'
+	    || strncmp(p, "obeylines", 9) == 0)
+		;
+	else if (strncmp(p, "align", 5) == 0
+		 || strncmp(p, "justify", 7) == 0)
+		option = T_JUSTIFY;
+	else if (strncmp(p, "ragged", 6) == 0
+		 || strncmp(p, "fill", 4) == 0)
+		option = T_FILL;
+	else if (strncmp(p, "center", 10) == 0)
+		option = OBEYCENTER;
+	else if (strncmp(p, "skip", 4) == 0)
+		option = SKIP;
+	else	option = -1;
+	return option;
+}
+
 /* -- read a boolean value -- */
 static int g_logv(char *p)
 {
@@ -461,148 +524,17 @@ static void g_fspc(char *p,
 
 /* -- parse a format line -- */
 /* return:
- *	0: format modified
- *	1: 'end' found
- *	2: format not modified */
+ *	0: command line format may be modified
+ *	1: format not modified */
 int interpret_format_line(char *w,		/* keyword */
 			  char *p)		/* argument */
 {
 	struct format *fd;
 
-	if (*w == '\0'
-	    || *w == '%')
-		return 2;
-	if (strcmp(w, "end") == 0)
-		return 1;
-
 	if (strcmp(w, "deco") == 0) {
 		deco_add(p);
-		return 2;
+		return 1;
 	}
-
-	if (strcmp(w, "postscript") == 0) {
-		if (!file_initialized)
-			user_ps_add(p);
-		return 2;
-	}
-
-	for (fd = format_tb; fd->name; fd++)
-		if (strcmp(w, fd->name) == 0)
-			break;
-	if (fd->name) {
-		switch (fd->type) {
-		case FORMAT_I:
-			if (fd->subtype == 3) {
-				unsigned i1, i2, i3;
-
-				if (sscanf(p, "%d %d %d", &i1, &i2, &i3) != 3
-				    || i1 > 2 || i2 > 2 || i3 > 2) {
-					error(1, 0,
-					      "Bad 'tuplets' value '%s' - ignored",
-					      p);
-					break;
-				}
-				cfmt.tuplets = (i1 << 8) | (i2 << 4) | i3;
-				break;
-			}
-			if (fd->subtype == 1 && !isdigit(*p))	/* 'encoding' */
-				*((int *) fd->v) = get_encoding(p);
-			else	sscanf(p, "%d", (int *) fd->v);
-			switch (fd->subtype) {
-			case 1:
-				if (isdigit(*p)
-				    && (unsigned) cfmt.encoding > MAXENC) {
-					error(1, 0,
-					      "Bad encoding value %d - reset to 0",
-					      cfmt.encoding);
-					cfmt.encoding = 0;
-				} else {
-					int i;
-
-					for (i = 0; i < nfontnames; i++) {
-						if (font_enc[i] == 0)
-							font_enc[i] = cfmt.encoding;
-					}
-				}
-				break;
-			case 2:
-				nbar = nbar_rep = cfmt.measurefirst;
-				break;
-			}
-			break;
-		case FORMAT_R:
-			*((float *) fd->v) = g_fltv(p);
-			if (fd->subtype == 1) {	/* note spacing factor */
-				int i;
-				float w;
-
-				if (cfmt.notespacingfactor <= 0) {
-					fprintf(stderr,
-						"Bad value for 'notespacingfactor'\n");
-					break;
-				}
-				dot_space = sqrt(cfmt.notespacingfactor);
-				i = C_XFLAGS;		/* crotchet index */
-				w = space_tb[i];
-				for ( ; --i >= 0; ) {
-					w /= cfmt.notespacingfactor;
-					space_tb[i] = w;
-				}
-				i = C_XFLAGS;
-				w = space_tb[i];
-				for ( ; ++i < NFLAGS_SZ; ) {
-					w *= cfmt.notespacingfactor;
-					space_tb[i] = w;
-				}
-			}
-			break;
-		case FORMAT_F: {
-			int b;
-
-			g_fspc(p, (struct FONTSPEC *) fd->v);
-			b = strstr(p, "box") != 0;
-			switch (fd->subtype) {
-			case 1:
-				cfmt.partsbox = b;
-				break;
-			case 2:
-				cfmt.measurebox = b;
-				break;
-			case 3:
-				cfmt.gchordbox = b;
-				break;
-			}
-			break;
-		    }
-		case FORMAT_U:
-			*((float *) fd->v) = scan_u(p);
-			if (fd->subtype == 1) {
-				float rmargin;
-
-				rmargin = (cfmt.landscape ? cfmt.pageheight : cfmt.pagewidth)
-					- staffwidth - cfmt.leftmargin;
-				if (rmargin < 0)
-					fprintf(stderr, "'staffwidth' too big\n");
-				cfmt.rightmargin = rmargin;
-			}
-			break;
-		case FORMAT_B:
-			*((int *) fd->v) = g_logv(p);
-			break;
-		case FORMAT_S: {
-			int l;
-
-			l = strlen(p) + 1;
-			*((char **) fd->v) = getarena(l);
-			if (*p == '"')
-				get_str(*((char **) fd->v), p, l);
-			else	strcpy(*((char **) fd->v), p);
-			break;
-		    }
-		}
-		return 0;
-	}
-
 	if (!strcmp(w, "font")) {
 		int fnum, encoding;
 		char fname[80];
@@ -615,44 +547,205 @@ int interpret_format_line(char *w,		/* keyword */
 		used_font[fnum] = 1;
 		return 0;
 	}
-	return 2;
+	if (strcmp(w, "format") == 0) {
+		read_fmt_file(p);
+		return 1;
+	}
+	if (strcmp(w, "postscript") == 0) {
+		if (!file_initialized)
+			user_ps_add(p);
+		return 1;
+	}
+	for (fd = format_tb; fd->name; fd++)
+		if (strcmp(w, fd->name) == 0)
+			break;
+	if (fd->name == 0)
+		return 1;
+	switch (fd->type) {
+	case FORMAT_I:
+		if (fd->subtype == 3) {
+			unsigned i1, i2, i3;
+
+			if (sscanf(p, "%d %d %d", &i1, &i2, &i3) != 3
+			    || i1 > 2 || i2 > 2 || i3 > 2) {
+				error(1, 0,
+				      "Bad 'tuplets' value '%s' - ignored",
+				      p);
+				break;
+			}
+			cfmt.tuplets = (i1 << 8) | (i2 << 4) | i3;
+			break;
+		}
+		if (fd->subtype == 1 && !isdigit(*p))	/* 'encoding' */
+			cfmt.encoding = get_encoding(p);
+		else if (fd->subtype == 4 && !isdigit(*p)) /* 'textoption' */
+			cfmt.textoption = get_textopt(p);
+		else	sscanf(p, "%d", (int *) fd->v);
+		switch (fd->subtype) {
+		case 1:
+			if (isdigit(*p)
+			    && (unsigned) cfmt.encoding > MAXENC) {
+				error(1, 0,
+				      "Bad encoding value %d - reset to 0",
+				      cfmt.encoding);
+				cfmt.encoding = 0;
+			} else {
+				int i;
+
+				for (i = 0; i < nfontnames; i++) {
+					if (font_enc[i] == 0)
+						font_enc[i] = cfmt.encoding;
+				}
+			}
+			break;
+		case 2:
+			nbar = nbar_rep = cfmt.measurefirst;
+			break;
+		case 4:
+			if (cfmt.textoption < 0) {
+				error(1, 0,
+				      "Bad 'textoption' value '%s'",
+				      p);
+				cfmt.textoption = OBEYLINES;
+			}
+			break;
+		}
+		break;
+	case FORMAT_R:
+		*((float *) fd->v) = g_fltv(p);
+		if (fd->subtype == 1) {	/* note spacing factor */
+			int i;
+			float w;
+
+			if (cfmt.notespacingfactor <= 0) {
+				fprintf(stderr,
+					"Bad value for 'notespacingfactor'\n");
+				break;
+			}
+			dot_space = sqrt(cfmt.notespacingfactor);
+			i = C_XFLAGS;		/* crotchet index */
+			w = space_tb[i];
+			for ( ; --i >= 0; ) {
+				w /= cfmt.notespacingfactor;
+				space_tb[i] = w;
+			}
+			i = C_XFLAGS;
+			w = space_tb[i];
+			for ( ; ++i < NFLAGS_SZ; ) {
+				w *= cfmt.notespacingfactor;
+				space_tb[i] = w;
+			}
+		}
+		break;
+	case FORMAT_F: {
+		int b;
+
+		g_fspc(p, (struct FONTSPEC *) fd->v);
+		b = strstr(p, "box") != 0;
+		switch (fd->subtype) {
+		case 1:
+			cfmt.partsbox = b;
+			break;
+		case 2:
+			cfmt.measurebox = b;
+			break;
+		case 3:
+			cfmt.gchordbox = b;
+			break;
+		}
+		break;
+	    }
+	case FORMAT_U:
+		*((float *) fd->v) = scan_u(p);
+		if (fd->subtype == 1) {
+			float rmargin;
+
+			rmargin = (cfmt.landscape ? cfmt.pageheight : cfmt.pagewidth)
+				- staffwidth - cfmt.leftmargin;
+			if (rmargin < 0)
+				fprintf(stderr, "'staffwidth' too big\n");
+			cfmt.rightmargin = rmargin;
+		}
+		break;
+	case FORMAT_B:
+		*((int *) fd->v) = g_logv(p);
+		break;
+	case FORMAT_S: {
+		int l;
+
+		if (*((char **) fd->v) != 0)	/* !!no static allocation!! */
+			free(*((char **) fd->v));
+		l = strlen(p) + 1;
+		*((char **) fd->v) = malloc(l);
+		if (*p == '"')
+			get_str(*((char **) fd->v), p, l);
+		else	strcpy(*((char **) fd->v), p);
+		break;
+	    }
+	}
+	return 0;
 }
 
 /* -- read a format file -- */
-int read_fmt_file(char *filename,
-		  char *dirname)
+int read_fmt_file(char *filename)
 {
 	FILE *fp;
-	char fname[256];
+	char fname[512];
 
 	strcpy(fname, filename);
 	strext(fname, "fmt");
 	if ((fp = fopen(fname, "r")) == 0) {
-		if (*dirname == '\0')
+		if (*styd == '\0')
 			return -1;
-                sprintf(fname, "%s%c%s", dirname, DIRSEP, filename);
+                sprintf(fname, "%s%c%s", styd, DIRSEP, filename);
+		strext(fname, "fmt");
 		if ((fp = fopen(fname, "r")) == 0)
 			return -1;
 	}
 
 	for (;;) {
 		char line[BSIZE], *p, *q;
+		int l;
 
-		if (!fgets(line, sizeof line, fp))
+		p = line;
+		if (!fgets(p, sizeof line, fp))
 			break;
-		line[strlen(line) - 1] = '\0';	/* remove '\n' */
-		q = line;
-		while (isspace(*q))
-			q++;
-		p = q;
-		while (*p != '\0' && !isspace(*p))
-			p++;
-		if (*p != '\0')
-			*p++ = '\0';
+		l = strlen(p) - 1;
+		if (p[l] == '\n')
+			p[l] = '\0';
 		while (isspace(*p))
 			p++;
-		if (interpret_format_line(q, p) == 1)
+		if (*p == '\0' || *p == '%')
+			continue;
+		if (strcmp(p, "beginps") == 0) {
+			for (;;) {
+				p = line;
+				if (!fgets(p, sizeof line, fp))
+					break;
+				l = strlen(p) - 1;
+				if (p[l] == '\n')
+					p[l] = '\0';
+				while (isspace(*p))
+					p++;
+				if (*p == '\0' || *p == '%')
+					continue;
+				if (strcmp(p, "endps") == 0)
+					break;
+				interpret_format_line("postscript", p);
+			}
+			continue;
+		}
+		if (strcmp(p, "end") == 0)
 			break;
+		q = p;
+		while (*p != '\0' && !isspace(*p))
+			p++;
+		if (*p != '\0') {
+			*p++ = '\0';
+			while (isspace(*p))
+				p++;
+		}
+		interpret_format_line(q, p);
 	}
 	fclose(fp);
 	return 0;
@@ -670,5 +763,8 @@ void set_font(struct FONTSPEC *font)
 		      fontnames[fnum]);
 		fnum = 0;
 	}
+	if (font->size == 0)
+		error(0, 0, "Font \"%s\" with a null size",
+		      fontnames[fnum]);
 	PUT2("%.1f F%d ", font->size, fnum);
 }
