@@ -23,6 +23,7 @@
 
 #include <stdio.h>
 #include <string.h>
+#include <time.h>
 #include <ctype.h>
 
 #include "abcparse.h"
@@ -184,18 +185,49 @@ static void set_head_directions(struct SYMBOL *s)
 static void insert_clef(struct SYMBOL *s,
 			int clef_type)
 {
-	struct SYMBOL *new_s;
-	int time, seq;
+	struct VOICE_S *p_voice;
+	struct SYMBOL *s2, *new_s;
+	int staff, time, seq;
+
+	time = s->time;
+	seq = s->seq;
+
+	/* search the main voice of the staff */
+	staff = s->staff;
+	s2 = s;
+	if (voice_tb[s->voice].second) {
+		while (s2->ts_prev->time == time
+		       && s2->ts_prev->seq == seq)
+			s2 = s2->ts_prev;
+		for (; s2 != 0; s2 = s2->ts_next) {
+			if (s2->staff == staff && !voice_tb[s2->voice].second)
+				break;
+		}
+	}
 
 	/* create the symbol */
-	new_s = ins_sym(CLEF, s->prev);
+	if (s2 != 0) {
+		p_voice = &voice_tb[s2->voice];
+		p_voice->last_symbol = s2->prev;
+		new_s = add_sym(p_voice, CLEF);
+		new_s->next = s2;
+		s2->prev = new_s;
+	} else {
+		for (p_voice = first_voice; ; p_voice = p_voice->next)
+			if (p_voice->staff == staff && !p_voice->second)
+				break;
+		for (s2 = p_voice->sym; s2->next != 0; s2 = s2->next)
+			;
+		p_voice->last_symbol = s2;
+		new_s = add_sym(p_voice, CLEF);
+		new_s->staff = staff;
+	}
+
 	new_s->as.u.clef.type = clef_type;
 	new_s->as.u.clef.line = clef_type == TREBLE ? 2 : 4;
 	new_s->u = 1;		/* small clef */
 
 	/* link in time */
-	time = s->time;
-	seq = s->seq;
 	do {
 		s = s->ts_prev;
 	} while (s->time == time
@@ -216,20 +248,15 @@ static void set_clef(int staff)
 	struct SYMBOL *last_chg;
 	int clef_type;
 
+	/* get the max and min pitches */
 	min = max = 16;			/* 'C' */
-
-	/* count the number of notes upper and lower than 'C' */
 	for (s = first_voice->sym; s != 0; s = s->ts_next) {
-		int xp;
-
-		if (s->staff != staff
-		    || s->type != NOTE)
+		if (s->staff != staff || s->type != NOTE)
 			continue;
-		xp = s->nhd;
-		if (s->pits[xp] > max)
-			max = s->pits[xp];
-		else if (s->pits[0] < min)
+		if (s->pits[0] < min)
 			min = s->pits[0];
+		else if (s->pits[s->nhd] > max)
+			max = s->pits[s->nhd];
 	}
 
 	staff_tb[staff].clef.type = TREBLE;
@@ -248,11 +275,10 @@ static void set_clef(int staff)
 	for (s = first_voice->sym; s != 0; s = s->ts_next) {
 		struct SYMBOL *s2, *s3;
 
-		if (s->staff != staff
-		    || s->type != NOTE)
+		if (s->staff != staff || s->type != NOTE)
 			continue;
 
-		/* check if a clef change must occur */
+		/* check if a clef change may occur */
 		if (clef_type == TREBLE) {
 			if (s->pits[0] > 12		/* F, */
 			    || s->pits[s->nhd] > 20)	/* G */
@@ -286,17 +312,23 @@ static void set_clef(int staff)
 		}
 
 		/* go backwards and search where to insert a clef change */
+#if 1 /*fixme:test*/
+		s3 = s;
+#else
 		if (!voice_tb[s->voice].second
 		    && voice_tb[s->voice].staff == staff)
 			s3 = s;
 		else	s3 = 0;
+#endif
 		for (s2 = s->ts_prev; s2 != last_chg; s2 = s2->ts_prev) {
 			if (s2->staff != staff)
 				continue;
 			if (s2->type == BAR) {
+#if 0 /*fixme:test*/
 				if (voice_tb[s2->voice].second
 				    || voice_tb[s2->voice].staff != staff)
 					continue;
+#endif
 				s3 = s2;
 				break;
 			}
@@ -314,14 +346,21 @@ static void set_clef(int staff)
 				}
 			}
 
+#if 1 /*fixme:test*/
+			/* have a 2nd choice on beam start */
+			if ((s2->sflags & S_WORD_ST)
+			    || (s3->sflags & S_WORD_ST) == 0)
+				s3 = s2;
+#else
 			/* have a 2nd choice if word starts on the main voice */
 			if (!voice_tb[s2->voice].second
 			    && voice_tb[s2->voice].staff == staff) {
-				if (s2->sflags & S_WORD_ST
+				if ((s2->sflags & S_WORD_ST)
 				    || s3 == 0
 				    || (s3->sflags & S_WORD_ST) == 0)
 					s3 = s2;
 			}
+#endif
 		}
 		s2 = last_chg;
 		last_chg = s;
@@ -330,8 +369,12 @@ static void set_clef(int staff)
 		if (s2 == 0) {
 			struct SYMBOL *s4;
 
+#if 1 /*fixme:test*/
+			s4 = s3;
+#else
 			if ((s4 = s3) == 0)
 				s4 = s;
+#endif
 			for (s4 = s4->ts_prev; s4 != 0; s4 = s4->ts_prev) {
 				if (s4->staff != staff)
 					continue;
@@ -354,7 +397,11 @@ static void set_clef(int staff)
 		}
 
 		/* no change possible if no insert point */
+#if 1 /*fixme:test*/
+		else if (s3->time == s2->time)
+#else
 		if (s3 == 0 || s3 == s2)
+#endif
 			continue;
 
 		/* insert a clef change */
@@ -642,9 +689,8 @@ static void set_pitch(void)
 			continue;
 		default:
 			set_yval(s);
-			if (s->type != GRACE)
+			if ((g = s->grace) == 0)
 				continue;
-			g = s->grace;
 			delta = staff_clef[staff];
 			for (; g != 0; g = g->next) {
 				if (delta != 0) {
@@ -682,7 +728,7 @@ static void set_pitch(void)
 		pav = 0;
 		for (m = np; m >= 0; m--)
 			pav += s->pits[m];
-		s->yav = 3 * ((pav / (np + 1)) - 18);
+		s->yav = 3 * pav / (np + 1) - 3 * 18;
 		s->ymx = 3 * (s->pits[np] - 18) + 2;
 		s->dc_top = s->ymx;
 		s->ymn = 3 * (s->pits[0] - 18) - 2;
@@ -743,16 +789,16 @@ static void set_multi(void)
 				vtb[t->voice].st1 = staff;
 			else if (vtb[t->voice].st1 == staff)
 				;
-			else if (vtb[t->voice].st2 < 0) {
-				if (vtb[t->voice].st1 < staff)
-					vtb[t->voice].st2 = staff;
-				else {
-					vtb[t->voice].st2 = vtb[t->voice].st1;
+			else {
+				if (staff > vtb[t->voice].st1) {
+					if (staff > vtb[t->voice].st2)
+						vtb[t->voice].st2 = staff;
+				} else {
+					if (vtb[t->voice].st1 > vtb[t->voice].st2)
+						vtb[t->voice].st2 = vtb[t->voice].st1;
 					vtb[t->voice].st1 = staff;
 				}
-			} else if (vtb[t->voice].st2 != staff)
-				error(1, t,
-				      "Voice on more than 2 staves");
+			}
 			rvoice = vtb[t->voice].range;
 			for (i = stb[staff].nvoice; i >= 0; i--) {
 				if (stb[staff].st[i].voice == rvoice)
@@ -785,22 +831,17 @@ static void set_multi(void)
 		     s = s->ts_next) {
 			int us, ls;
 
-#if 0
-			if (s->len == 0		/* not a note nor a rest */
-			    || s->as.u.note.invis)
-				continue;
-#endif
 			staff = s->staff;
 			if (vtb[s->voice].st2 >= 0) {
-				if (staff != vtb[s->voice].st2)
+				if (staff == vtb[s->voice].st1)
 					s->multi = -1;
-				else	s->multi = 1;
+				else if (staff == vtb[s->voice].st2)
+					s->multi = 1;
 			}
-			if (stb[staff].nvoice <= 0) {
-
-				/* voice alone on the staff */
+			if (stb[staff].nvoice <= 0) { /* voice alone on the staff */
 				if (s->multi != 0)
 					continue;
+/*fixme:could be done in set_float()*/
 				p_voice = &voice_tb[s->voice];
 				if (p_voice->floating) {
 					if (s->staff == p_voice->staff)
@@ -826,7 +867,7 @@ static void set_multi(void)
 					 * have stems down for the middle voice */
 					if (i != 0
 					    && i + 1 == stb[staff].nvoice) {
-						if (stb[staff].st[i].ymn - STEM
+						if (stb[staff].st[i].ymn - cfmt.stemheight
 						    > stb[staff].st[i + 1].ymx)
 							s->multi = -1;
 
@@ -851,16 +892,19 @@ static void set_multi(void)
 			us = rest_sp[C_XFLAGS - s->nflags].u;
 			ls = rest_sp[C_XFLAGS - s->nflags].l;
 
-			if (i == 0) {
-				t = s->ts_next;
-				if (t != 0
-				    && t->type == REST
-				    && t->as.u.note.invis
-				    && t->ts_next != 0
-				    && t->ts_next->staff == staff
-				    && t->ts_next->time == s->time)
+			if (i == 0) {			/* first voice */
+				t = s;
+				while (t->ts_next != 0
+				       && t->ts_next->staff == staff
+				       && t->ts_next->time == s->time) {
 					t = t->ts_next;
-				if (t == 0
+					if (t->type != REST
+					     || !t->as.u.note.invis)
+						break;
+				}
+				if (t == s
+				    || t->staff != staff
+				    || t->time != s->time
 				    || t->type != REST
 				    || !t->as.u.note.invis) {
 					s->y = (stb[staff].st[1].ymx + ls)
@@ -868,39 +912,42 @@ static void set_multi(void)
 					if (s->y < 12)
 						s->y = 12;
 				}
-			} else if (i == stb[staff].nvoice) {
-				t = s->ts_prev;
-				if (t->type == REST
-				    && t->as.u.note.invis
-				    && t->ts_prev != 0
-				    && t->ts_prev->staff == staff
-				    && t->ts_prev->time == s->time)
+			} else if (i == stb[staff].nvoice) { /* last voice */
+				t = s;
+				while (t->ts_prev->staff == staff
+				       && t->ts_prev->time == s->time) {
 					t = t->ts_prev;
-				if (t->type != REST
+					if (t->type != REST
+					     || !t->as.u.note.invis)
+						break;
+				}
+				if (t == s
+				    || t->staff != staff
+				    || t->time != s->time
+				    || t->type != REST
 				    || !t->as.u.note.invis) {
 					s->y = (stb[staff].st[i - 1].ymn - us + 48)
 						/ 6 * 6 - 48;
 					if (s->y > 12)
 						s->y = 12;
 				}
-			} else {
-				if ((t = s->ts_next) != 0
-				    && t->staff == staff
-				    && t->time == s->time
-				    && t->len != 0
-				    && !t->as.u.note.invis
-				    && s->ts_prev->staff == staff
-				    && s->ts_prev->time == s->time
-				    && s->ts_prev->len != 0
-				    && !s->ts_prev->as.u.note.invis) {
-
-					/* rest in the middle of 3 voices */
-					s->shhd[0] = 10;
+			} else {		/* middle voice */
 /*fixme: may be too high*/
-					s->y = (stb[staff].st[i - 1].ymn
-						+ stb[staff].st[i + 1].ymx)
-						/ 12 * 6;
-				}
+				s->y = (stb[staff].st[i - 1].ymn
+					+ stb[staff].st[i + 1].ymx)
+					/ 12 * 6;
+				if (((t = s->ts_next) != 0
+				     && t->staff == staff
+				     && t->time == s->time
+				     && t->len != 0
+				     && !t->as.u.note.invis
+				     && t->ymx > s->y - ls)
+				    || (s->ts_prev->staff == staff
+					&& s->ts_prev->time == s->time
+					&& s->ts_prev->len != 0
+					&& !s->ts_prev->as.u.note.invis
+					&& s->ts_prev->ymn < s->y + us))
+					s->shhd[0] = 10;
 			}
 			s->ymx = s->y + us;
 			s->dc_top = s->ymx;
@@ -1107,11 +1154,11 @@ static void set_global(void)
 				seq = 0;
 				break;
 			case REST:
-				if (s->len != 0) {
+				if (s->len != 0) {	/* if not space */
 					seq = 0;
 					break;
 				}
-				s->seq = 0;	/* space ('y') */
+				s->seq = s->next == 0 ? 0 : (s->next->seq - 1);
 			default:
 				if (s->seq <= seq)
 					s->seq = seq + 1;
@@ -1152,9 +1199,9 @@ static void set_global(void)
 static float set_indent(int first_line)
 {
 	int staff;
-	float more_shift, w, maxw;
+	float w, maxw;
 	struct VOICE_S *p_voice;
-	char t[64], *p, *q;
+	char *p, *q;
 
 	maxw = 0;
 	for (p_voice = first_voice; p_voice; p_voice = p_voice->next) {
@@ -1165,7 +1212,7 @@ static float set_indent(int first_line)
 		for (;;) {
 			if ((q = strstr(p, "\\n")) != 0)
 				*q = '\0';
-			w = tex_str(t, p, sizeof t);
+			w = tex_str(p);
 			if (w > maxw)
 				maxw = w;
 			if (q == 0)
@@ -1179,16 +1226,15 @@ static float set_indent(int first_line)
 	if (maxw == 0)
 		return first_line ? cfmt.indent : 0;
 
-	more_shift = 0;
+	maxw += 4 * cwid(' ') * cfmt.voicefont.swfac;
 	for (staff = 0; staff <= nstaff; staff++) {
 		if (staff_tb[staff].brace
 		    || staff_tb[staff].bracket) {
-			more_shift = 10;
+			maxw += 10;
 			break;
 		}
 	}
-	return cfmt.voicefont.swfac * cfmt.voicefont.size * (maxw + 4 * cwid(' '))
-		+ more_shift;
+	return maxw;
 }
 
 /* -- set the y offset of the staves and return the whole height -- */
@@ -1227,7 +1273,8 @@ static float set_staff(void)
 	delta_tb[nstaff].vocal = 1;
 
 	any_part = any_tempo = 0;
-	for (s = first_voice->sym; s != 0; s = s->ts_next) {
+	for (s = first_voice->sym->next;	/* skip the clefs */
+	     s != 0; s = s->ts_next) {
 /*fixme: bad! better have a flag in the voice*/
 		if (s->voice == first_voice - voice_tb) {
 			switch (s->type) {
@@ -1295,23 +1342,18 @@ static float set_staff(void)
 	     staff++, p_delta++) {
 		p_delta->ctop = y + dy;
 		dy += p_delta->mtop + vocal_height * p_delta->avoc + 2;
-/*fixme:test
-fprintf(stderr, "s:%d dy:%.2f sep:%.2f max:%.2f\n", staff, dy, staffsep, maxsep);
-*/
 		if (dy < staffsep)
 			dy = staffsep;
 		else if (dy > maxsep)
 			dy = maxsep;
 		y += dy;
 		staff_tb[staff].y = -y;
+		PUT2("/y%d{%.1f add}def\n", staff, -y);
 		if (staff == 0) {
 			staffsep = cfmt.sysstaffsep + 24;
 			if (cfmt.maxsysstaffsep >= cfmt.sysstaffsep)
 				maxsep = cfmt.maxsysstaffsep + 24;
 			else	maxsep = 800;
-/*fixme:test
-fprintf(stderr, "c.sep:%.2f c.max:%.2f sep:%.2f max:%.2f\n", cfmt.sysstaffsep, cfmt.sysstaffsep, staffsep, maxsep);
-*/
 		}
 		dy = -p_delta->mbot;
 		p_delta->cbot = y + dy + 2;
@@ -1376,10 +1418,9 @@ static void set_beams(struct SYMBOL *sym)
 	do_break = 0;
 	for (s = sym; s != 0; s = s->next) {
 		if (s->type != NOTE) {
-			if (s->type == GRACE) {
-				struct SYMBOL *g;
+			struct SYMBOL *g;
 
-				g = s->grace;
+			if ((g = s->grace) != 0) {
 				if (s->multi == 0)
 					s->multi = 1;
 				s->stem = s->multi;
@@ -1406,25 +1447,24 @@ static void set_beams(struct SYMBOL *sym)
 
 			/* notes in a beam have the same stem direction */
 			if ((s->sflags & S_WORD_ST)
-			    && !s->as.u.note.word_end) {	/* start of beam */
+			    && !s->as.u.note.word_end) { /* start of beam */
 				int avg, n;
 
 				avg = s->yav;
-				n = 1;
+				n = 12;
 				for (t = s->next; t != 0; t = t->next) {
 					if (t->type == NOTE) {
 						avg += t->yav;
-						n++;
+						n += 12;
 					}
 					if (t->as.u.note.word_end)
 						break;
 				}
-				avg /= n;
 				stem = 1;
-				if (avg >= 12) {
+				if (avg >= n) {
 					stem = -1;
 					if (!cfmt.bstemdown
-					    && avg == 12
+					    && avg < n + n / 12
 					    && laststem != 0)
 						stem = laststem;
 				}
@@ -1551,7 +1591,8 @@ static void set_overlap(void)
 		int d, m, nhd2;
 		float d1, d2, x1, x2, dy1, dy2, noteshift;
 
-		if (s->type != NOTE)
+		if (s->type != NOTE
+		    || s->as.u.note.invis)
 			continue;
 
 		if ((s2 = s->ts_next) == 0)
@@ -1568,7 +1609,8 @@ static void set_overlap(void)
 			    || s->staff != s2->staff
 			    || s->time != s2->time)
 				continue;
-		}
+		} else if (s->as.u.note.invis)
+			continue;
 
 		nhd2 = s2->nhd;
 
@@ -1633,7 +1675,7 @@ static void set_overlap(void)
 				break;
 			s2->sflags |= S_NO_HEAD;	/* same head */
 			s2->as.u.note.accs[nhd2] = 0;
-			d2 = s1->shhd[0];
+/*			d2 = s1->shhd[0]; */
 			dy2 = 0;
 			if (l1 == l2)
 				goto do_shift;
@@ -1762,7 +1804,7 @@ static void set_overlap(void)
 				s1->xmx += d1;
 			}
 			s2->xmx = x2;
-		} else /*if (d2 > 0)*/ {
+		} else {
 			if (s1->stem > 0)
 				d2 += s1->shhd[0];
 			else	d2 += s1->shhd[nhd2];
@@ -1791,9 +1833,8 @@ static void set_stems(void)
 
 	for (s = first_voice->sym; s != 0; s = s->ts_next) {
 		if (s->type != NOTE) {
-			if (s->type != GRACE)
+			if ((g = s->grace) == 0)
 				continue;
-			g = s->grace;
 			ymin = 0;
 			ymax = 24;
 			for (; g != 0; g = g->next) {
@@ -1831,13 +1872,15 @@ static void set_stems(void)
 		set_head_directions(s);
 
 		/* set height of stem end, without considering beaming for now */
+		slen = cfmt.stemheight;
 		switch (s->nflags) {
-		default: slen = STEM; break;
-		case 2: slen = STEM + 2; break;
-		case 3:	slen = STEM + 5; break;
-		case 4:	slen = STEM + 10; break;
-		case 5:	slen = STEM + 16; break;
+		case 2: slen += 2; break;
+		case 3:	slen += 5; break;
+		case 4:	slen += 10; break;
+		case 5:	slen += 16; break;
 		}
+		if (s->nhd > 0)
+			slen -= 2;
 		ymn = 3 * (s->pits[0] - 18);
 		ymx = 3 * (s->pits[s->nhd] - 18);
 		if (s->as.u.note.stemless) {
@@ -1851,7 +1894,7 @@ static void set_stems(void)
 			ymin = (float) (ymn - 4);
 			ymax = (float) (ymx + 4);
 		} else if (s->stem >= 0) {
-			if (s->nflags == 2)
+			if (s->nflags >= 2)
 				slen -= 1;
 			if (s->pits[s->nhd] > 26
 			    && (s->nflags <= 0
@@ -1909,7 +1952,6 @@ static void set_width(struct SYMBOL *s)
 	int i, m;
 	struct SYMBOL *s2, *k;
 	float xx, w, wlnote, wlw;
-	unsigned char t[81];
 
 	switch (s->type) {
 	case NOTE:
@@ -1918,13 +1960,13 @@ static void set_width(struct SYMBOL *s)
 		/* set the note widths */
 		switch (s->head) {
 		case H_SQUARE:
-			wlnote = 8.0;
+			wlnote = 8;
 			break;
 		case H_OVAL:
-			wlnote = 6.0;
+			wlnote = 6;
 			break;
 		case H_EMPTY:
-			wlnote = 5.0;
+			wlnote = 5;
 			break;
 		default:
 			wlnote = 4.5;
@@ -2001,24 +2043,30 @@ static void set_width(struct SYMBOL *s)
 		/* !! this sequence is tied to draw_gchord() !! */
 		if (s->as.text != 0) {
 			float lspc, rspc;
-			char *p, *q;
+			char *p, *q, sep, antype;
 
-			str_font(&cfmt.gchordfont);
+			str_font(&cfmt.font_tb[s->gcf]);
+			lspc = rspc = cwid(' ') * cfmt.font_tb[s->gcf].swfac;
 			p = s->as.text;
-			lspc = rspc = 0;
+			antype = '\0';
+			sep = '\n';
 			for (;;) {
-				if ((q = strchr(p, '\n')) != 0)
-					*q = '\0';
-				w = tex_str(t, p, sizeof t)
-					* cfmt.gchordfont.size;
-				p = t;
-				switch (*p) {
+				if (sep == '\n') {
+					if (*p != '\0' && strchr("^_<>@", *p) != 0)
+						antype = *p++;
+					else	antype = '\0';
+				}
+				for (q = p; ; q++) {
+					if (*q == '\0' || *q == ';' || *q == '\n') {
+						sep = *q;
+						*q = '\0';
+						break;
+					}
+				}
+				w = tex_str(p);
+				switch (antype) {
 				case '^':		/* above */
 				case '_':		/* below */
-					if (*p == '^')
-						w -= cwid('^') * cfmt.gchordfont.size;
-					else	w -= cwid('_') * cfmt.gchordfont.size;
-					/* fall thru */
 				default: {		/* default = above */
 					float wl;
 
@@ -2033,7 +2081,6 @@ static void set_width(struct SYMBOL *s)
 					break;
 				    }
 				case '<':		/* left */
-					w -= cwid('<') * cfmt.gchordfont.size;
 					w += wlnote;
 					if (w > lspc)
 						lspc = w;
@@ -2046,81 +2093,97 @@ static void set_width(struct SYMBOL *s)
 				case '@':		/* absolute */
 					break;
 				}
-				if (q == 0)
+				if (sep == '\0')
 					break;
-				*q = '\n';
+				*q = sep;
 				p = q + 1;
 			}
 /*fixme: pb when '<' only*/
 			if (s2->as.text != 0)
 				AT_LEAST(wlw, lspc);
 /*fixme: pb when '>' only*/
-			if ((k = s->next) != 0
-			    && k->as.text != 0
-/*fixme: may have some other symbols with guitar chords?*/
-			    && (k->len > 0 || k->type == BAR))
-				AT_LEAST(s->wr, rspc);
+			for (k = s->next; k != 0; k = k->next) {
+				switch (k->type) {
+				default:
+					continue;
+				case NOTE:
+				case REST:
+				case BAR:
+					if (k->as.text != 0)
+						AT_LEAST(s->wr, rspc);
+					break;
+				}
+				break;
+			}
 		}
 
 		/* leave room for vocals under note */
 		/* related to draw_vocals() */
-/*fixme: pb when lyrics of 2 voices in the same staff */
 		if (s->ly) {
 			struct lyrics *ly = s->ly;
-			float swfac;
+			struct lyl *lyl;
+			float align = 0;
 
-			swfac = cfmt.vocalfont.size * cfmt.vocalfont.swfac;
 			for (i = 0; i < MAXLY; i++) {
-				float shift;
-				char *p;
+				float swfac, shift;
+				unsigned char *p;
 
-				if ((p = ly->w[i]) == 0)
+				if ((lyl = ly->lyl[i]) == 0)
 					continue;
-				str_font(&cfmt.vocalfont);
-				p++;
-				w = tex_str(t, p, sizeof t);
-				xx = swfac * (w + 2 * cwid(' '));
-				if (isdigit(t[0]) || t[1] == ':'
-				    || *p == '(' || *p == ')') {
+				p = lyl->t;
+				w = lyl->w;
+				swfac = lyl->f->swfac;
+				xx = w + 2 * cwid(' ') * swfac;
+				if (isdigit(*p) || p[1] == ':'
+				    || p[1] == '(' || p[1] == ')') {
 					float sz;
 
-					if (*p == '(')
-						sz = cwid(*p);
+					if (p[1] == '(')
+						sz = cwid(p[1]);
 					else {
 						sz = 0;
-						p = t;
-						if (*p == '\\')
-							p++;
 						while (*p != '\0') {
 /*fixme: KO when '\ooo'*/
+							if (*p == '\\') {
+								p++;
+								continue;
+							}
 							sz += cwid(*p);
 							if (*p == ' ')
 								break;
 							p++;
 						}
 					}
-					shift = swfac * (w - sz + 2 * cwid(' ')) * VOCPRE;
+					sz *= swfac;
+					shift = (w - sz + 2 * cwid(' ') * swfac)
+						* VOCPRE;
 					if (shift > 20.)
 						shift = 20.;
-					shift += swfac * sz;
-				} else {
+					shift += sz;
+					if (isdigit(lyl->t[0])) {
+						if (shift > align)
+							align = shift;
+					}
+				} else if (*p == '\x02' || *p == '\x03')
+					shift = 0;
+				else {
 					shift = xx * VOCPRE;
 					if (shift > 20.)
 						shift = 20.;
 				}
+				lyl->s = shift;
 				AT_LEAST(wlw, shift);
 				xx -= shift;
-#if 1
-				shift = swfac * 2 * cwid(' ');
+				shift = 2 * cwid(' ') * swfac;
 				for (k = s->next; k != 0; k = k -> next) {
 					switch (k->type) {
 					case NOTE:
 					case REST:
 						if (k->ly == 0
-						    || k->ly->w[i] == 0)
+						    || k->ly->lyl[i] == 0)
 							xx -= 9;
-						else if (k->ly->w[i][1] == '\x02'
-							 || k->ly->w[i][1] == '\x03')
+						else if (k->ly->lyl[i]->t[0] == '\x02'
+							 || k->ly->lyl[i]->t[0] == '\x03')
 							xx -= shift;
 						else	break;
 						if (xx <= 0)
@@ -2143,18 +2206,15 @@ static void set_width(struct SYMBOL *s)
 				}
 				if (xx > s->wr)
 					s->wr = xx;
-#else
-				if ((k = s->next) != 0
-#if 0
-				    && k->len > 0	/* note or rest */
-#endif
-				    && (k->ly == 0
-					|| k->ly->w[i] == 0
-					|| k->ly->w[i][1] == '\x02'
-					|| k->ly->w[i][1] == '\x03'))
-					xx -= 10.;	/*fixme: which max width?*/
-				AT_LEAST(s->wr, xx);
-#endif
+			}
+			if (align > 0) {
+				for (i = 0; i < MAXLY; i++) {
+
+					if ((lyl = ly->lyl[i]) == 0)
+						continue;
+					if (isdigit(lyl->t[0]))
+					lyl->s = align;
+				}
 			}
 		}
 
@@ -2270,6 +2330,7 @@ static void set_width(struct SYMBOL *s)
 				s->wr = 5;
 				s->pr = 5.5;
 			}
+			s->shhd[0] = (w - 5) * -0.5;
 		}
 		if (s->as.u.bar.dc.n > 0)
 			s->wl += deco_width(s);
@@ -2277,17 +2338,16 @@ static void set_width(struct SYMBOL *s)
 		/* have room for the repeat numbers / guitar chord */
 		if (s->as.text == 0)
 			break;
-		str_font(&cfmt.gchordfont);
-		w = tex_str(t, s->as.text, sizeof t)
-			+ cwid(' ') * 1.5;
-		xx = cfmt.gchordfont.size * cfmt.gchordfont.swfac * w;
+		str_font(&cfmt.font_tb[s->gcf]);
+		xx = tex_str(s->as.text)
+			+ cwid(' ') * cfmt.font_tb[s->gcf].swfac * 1.5;
 		if (!s->as.u.bar.repeat_bar) {
 			if (s->prev->as.text != 0) {
 				float spc;
 
 				spc = xx * GCHPRE;
-				if (spc > 8.0)
-					spc = 8.0;
+				if (spc > 8)
+					spc = 8;
 				AT_LEAST(s->wl, spc);
 				s->pl = s->wl;
 				xx -= spc;
@@ -2588,6 +2648,8 @@ static void set_xoffset(struct SYMBOL *s,
 	case REST:
 		if (prev->type == GRACE)
 			break;
+		if (prev->type == BAR)
+			space *= 1.5;
 		if ((s->sflags & S_WORD_ST) == 0) /* reduce spacing within a beam */
 			space *= fnnp;
 		if (s->sflags & S_IN_TUPLET)	/* reduce spacing in n-plet */
@@ -2806,6 +2868,10 @@ static void set_sym_glue(float width)
 		if (s2 != 0
 		    && s2->time > first_voice->sym->time)
 			s = s2;
+		else if (s->time <= first_voice->sym->time) {
+			error(1, s, "No symbol in the music line");
+			break;
+		}
 
 		/* don't cut in a grace note sequence if followed by note */
 		if (s->type == GRACE
@@ -2894,9 +2960,8 @@ static void set_sym_glue(float width)
 		for (s = first_voice->sym; s != 0; s = s->ts_next) {
 			struct SYMBOL *g;
 
-			if (s->type != GRACE)
+			if ((g = s->grace) == 0)
 				continue;
-			g = s->grace;
 			for ( ; g->next != 0; g = g->next)
 				;
 			space = s->x - g->x;
@@ -3024,7 +3089,7 @@ static void init_music_line(struct VOICE_S *p_voice)
 	if (sym != 0) {
 		while (sym->type == FMTCHG
 		       && sym->u == PSSEQ) {
-			PUT1("%s\n", &sym->as.text[13]);
+			PUT1("%s\n", sym->as.text);
 			if (sym->next != 0)
 				sym->next->prev = 0;
 			if (sym->ts_next != 0)
@@ -3185,17 +3250,6 @@ static void cut_symbols(void)
 		tsnext->ts_prev = s;
 }
 
-/* -- adjust the values in the postscript buffer -- */
-static void buffer_adjust(void)
-{
-	int i;
-	float v_tb[MAXSTAFF];
-
-	for (i = 0; i <= nstaff; i++)
-		v_tb[i] = staff_tb[i].y;
-	set_buffer(v_tb);
-}
-
 /* -- output for parsed symbol list -- */
 void output_music(void)
 {
@@ -3221,7 +3275,7 @@ void output_music(void)
 	alfa_last = 0.1;
 	beta_last = 0;
 
-	check_buffer();	/* dump buffer if not enough space for a staff line */
+	check_buffer();	/* dump buffer if not enough space for a music line */
 
 	set_global();			/* set global characteristics */
 	if (first_voice->next != 0)	/* when multi-voices */
@@ -3237,6 +3291,10 @@ void output_music(void)
 	lwidth = ((cfmt.landscape ? cfmt.pageheight : cfmt.pagewidth)
 		- cfmt.leftmargin - cfmt.rightmargin)
 			/ cfmt.scale;
+	if (lwidth < 1*CM) {
+		error(1, 0, "Bad page width %.1f", lwidth);
+		lwidth = 1*CM;
+	}
 	for (;;) {		/* loop over pieces of line for output */
 		find_piece();
 
@@ -3247,9 +3305,11 @@ void output_music(void)
 			set_sym_glue(lwidth - indent);
 			if (indent != 0)
 				PUT1("%.2f 0 T\n", indent); /* do indentation */
+			PUT0("/dlsym{\n");
 			draw_sym_near();
+			PUT0("}def\n");
 			line_height = set_staff();
-			buffer_adjust();
+			PUT0("dlsym\n");
 			draw_staff(first_line, indent);
 			for (p_voice = first_voice; p_voice; p_voice = p_voice->next)
 				draw_symbols(p_voice);
