@@ -24,6 +24,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <time.h>
 #include <ctype.h>
 
 #include "abcparse.h"
@@ -32,9 +33,6 @@
 struct STAFF staff_tb[MAXSTAFF];	/* staff table */
 int nstaff;				/* (0..MAXSTAFF-1) */
 
-struct FONTSPEC lyric_fonts[8];
-int nlyric_font;
-
 struct VOICE_S voice_tb[MAXVOICE];	/* voice table */
 static struct VOICE_S *curvoice;	/* current voice while parsing */
 struct VOICE_S *first_voice;		/* first voice */
@@ -57,17 +55,23 @@ float multicol_start;			/* (for multicol) */
 static float multicol_max;
 static float lmarg, rmarg;
 
-#define SQ_ANY 0x00
-#define SQ_CLEF 0x10
-#define SQ_SIG 0x20
-#define SQ_GRACE 0x30
-#define SQ_BAR 0x40
-#define SQ_EXTRA 0x50
-#define SQ_NOTE 0x70
-static unsigned char seq_tb[15] = {	/* sequence # indexed by symbol type */
-	SQ_EXTRA, SQ_NOTE, SQ_NOTE, SQ_BAR, SQ_CLEF,
-	SQ_SIG, SQ_SIG, SQ_ANY, SQ_ANY, SQ_NOTE,
-	SQ_ANY, SQ_NOTE, SQ_GRACE, SQ_ANY, SQ_ANY
+/* sequence numbers for symbol grouping - index = symbol type */
+static unsigned char seq_tb[15] = {
+	0x60,		/* 0: notype */
+	0x78,		/* 1: note */
+	0x78,		/* 2: rest */
+	0x40,		/* 3: bar */
+	0x10,		/* 4: clef */
+	0x20,		/* 5: timesig */
+	0x18,		/* 6: keysig */
+	0x28,		/* 7: tempo */
+	0x00,		/* 8: staves */
+	0x78,		/* 9: mrest */
+	0x30,		/* 10: part */
+	0x78,		/* 11: mrep */
+	0x38,		/* 12: grace */
+	0x08,		/* 13: fmtchg */
+	0x60,		/* 14: tuplet */
 };
 
 static void get_clef(struct SYMBOL *s);
@@ -100,27 +104,11 @@ struct SYMBOL *add_sym(struct VOICE_S *p_voice,
 	return s;
 }
 
-/* -- insert a symbol after a reference one -- */
-struct SYMBOL *ins_sym(int type,
-		       struct SYMBOL *s)	/* previous symbol */
-{
-	struct VOICE_S *p_voice;
-	struct SYMBOL *new_s, *next;
-
-	curvoice = p_voice = &voice_tb[s->voice];
-	p_voice->last_symbol = s;
-	next = s->next;
-	new_s = add_sym(p_voice, type);
-	if ((new_s->next = next) != 0)
-		next->prev = new_s;
-	return new_s;
-}
-
 /* -- duplicate the symbols of the voices appearing in many staves -- */
 void voice_dup(void)
 {
 	struct VOICE_S *p_voice, *p_voice1;
-	struct SYMBOL *s, *s2;
+	struct SYMBOL *s, *s2, *g, *g2;
 
 	for (p_voice = first_voice; p_voice; p_voice = p_voice->next) {
 		int voice;
@@ -140,10 +128,26 @@ void voice_dup(void)
 				s2->prev = p_voice->last_symbol;
 			} else	p_voice->sym = s2;
 			p_voice->last_symbol = s2;
-
 			s2->voice = voice;
 			s2->staff = p_voice->staff;
 			s2->ly = 0;
+			if ((g = s2->grace) != 0) {
+				g2 = (struct SYMBOL *) getarena(sizeof *g2);
+				memcpy(g2, g, sizeof *g2);
+				s2->grace = g2;
+				s2 = g2;
+				s2->voice = voice;
+				s2->staff = p_voice->staff;
+				for (g = g->next; g != 0; g = g->next) {
+					g2 = (struct SYMBOL *) getarena(sizeof *g2);
+					memcpy(g2, g, sizeof *g2);
+					s2->next = g2;
+					g2->prev = s2;
+					s2 = g2;
+					s2->voice = voice;
+					s2->staff = p_voice->staff;
+				}
+			}
 		}
 	}
 }
@@ -154,9 +158,11 @@ static void gchord_adjust(struct SYMBOL *s)
 	char *p;
 	int freegchord, l;
 
+	s->gcf = cfmt.gcf;
+	s->anf = cfmt.anf;
 	freegchord = cfmt.freegchord;
 	p = s->as.text;
-	if (strchr("^_<>@", *p) != 0)
+	if (*p != '\0' && strchr("^_<>@", *p) != 0)
 		freegchord = 1;		/* annotation */
 /*fixme: treat 'dim' as 'o', 'halfdim' as '/o', and 'maj' as a triangle*/
 	while (*p != '\0') {
@@ -173,16 +179,13 @@ static void gchord_adjust(struct SYMBOL *s)
 			if (!freegchord)
 				*p = '\203';
 			break;
-		case ';':
-			*p = '\n';	/* abcMIDI compatibility */
-			break;
 		case '\\':
 			p++;
 			switch (*p) {
 			case '\0':
 				return;
 			case 'n':
-				p[-1] = '\n';
+				p[-1] = ';';
 				goto move;
 			case '#':
 				p[-1] = '\201';
@@ -200,8 +203,11 @@ static void gchord_adjust(struct SYMBOL *s)
 			}
 			break;
 		}
+		if (*p == ';' && p[-1] != '\\')
+			if (p[1] != '\0' && strchr("^_<>@", p[1]) != 0)
+				*p = '\n';
 		if (*p == '\n') {
-			if (strchr("^_<>@", p[1]) != 0)
+			if (p[1] != '\0' && strchr("^_<>@", p[1]) != 0)
 				freegchord = 1;
 			else	freegchord = cfmt.freegchord;
 		}
@@ -213,23 +219,12 @@ static void gchord_adjust(struct SYMBOL *s)
 static char *get_lyric(unsigned char *p)
 {
 	struct SYMBOL *s;
-	char word[81], *w;
+	unsigned char word[81], *q;
 	int ln;
-	int curfont;
+	struct FONTSPEC *f;
 
-	/* search/create the lyric font */
-	for (curfont = 0; curfont < nlyric_font; curfont++) {
-		if (lyric_fonts[curfont].fnum == cfmt.vocalfont.fnum
-		    && lyric_fonts[curfont].size == cfmt.vocalfont.size)
-			break;
-	}
-	if (curfont >= nlyric_font) {
-		if (curfont >= sizeof lyric_fonts / sizeof lyric_fonts[0])
-			return "Too many lyric fonts";
-		memcpy(&lyric_fonts[curfont], &cfmt.vocalfont,
-			sizeof lyric_fonts[0]);
-		nlyric_font++;
-	}
+	f = &cfmt.font_tb[cfmt.vof];
+	str_font(f);				/* (for tex_str) */
 
 	if ((s = lyric_cont) == 0) {
 		if (lyric_nb >= MAXLY)
@@ -278,7 +273,7 @@ static char *get_lyric(unsigned char *p)
 			}
 			/* fall thru */
 		default:
-			w = word;
+			q = word;
 			for (;;) {
 				unsigned char c;
 
@@ -313,8 +308,8 @@ static char *get_lyric(unsigned char *p)
 					/* fall thru */
 				default:
 				addch:
-					if (w < &word[sizeof word - 1])
-						*w++ = c;
+					if (q < &word[sizeof word - 1])
+						*q++ = c;
 					p++;
 					if (c == '\x02')
 						break;
@@ -322,7 +317,7 @@ static char *get_lyric(unsigned char *p)
 				}
 				break;
 			}
-			*w = '\0';
+			*q = '\0';
 			break;
 		}
 
@@ -332,17 +327,20 @@ static char *get_lyric(unsigned char *p)
 		if (s == 0)
 			return "Not enough notes for lyric line";
 		if (word[0] != '*') {
-			int l;
+			struct lyl *lyl;
+			float w;
 
 			if (s->ly == 0) {
 				s->ly = (struct lyrics *) getarena(sizeof (struct lyrics));
 				memset(s->ly, 0, sizeof (struct lyrics));
 			}
-			l = strlen(word) + 1 + 1;
-			w = getarena(l);
-			s->ly->w[ln] = w;
-			w[0] = curfont;		/* the 1st char is the font index */
-			strcpy(w + 1, word);
+			w = tex_str(word);
+			lyl = (struct lyl *) getarena(sizeof *s->ly->lyl[0]
+						    + strlen(tex_buf));
+			s->ly->lyl[ln] = lyl;
+			lyl->f = f;
+			lyl->w = w;
+			strcpy(lyl->t, tex_buf);
 		}
 		s = s->next;
 	}
@@ -358,7 +356,7 @@ static void get_over(struct SYMBOL *s)
 {
 	struct VOICE_S *p_voice, *p_voice2;
 	struct SYMBOL *s1, *s2;
-	int ctime, otype, linenum;
+	int ctime, linenum;
 
 	/* treat the end of overlay */
 	p_voice = curvoice;
@@ -370,7 +368,7 @@ static void get_over(struct SYMBOL *s)
 		s2->as.type = ABC_T_BAR;
 		s2->as.linenum = linenum;
 		s2->time = curvoice->time;
-		s2->seq = SQ_BAR;
+		s2->seq = seq_tb[BAR];
 	}
 	if (s == 0
 	    || s->as.u.v_over.type == V_OVER_E)  {
@@ -389,9 +387,7 @@ static void get_over(struct SYMBOL *s)
 	}
 
 	/* treat the overlay start */
-	otype = s->as.u.v_over.type;
-	if (otype == V_OVER_SS
-	    || otype == V_OVER_SD) {
+	if (s->as.u.v_over.type == V_OVER_S) {
 		voice_over = s;
 		return;
 	}
@@ -413,10 +409,6 @@ static void get_over(struct SYMBOL *s)
 			p_voice2->next->prev = p_voice2;
 		p_voice->next = p_voice2;
 		p_voice2->prev = p_voice;
-#if 0
-		if (otype == V_OVER_S)
-			p_voice2->stem = 1;
-#endif
 	}
 
 	/* search the start of sequence */
@@ -460,7 +452,7 @@ static void get_over(struct SYMBOL *s)
 			s2->as.linenum = linenum;
 			s2->as.u.bar.type = s1->as.u.bar.type;
 			s2->time = ctime;
-			s2->seq = SQ_BAR;
+			s2->seq = seq_tb[BAR];
 		}
 	}
 	while (ctime < s->time) {
@@ -476,7 +468,7 @@ static void get_over(struct SYMBOL *s)
 			s2->as.u.note.invis = 1;
 			s2->len = s2->as.u.note.lens[0] = s1->time - ctime;
 			s2->time = ctime;
-			s2->seq = SQ_NOTE;
+			s2->seq = seq_tb[NOTE];
 			ctime = s1->time;
 		}
 		while (s1->type == BAR) {
@@ -484,7 +476,7 @@ static void get_over(struct SYMBOL *s)
 			s2->as.linenum = linenum;
 			s2->as.u.bar.type = s1->as.u.bar.type;
 			s2->time = ctime;
-			s2->seq = SQ_BAR;
+			s2->seq = seq_tb[BAR];
 			if ((s1 = s1->next) == 0)
 				break;
 		}
@@ -575,6 +567,11 @@ static void get_staves(struct SYMBOL *s)
 			memcpy(&staff_tb[staff].clef, &p_voice->clef,
 			       sizeof staff_tb[0].clef);
 		}
+		if ((flags & (OPEN_PARENTH | CLOSE_PARENTH))
+				== (OPEN_PARENTH | CLOSE_PARENTH))
+			flags &= ~(OPEN_PARENTH | CLOSE_PARENTH);
+		if (flags == 0)
+			continue;
 		if (flags & STOP_BAR)
 			staff_tb[staff].stop_bar = 1;
 		if (flags & OPEN_BRACKET)
@@ -602,9 +599,6 @@ static void get_staves(struct SYMBOL *s)
 				p_voice->staff = p_voice->cstaff = staff;
 				break;
 			case 3:				/* {a b c d} */
-				if (flags & OPEN_PARENTH
-				    && (p_staff[2].flags & OPEN_PARENTH))
-					break;
 				if (flags & OPEN_PARENTH
 				    || (p_staff[1].flags & OPEN_PARENTH)
 				    || (p_staff[2].flags & OPEN_PARENTH))
@@ -616,8 +610,6 @@ static void get_staves(struct SYMBOL *s)
 				p_staff[2].flags |= OPEN_PARENTH;
 				p_staff[3].flags |= CLOSE_PARENTH;
 				break;
-			default:
-				goto err;
 			}
 			staff_tb[staff].brace = 1;
 		}
@@ -639,8 +631,8 @@ static void get_staves(struct SYMBOL *s)
 				staff_tb[staff].bracket_end = 1;
 			if (p_staff->flags & CLOSE_BRACE) {
 				staff_tb[staff].brace_end = 1;
-
-				/* the lower voice must be main */
+#if 0
+				/* the lowest voice must be main */
 				if (p_voice->second) {
 					p_voice->second = 0;
 					do {
@@ -648,19 +640,24 @@ static void get_staves(struct SYMBOL *s)
 					} while (p_voice->second);
 					p_voice->second = 1;
 				}
+#endif
 			}
 		}
 	}
+	if (staff < 0)
+		staff = 0;
 	nstaff = staff;
 	return;
 
 	/* when error, let one voice per staff */
 err:
-	error(1, s, "%%%%staves error");
+	error(1, s, "Cannot handle this %%%%staves");
 	for (p_voice = voice_tb, staff = 0;
 	     p_voice != 0;
 	     p_voice = p_voice->next, staff++)
 		p_voice->staff = p_voice->cstaff = staff;
+	if (staff < 0)
+		staff = 0;
 	nstaff = staff;
 }
 
@@ -677,8 +674,8 @@ static void voice_init(void)
 		p_voice->clone = -1;
 		p_voice->bar_start = 0;
 		p_voice->time = 0;
-		p_voice->slur_start = 0;
-		p_voice->lystart = 0;
+		p_voice->slur_st = 0;
+		p_voice->hy_st = 0;
 		p_voice->tie = 0;
 		p_voice->rtie = 0;
 	}
@@ -749,7 +746,7 @@ static void get_info(struct SYMBOL *s,
 		if (s->as.state != ABC_S_HEAD)
 			return;
 		tunenum++;
-		PUT2("\n%% --- %s (%s) ---\n",
+		PUT2("%% --- %s (%s) ---\n",
 		     info.xref, &info.title->as.text[2]);
 		if (!epsf)
 			bskip(cfmt.topspace);
@@ -782,6 +779,12 @@ static void get_info(struct SYMBOL *s,
 			curvoice = first_voice;
 			sym_link(s);
 			s->type = PART;
+			if (p_voice->time != curvoice->time) {
+				error(1, s, "Misplaced P:");
+				if (p_voice->time > curvoice->time)
+					curvoice->time = p_voice->time;
+				else	p_voice->time = curvoice->time;
+			}
 			curvoice = p_voice;
 			break;
 		    }
@@ -816,22 +819,14 @@ static void get_info(struct SYMBOL *s,
 		return;
 	case 'T':
 		switch (s->as.state) {
-		case ABC_S_GLOBAL:	/* new tune */
-			if (!epsf)
-				write_buffer();
-			dfmt = cfmt;
-			memcpy(&info, &default_info, sizeof info);
-			inf = &info;
-			inf->xref = p;
-			memcpy(&deco_tune, &deco_glob, sizeof deco_tune);
-			voice_init();
-			inf->title = s;
-			break;
+		case ABC_S_GLOBAL:	/* T: without X: */
+			goto newtune;
 		case ABC_S_HEAD:
 			add_info(s, &inf->title);
 			break;
 		default:
 			output_music();
+			buffer_eob();
 			write_title(s);
 			bskip(cfmt.musicspace + 0.2 * CM);
 			voice_init();
@@ -846,7 +841,7 @@ static void get_info(struct SYMBOL *s,
 		deco = s->as.state == ABC_S_GLOBAL ? deco_glob : deco_tune;
 		deco[s->as.u.user.symbol] = deco_intern(s->as.u.user.value);
 		return;
-	}
+	    }
 	case 'u':
 		return;
 	case 'V':
@@ -855,8 +850,9 @@ static void get_info(struct SYMBOL *s,
 	case 'w':
 		if (cfmt.musiconly)
 			return;
-		if (s->as.state != ABC_S_TUNE
-		    || lyric_start == 0)
+		if (s->as.state != ABC_S_TUNE)
+			return;
+		if (lyric_start == 0)
 			break;
 		if ((p = get_lyric(p)) != 0)
 			error(1, s, "%s", p);
@@ -867,11 +863,14 @@ static void get_info(struct SYMBOL *s,
 		add_info(s, &words);
 		return;
 	case 'X':
+newtune:
 		if (!epsf)
 			write_buffer();	/* flush stuff left from %% lines */
 		dfmt = cfmt;		/* save the format at start of tune */
 		memcpy(&info, &default_info, sizeof info);
 		info.xref = p;
+		if (info_type == 'T')
+			info.title = s;
 		memcpy(&deco_tune, &deco_glob, sizeof deco_tune);
 		voice_init();		/* initialize all the voices */
 		return;
@@ -951,8 +950,11 @@ static void get_bar(struct SYMBOL *s)
 		s->as.text = 0;
 	}
 
-	/* remove the invisible repeat bars of the 1st voice */
-	if (curvoice == first_voice && bar_type == B_INVIS) {
+	/* remove the invisible repeat bars when no shift needed */
+	if (bar_type == B_INVIS
+	    && (curvoice == first_voice
+		|| curvoice->second
+		|| staff_tb[curvoice->staff - 1].stop_bar)) {
 		s2 = curvoice->last_symbol;
 		if (s2 != 0 && s2->type == BAR
 		    && s2->as.text == 0) {
@@ -1002,7 +1004,8 @@ static void get_bar(struct SYMBOL *s)
 	/* the bar must be before a key signature */
 /*fixme: and also before a time signature*/
 	if ((s2 = s->prev) != 0
-	    && s2->type == KEYSIG) {
+	    && s2->type == KEYSIG
+	    && (s2->prev == 0 || s2->prev->type != BAR)) {
 		curvoice->last_symbol = s2;
 		s2->next = 0;
 		s2->prev->next = s;
@@ -1025,6 +1028,7 @@ void do_tune(struct abctune *t,
 	     int header_only)
 {
 	struct abcsym *as;
+	int i;
 
 	/* initialize */
 	memset(voice_tb, 0, sizeof voice_tb);
@@ -1035,19 +1039,18 @@ void do_tune(struct abctune *t,
 	nstaff = 0;
 	memset(staff_tb, 0, sizeof staff_tb);
 	staves_found = 0;
-	{
-		int i;
 
-		for (i = MAXVOICE; --i >= 0; ) {
-			voice_tb[i].clef.line = 2;	/* treble clef on 2nd line */
-			voice_tb[i].meter.nmeter = 1;
-			voice_tb[i].meter.wmeasure = BASE_LEN;
-			voice_tb[i].meter.meter[0].top[0] = '4';
-			voice_tb[i].meter.meter[0].bot[0] = '4';
-		}
-		for (i = MAXSTAFF; --i >= 0; )
-			staff_tb[i].clef.line = 2;
+	for (i = MAXVOICE; --i >= 0; ) {
+		voice_tb[i].clef.line = 2;	/* treble clef on 2nd line */
+		voice_tb[i].meter.nmeter = 1;
+		voice_tb[i].meter.wmeasure = BASE_LEN;
+		voice_tb[i].meter.meter[0].top[0] = '4';
+		voice_tb[i].meter.meter[0].bot[0] = '4';
+		voice_tb[i].wmeasure = BASE_LEN;
 	}
+	for (i = MAXSTAFF; --i >= 0; )
+		staff_tb[i].clef.line = 2;
+
 	curvoice = first_voice = voice_tb;
 	use_buffer = !cfmt.splittune;
 
@@ -1094,7 +1097,7 @@ void do_tune(struct abctune *t,
 				p = &as->text[0];
 			}
 			break;
-		}
+		    }
 		case ABC_T_PSCOM:
 			as = process_pscomment(as);
 			break;
@@ -1118,7 +1121,7 @@ void do_tune(struct abctune *t,
 		case ABC_T_MREP: {
 			int len;
 
-			len = curvoice->meter.wmeasure * as->u.bar.len;
+			len = curvoice->wmeasure * as->u.bar.len;
 			if (as->type == ABC_T_MREP
 			    && as->u.bar.len > 1) {
 				struct SYMBOL *s2;
@@ -1142,6 +1145,7 @@ void do_tune(struct abctune *t,
 			sym_link(s);
 			s->type = as->type == ABC_T_MREST ? MREST : MREP;
 			s->len = len;
+			s->gcf = cfmt.gcf;
 			break;
 		    }
 		case ABC_T_V_OVER:
@@ -1160,6 +1164,7 @@ void do_tune(struct abctune *t,
 	}
 
 	output_music();
+	buffer_eob();
 	if (words != 0)
 		put_words(words);
 	if (cfmt.writehistory)
@@ -1256,7 +1261,7 @@ static void get_key(struct SYMBOL *s)
 	struct VOICE_S *p_voice;
 	int i;
 
-	if (s->as.u.key.empty)
+	if (s->as.u.key.empty == 1)
 		return;			/* clef only */
 	switch (s->as.state) {
 	case ABC_S_HEAD: {
@@ -1313,9 +1318,11 @@ static void get_meter(struct SYMBOL *s)
 
 		for (i = MAXVOICE, p_voice = voice_tb;
 		     --i >= 0;
-		     p_voice++)
+		     p_voice++) {
 			memcpy(&p_voice->meter, &s->as.u.meter,
-			       sizeof curvoice->meter);
+			       sizeof p_voice->meter);
+			p_voice->wmeasure = s->as.u.meter.wmeasure;
+		}
 		break;
 	    }
 	case ABC_S_TUNE:
@@ -1323,9 +1330,11 @@ static void get_meter(struct SYMBOL *s)
 		if (curvoice->sym == 0) {
 			memcpy(&curvoice->meter, &s->as.u.meter,
 			       sizeof curvoice->meter);
+			curvoice->wmeasure = s->as.u.meter.wmeasure;
 			reset_gen();	/* (display the time signature) */
 			break;
 		}
+		curvoice->wmeasure = s->as.u.meter.wmeasure;
 		if (s->as.u.meter.nmeter == 0)
 			break;		/* M:none */
 		sym_link(s);
@@ -1386,6 +1395,9 @@ static void get_voice(struct SYMBOL *s)
 	if (s->as.u.voice.gstem != 0)
 		p_voice->gstem = s->as.u.voice.gstem != 2
 			? s->as.u.voice.gstem : 0;
+	if (s->as.u.voice.dyn != 0)
+		p_voice->dyn = s->as.u.voice.dyn != 2
+			? s->as.u.voice.dyn : 0;
 }
 
 /* -- note or rest -- */
@@ -1437,26 +1449,17 @@ static void get_note(struct SYMBOL *s)
 	s->type = s->as.type == ABC_T_NOTE ? NOTE : REST;
 
 	/* convert the decorations */
-	if (s->as.u.note.dc.n > 0) {
+	if (s->as.u.note.dc.n > 0)
 		deco_cnv(&s->as.u.note.dc, s);
-		for (i = 0; i <= m; i++) {
-			if (s->as.u.note.decs[i] != 0) {
-				s->as.u.note.dc.n = s->as.u.note.decs[i] >> 3;
-				break;
-			}
-		}
-	}
 
 	/* change the figure of whole measure rests */
-	if (s->type == REST && s->len == curvoice->meter.wmeasure) {
+	if (s->type == REST && s->len == curvoice->wmeasure) {
 		if (s->len < BASE_LEN * 2)
 			s->as.u.note.lens[0] = BASE_LEN;
 		else if (s->len < BASE_LEN * 4)
 			s->as.u.note.lens[0] = BASE_LEN * 2;
 		else	s->as.u.note.lens[0] = BASE_LEN * 4;
 	}
-	if (s->len >= BASE_LEN)
-		s->as.u.note.stemless = 1;
 
 	/* sort by pitch the notes of the chord (lowest first) */
 	for (;;) {
@@ -1516,6 +1519,8 @@ static void get_note(struct SYMBOL *s)
 			}
 		}
 	}
+	if (s->nflags <= -2)
+		s->as.u.note.stemless = 1;
 
 	for (i = 0; i <= m; i++) {
 		if (s->as.u.note.sl1[i] != 0)
@@ -1537,6 +1542,25 @@ static void get_note(struct SYMBOL *s)
 		gchord_adjust(s);
 }
 
+/* -- treat a postscript definition -- */
+static void ps_def(struct SYMBOL *s,
+		   unsigned char *p)
+{
+	if (s->as.state == ABC_S_TUNE
+	    || s->as.state == ABC_S_EMBED) {
+		sym_link(s);
+		s->type = FMTCHG;
+		s->u = PSSEQ;
+		s->as.text = p;
+		return;
+	}
+	if (file_initialized) {
+		PUT1("%s\n", p);
+		return;
+	}
+	interpret_format_line("postscript", p);
+}
+
 /* -- process a pseudo-comment (%% or I:) -- */
 static struct abcsym *process_pscomment(struct abcsym *as)
 {
@@ -1546,46 +1570,72 @@ static struct abcsym *process_pscomment(struct abcsym *as)
 	struct SYMBOL *s = (struct SYMBOL *) as;
 
 	p = as->text + 2;		/* skip '%%' */
-	if (strncasecmp(p, "fmt ", 4) == 0)
+	while (isspace(*p))
+		p++;
+	if (strncasecmp(p, "fmt ", 4) == 0) {
 		p += 4;			/* skip 'fmt' */
-
+		while (isspace(*p))
+			p++;
+	}
 	p = get_str(w, p, sizeof w);
 	switch (w[0]) {
 	case 'b':
+		if (strcmp(w, "beginps") == 0) {
+			for (;;) {
+				if (as->next == 0)
+					return as;
+				as = as->next;
+				p = as->text;
+				if (*p == '%' && p[1] == '%') {
+					p += 2;
+					while (isspace(*p))
+						p++;
+					if (strncasecmp(p, "fmt ", 4) == 0) {
+						p += 4;
+						while (isspace(*p))
+							p++;
+					}
+					if (strncmp(p, "endps", 5) == 0)
+						return as;
+				}
+				if (*p == '%')
+					continue;	/* skip comment lines */
+				s = (struct SYMBOL *) as;
+				ps_def(s, p);
+			}
+			/* not reached */
+		}
 		if (strcmp(w, "begintext") == 0) {
 			int job;
 
 			if (epsf && as->state != ABC_S_HEAD)
 				return as;
-			job = OBEYLINES;
-			if (*p == '\0'
-			    || strncmp(p, "obeylines", 9) == 0)
-				;
-			else if (strncmp(p, "align", 5) == 0
-				 || strncmp(p, "justify", 7) == 0)
-				job = T_JUSTIFY;
-			else if (strncmp(p, "skip", 4) == 0)
-				job = SKIP;
-			else if (strncmp(p, "ragged", 6) == 0
-				 || strncmp(p, "fill", 4) == 0)
-				job = T_FILL;
-			else	{
-				error(1, s,
-				      "Bad argument for begintext: %s", p);
+			if (*p == '\0')
+				job = cfmt.textoption;
+			else {
+				job = get_textopt(p);
+				if (job < 0) {
+					error(1, s,
+					      "Bad argument for begintext: %s", p);
+					job = OBEYLINES;
+				}
 			}
 			output_music();
+			buffer_eob();
 			for (;;) {
-				if (as->next == 0) {
-					error(1, (struct SYMBOL *) as,
-					      "EOF found while scanning %%%%begintext");
+				if (as->next == 0)
 					return as;
-				}
 				as = as->next;
 				p = as->text;
 				if (*p == '%' && p[1] == '%') {
 					p += 2;
-					if (strncasecmp(p, "fmt ", 4) == 0)
+					while (isspace(*p))
+						p++;
+					if (strncasecmp(p, "fmt ", 4) == 0) {
 						p += 4;
+						while (isspace(*p))
+							p++;
+					}
 					if (strncmp(p, "endtext", 7) == 0) {
 						if (job != SKIP)
 							write_text_block(job, as->state);
@@ -1605,6 +1655,7 @@ static struct abcsym *process_pscomment(struct abcsym *as)
 			char line[BSIZE];
 
 			output_music();
+			buffer_eob();
 			if ((epsf = fopen(p, "r")) == 0) {
 				error(1, s,
 				      "No such file: %s", p);
@@ -1644,6 +1695,7 @@ static struct abcsym *process_pscomment(struct abcsym *as)
 
 			output_music();
 			if (strncmp(p, "start", 5) == 0) {
+				buffer_eob();
 				multicol_max = multicol_start = get_bposy();
 				lmarg = cfmt.leftmargin;
 				rmarg = cfmt.rightmargin;
@@ -1688,23 +1740,14 @@ static struct abcsym *process_pscomment(struct abcsym *as)
 			use_buffer = 0;
 			if (isdigit(*p))
 				pagenum = atoi(p);
-			write_pagebreak();
+			close_page();
 			return as;
 		}
 		break;
 	case 'p':
 		if (strcmp(w, "postscript") == 0) {
-			if (as->state == ABC_S_TUNE
-			    || as->state == ABC_S_EMBED) {
-				sym_link(s);
-				s->type = FMTCHG;
-				s->u = PSSEQ;
-				return as;
-			}
-			if (file_initialized) {
-				PUT1("%s\n", p);
-				return as;
-			}
+			ps_def(s, p);
+			return as;
 		}
 		break;
 	case 'r':
@@ -1718,7 +1761,7 @@ static struct abcsym *process_pscomment(struct abcsym *as)
 	case 's':
 		if (strcmp(w, "setbarnb") == 0) {
 			if (as->state == ABC_S_TUNE
-			    && as->state == ABC_S_EMBED) {
+			    || as->state == ABC_S_EMBED) {
 				bar_number = atoi(p);
 				return as;
 			}
@@ -1769,9 +1812,10 @@ static struct abcsym *process_pscomment(struct abcsym *as)
 			    && as->state != ABC_S_EMBED)
 				return as;
 			staff = atoi(p) - 1;
-			if ((unsigned) staff > nstaff)
-				error(1, s,
-				      "Bad staff in %%%%staff");
+			if ((unsigned) staff > nstaff) {
+				error(1, s, "Bad staff in %%%%staff");
+				return as;
+			}
 			curvoice->floating = 0;
 			curvoice->cstaff = staff;
 			return as;
@@ -1791,6 +1835,7 @@ static struct abcsym *process_pscomment(struct abcsym *as)
 		if (strcmp(w, "staves") == 0) {
 			if (as->state == ABC_S_TUNE) {
 				output_music();
+				buffer_eob();
 				voice_init();
 			}
 			get_staves((struct SYMBOL *) as);
@@ -1808,6 +1853,7 @@ static struct abcsym *process_pscomment(struct abcsym *as)
 				return as;
 			job = w[0] == 't' ? OBEYLINES : OBEYCENTER;
 			output_music();
+			buffer_eob();
 			add_to_text_block(p, job);
 			write_text_block(job, as->state);
 			return as;
