@@ -31,10 +31,16 @@
 #include "abcparse.h"
 #include "abc2ps.h" 
 
+unsigned char tex_buf[512];	/* result of tex_str() */
 struct FONTSPEC *font_init;	/* font in case of page break */
-static int curfont;		/* current string font */
+int strcf;			/* current string font */
 
-static float twidth = -1;	/* text width for %%begintext..%%endtext */
+static char *strop;		/* current string output operation */
+static float strlw;		/* line width */
+static float strtw = -1;	/* current text width */
+static int strns;		/* number of spaces (justify) */
+static int strfc;		/* force a font change before bext */
+static int strtx;		/* PostScript text outputing */
 
 /* width of characters according to the encoding */
 /* these are the widths for Times-Roman, extracted from the 'a2ps' package */
@@ -225,8 +231,9 @@ static struct SYMBOL *t;
 	}
 	fprintf(stderr, sev == 0 ? "Warning " : "Error ");
 	if (s != 0)
-		fprintf(stderr, "in line %d.%d: ",
+		fprintf(stderr, "in line %d.%d",
 			s->as.linenum, s->as.colnum);
+	fprintf(stderr, ": ");
 	va_start(args, fmt);
 	vfprintf(stderr, fmt, args);
 	va_end(args);
@@ -272,8 +279,6 @@ void cap_str(unsigned char *p)
 	}
 }
 
-/*  miscellaneous subroutines  */
-
 /* -- return the character width -- */
 float cwid(unsigned char c)
 {
@@ -292,30 +297,30 @@ float cwid(unsigned char c)
 /* Puts \ in front of ( and ) in case brackets are not balanced,
  * interprets all ISOLatin1..6 escape sequences as defined in rfc1345.
  * Returns an estimate of the string width. */
-float tex_str(char *d,
-	      char *s,
-	      int maxlen)
+float tex_str(char *s)
 {
-	float w;
-	char c1, c2;
-	char *p_enc, *p;
-	int i;
+	unsigned char *d, c1, c2, *p_enc, *p;
+	int maxlen, i;
+	float w, swfac;
 
 	w = 0;
-	maxlen--;		/* have room for EOS */
-	i = font_enc[cfmt.font_tb[0].fnum];
+	d = tex_buf;
+	maxlen = sizeof tex_buf - 1;		/* have room for EOS */
+	if ((i = strcf) < 0)
+		i = 0;
+	swfac = cfmt.font_tb[i].swfac;
+	i = font_enc[cfmt.font_tb[i].fnum];
 	if ((unsigned) i >= sizeof esc_tb / sizeof esc_tb[0])
 		i = 0;
 	p_enc = esc_tb[i];
-	while ((c1 = *s) != '\0') {
+	while ((c1 = *s++) != '\0') {
 		switch (c1) {
 		case '\\':			/* backslash sequences */
-			s++;
-			if ((c1 = *s) == '\0')
+			if ((c1 = *s++) == '\0')
 				break;
 			if (c1 == ' ')
 				goto addchar1;
-			if (c1 == '\\' || (c2 = s[1]) == '\0') {
+			if (c1 == '\\' || (c2 = *s) == '\0') {
 				if (--maxlen <= 0)
 					break;
 				*d++ = '\\';
@@ -324,15 +329,15 @@ float tex_str(char *d,
 			/* treat escape with octal value */
 			if ((unsigned) (c1 - '0') <= 3
 			    && (unsigned) (c2 - '0') <= 7
-			    && (unsigned) (s[2] - '0') <= 7) {
+			    && (unsigned) (s[1] - '0') <= 7) {
 				if ((maxlen -= 4) <= 0)
 					break;
 				*d++ = '\\';
 				*d++ = c1;
 				*d++ = c2;
-				*d++ = s[2];
-				c1 = ((c1 - '0') << 6) + ((c2 - '0') << 3) + s[2] - '0';
-				w += cwid(c1);
+				*d++ = s[1];
+				c1 = ((c1 - '0') << 6) + ((c2 - '0') << 3) + s[1] - '0';
+				w += cwid(c1) * swfac;
 				s += 2;
 				break;
 			}
@@ -364,24 +369,26 @@ float tex_str(char *d,
 				}
 			}
 			goto addchar1;
+#if 0
 		case '{':
 		case '}':
 			break;
+#endif
 		case '$':
-			if (isdigit(s[1])
-			    && (unsigned) (s[1] - '0')
-				 < sizeof cfmt.font_tb / sizeof cfmt.font_tb[0]) {
-				i = cfmt.font_tb[s[1] - '0'].fnum;
+			if (isdigit(*s)
+			    && (unsigned) (*s - '0') < DFONT_MIN) {
+				i = *s - '0';
+				swfac = cfmt.font_tb[i].swfac;
+				i = cfmt.font_tb[i].fnum;
 				i = font_enc[i];
 				if ((unsigned) i >= sizeof esc_tb / sizeof esc_tb[0])
 					i = 0;
 				p_enc = esc_tb[i];
-			}
-			if (s[1] == '$') {
+			} else if (*s == '$') {
 				if (--maxlen <= 0)
 					break;
 				*d++ = c1;
-				w += cwid(c1);
+				w += cwid(c1) * swfac;
 				s++;
 			}
 			goto addchar1;
@@ -396,9 +403,9 @@ float tex_str(char *d,
 			if (--maxlen <= 0)
 				break;
 			*d++ = c1;
-			w += cwid(c1);
+			w += cwid(c1) * swfac;
+			break;
 		}
-		s++;
 	}
 	*d = '\0';
 	return w;
@@ -408,112 +415,124 @@ float tex_str(char *d,
 void str_font(struct FONTSPEC *font)
 {
 	memcpy(&cfmt.font_tb[0], font, sizeof cfmt.font_tb[0]);
-	curfont = -1;
+	strcf = -1;
 }
 
-/* output a string, handling the font changes -- */
-float str_out(unsigned char *p, int action)
+/* -- output one string -- */
+static void str_ft_out1(unsigned char *p, int l)
+{
+	if (strfc) {
+		if (strtx) {
+			PUT1(")%s ", strop);
+			strtx = 0;
+		}
+		set_font(&cfmt.font_tb[strcf]);
+		strfc = 0;
+	}
+	if (!strtx) {
+		PUT0("(");
+		strtx = 1;
+	}
+	PUT2("%.*s", l, p);
+}
+
+/* -- output a string and the font changes -- */
+static void str_ft_out(unsigned char *p, int end)
 {
 	unsigned char *q;
-	int fontdef;
-	float swfac, w;
-	struct FONTSPEC *f;
 
-	fontdef = 0;
-	if (curfont < 0) {		/* first call */
-		curfont = 0;
-		fontdef = 1;
-	}
-	f = &cfmt.font_tb[curfont];
-	swfac = f->size * f->swfac;
-
-	/* compute the string width */
-	w = 0;
 	q = p;
 	while (*p != '\0') {
-		if (*p == '$' && isdigit(p[1])
-		    && (unsigned) (p[1] - '0')
-			 < sizeof cfmt.font_tb / sizeof cfmt.font_tb[0]) {
-			f = &cfmt.font_tb[p[1] - '0'];
-			swfac = f->size * f->swfac;
-			p += 2;
-			continue;
-		}
-		if (*p == '$' && p[1] == '$')
-			p++;
-		else if (*p == '\\') {
-			p++;
-			if (isdigit(*p))	/* '\ooo' */
-				p += 2;		/*fixme: should get the char*/
-		}
-		w += cwid(*p++) * swfac;
-	}
-
-	/* center / right align */
-	if (action == A_CENTER)
-		PUT1("-%.1f 0 RM ", 0.5 * w);
-	else if (action == A_RIGHT)
-		PUT1("-%.1f 0 RM ", w);
-
-	/* output the string */
-	p = q;
-	while (*p != '\0') {
-		if (*p == '$' && isdigit(p[1])
-		    && (unsigned) (p[1] - '0')
-			 < sizeof cfmt.font_tb / sizeof cfmt.font_tb[0]) {
-			if (p > q) {
-				if (fontdef) {
-					set_font(f);
-					fontdef = 0;
+		if (*p == '$') {
+			if (isdigit(p[1])
+			    && (unsigned) (p[1] - '0') < DFONT_MIN) {
+				if (p > q)
+					str_ft_out1(q, p - q);
+				if (strcf != p[1] - '0') {
+					strcf = p[1] - '0';
+					strfc = 1;
 				}
-				PUT2("(%.*s) show ", p - q, q);
+				p += 2;
+				q = p;
+				continue;
 			}
-			if (curfont != p[1] - '0') {
-				curfont = p[1] - '0';
-				f = &cfmt.font_tb[curfont];
-				fontdef = 1;
+			if (p[1] == '$') {
+				str_ft_out1(q, p - q);
+				q = ++p;
 			}
-			p += 2;
-			q = p;
-			continue;
-		}
-		if (*p == '$' && *p == '$') {
-			p++;
-			if (fontdef) {
-				set_font(f);
-				fontdef = 0;
-			}
-			PUT2("(%.*s) show ", p - q, q);
-			q = p + 1;
 		}
 		p++;
 	}
-	if (p > q) {
-		if (fontdef) {
-			set_font(f);
-			fontdef = 0;
-		}
-		PUT2("(%.*s) show ", p - q, q);
+	if (p > q)
+		str_ft_out1(q, p - q);
+	if (end && strtx) {
+		PUT1(")%s ", strop);
+		strtx = 0;
 	}
-	return w;
+}
+
+/* -- output a string, handling the font changes -- */
+void str_out(unsigned char *p, int action)
+{
+	if (strcf < 0) {		/* first call */
+		strcf = 0;
+		strfc = 1;
+	}
+
+	/* special case when font change at start of text */
+	if (*p == '$' && isdigit(p[1])
+	    && (unsigned) (p[1] - '0') < DFONT_MIN) {
+		if (strcf != p[1] - '0') {
+			strcf = p[1] - '0';
+			strfc = 1;
+		}
+		p += 2;
+	}
+
+	/* direct output if no font change */
+	if (strchr(p, '$') == 0) {
+		char *op;
+
+		if (strfc) {
+			set_font(&cfmt.font_tb[strcf]);
+			strfc = 0;
+		}
+		if (action == A_CENTER)
+			op = "c";
+		else if (action == A_RIGHT)
+			op = "r";
+		else	op = "";
+		PUT2("(%s)show%s ", p, op);
+		return;
+	}
+
+	/* if not left aligned, build a PS function */
+	if (action == A_LEFT)
+		strop = "show";
+	else {
+		PUT0("/str{");
+		strfc = 1;
+		strop = "strop";
+	}
+
+	str_ft_out(p, 1);		/* output the string */
+
+	/* if not left aligned, call the PS function */
+	if (action == A_LEFT)
+		return;
+	PUT0("}def\n"
+		"/strop/strw load def/w 0 def str w ");
+	if (action == A_CENTER)
+		PUT0("0.5 mul ");
+	PUT0("neg 0 RM/strop/show load def str ");
 }
 
 /* -- output a string with TeX translation -- */
 void put_str(unsigned char *str, int action)
 {
-	unsigned char buf[800];
-
-	tex_str(buf, str, sizeof buf);
-	str_out(buf, action);
-}
-
-/* -- output a TeX string -- */
-static void put_texstr(char *str)
-{
-	char s[800];
-
-	tex_str(s, str, sizeof s);
-	PUT1("%s", s);
+	tex_str(str);
+	str_out(tex_buf, action);
+	PUT0("\n");
 }
 
 /* -- output a header information -- */
@@ -534,129 +553,189 @@ static void put_inf2r(struct SYMBOL *s1,
 		      struct SYMBOL *s2,
 		      int action)
 {
-	char *p, *q;
-	char buf[800];
+	unsigned char buf[512], *p, *q;
 
-	if (s1 != 0) {
+	if (s1 == 0) {
+		s1 = s2;
+		s2 = 0;
+	}
+	if (s2 == 0) {
 		p = s1->as.text;
 		if (p[1] == ':')
 			p += 2;
 		while (isspace(*p))
 			p++;
-		tex_str(buf, p, sizeof buf);
-		q = &buf[strlen(buf)];
-	} else	q = buf;
-	if (s2 != 0) {
-		if (s1 != 0)
-			*q++ = ' ';
-		*q++ = '\\';
-		*q++ = '(';
-		p = s2->as.text;
+	} else {
+		buf[sizeof buf - 1] = '\0';
+		p = s1->as.text;
 		if (p[1] == ':')
 			p += 2;
 		while (isspace(*p))
 			p++;
-		tex_str(q, p, sizeof buf - (q - buf) - 2);
-		q += strlen(q);
-		*q++ = '\\';
-		*q++ = ')';
+		strncpy(buf, p, sizeof buf - 1);
+		q = buf + strlen(buf);
+		if (q < buf + sizeof buf - 4) {
+			*q++ = ' ';
+			*q++ = '(';
+			p = s2->as.text;
+			if (p[1] == ':')
+				p += 2;
+			while (isspace(*p))
+				p++;
+			strncpy(q, p, buf + sizeof buf - 2 - q);
+			q += strlen(q);
+			*q++ = ')';
+			*q = '\0';
+		}
+		p = buf;
 	}
-	*q = '\0';
-	str_out(buf, action);
+	put_str(p, action);
 }
 
 /* -- memorize the current font in case of page break -- */
 static void set_font_init(void)
 {
-	if (curfont < 0)
+	if (strcf < 0)
 		return;
-	font_init = &cfmt.font_tb[curfont];
+	font_init = &cfmt.font_tb[strcf];
 }
 
-/* -- add_to_text_block -- */
+/* -- add text to a block -- */
 void add_to_text_block(char *s, int job)
 {
-	float lw;
-	char buf[256];
+	float baseskip, lw;
+	unsigned char *p, sep;
+	struct FONTSPEC *f;
 
 	/* if first line, set the fonts */
-	if (twidth < 0) {
+	if (strtw < 0) {
 		font_init = &cfmt.textfont;
-		set_font(&cfmt.textfont);
 		str_font(&cfmt.textfont);
-		curfont = 0;
+		strlw = ((cfmt.landscape ? cfmt.pageheight : cfmt.pagewidth)
+			- cfmt.leftmargin - cfmt.rightmargin) / cfmt.scale;
 	}
 
-	lw = tex_str(buf, s, sizeof buf);
+	if (strcf >= 0)
+		f = &cfmt.font_tb[strcf];
+	else	f = &cfmt.font_tb[0];
+	baseskip = f->size * cfmt.lineskipfac;
 
 	/* follow lines */
 	if (job == OBEYLINES || job == OBEYCENTER) {
-		bskip(cfmt.textfont.size * cfmt.lineskipfac);
-		if (buf[0] != '\0') {
+		if (*s != '\0') {
+			bskip(baseskip);
 			if (job == OBEYLINES) {
 				PUT0("0 0 M ");
-				str_out(buf, A_LEFT);
+				put_str(s, A_LEFT);
 			} else	{
-				PUT1("%.1f 0 M ",
-				     ((cfmt.landscape ? cfmt.pageheight : cfmt.pagewidth)
-					- cfmt.leftmargin - cfmt.rightmargin)
-					* 0.5 / cfmt.scale);
-				str_out(buf, A_CENTER);
+				PUT1("%.1f 0 M ", strlw * 0.5);
+				put_str(s, A_CENTER);
 			}
-			PUT0("\n");
-		}
+		} else	bskip(baseskip * 0.5);
 		buffer_eob();
-		twidth = 0;
+		strtw = 0;
 		return;
 	}
 
 	/* fill or justify lines */
-	if (twidth < 0) {		/* if first line */
-		float baseskip;
-
-		baseskip = cfmt.textfont.size * cfmt.lineskipfac;
-		PUT1("/LF {0 %.1f RM}!\n", -baseskip);
+	if (strtw < 0) {		/* if first line */
+		strcf = 0;
+		strfc = 1;
 		bskip(baseskip);
-		PUT0("0 0 M (");
-		twidth = 0;
+		PUT0("0 0 M ");
+		if (job == T_FILL)
+			strop = "show";
+		else {
+			PUT0("/str{");
+			strop = "strop";
+		}
+		strns = 0;
+		strtw = 0;
+	}
+
+	if (*s == '\0') {			/* empty line */
+		if (strtx) {
+			PUT1(")%s", strop);
+			strtx = 0;
+		}
+		if (job == T_JUSTIFY)
+			PUT0("}def\n"
+				"/strop/show load def str\n");
+		else	PUT0("\n");
+		font_init = f;
+		bskip(f->size * cfmt.lineskipfac * 1.5);
+		buffer_eob();
+		PUT0("0 0 M ");
+		if (job == T_JUSTIFY) {
+			PUT0("/str{");
+			strfc = 1;
+		}
+		strns = 0;
+		strtw = 0;
+		return;
+	}
+
+	p = s;
+	for (;;) {
+		while (*p != ' ' && *p != '\0')
+			p++;
+		sep = *p;
+		*p = '\0';
+		lw = tex_str(s);
+		if (strtw + lw > strlw) {
+			if (strtx) {
+				PUT1(")%s ", strop);
+				strtx = 0;
+			}
+			if (job == T_JUSTIFY) {
+				if (strns == 0)
+					strns = 1;
+				PUT2("}def\n"
+					"/strop/strw load def/w 0 def str"
+					"/w %.1f w sub %d div def"
+					"/strop/jshow load def str ",
+					strlw, strns);
+				strns = 0;
+			}
+			bskip(cfmt.font_tb[strcf].size * cfmt.lineskipfac);
+			PUT0("0 0 M ");
+			if (job == T_JUSTIFY) {
+				PUT0("/str{");
+				strfc = 1;
+			}
+			strtw = 0;
+		}
+		if (strtw != 0) {
+			str_ft_out(" ", 0);
+			strtw += cwid(' ') * cfmt.font_tb[strcf].swfac;
+			strns++;
+		}
+		str_ft_out(tex_buf, 0);
+		strtw += lw;
+		*p = sep;
+		while (*p == ' ')
+			p++;
+		if (*p == '\0')
+			break;
+		s = p;
 	}
-	twidth += lw;
-	PUT1("%s ", buf);
 }
 
-/* -- write_text_block -- */
-void write_text_block(int job,
-		      int abc_state)
+/* -- write a text block -- */
+void write_text_block(int job, int abc_state)
 {
-	if (twidth < 0)
+	if (strtw < 0)
 		return;
 
-	if (job == T_FILL || job == T_JUSTIFY) {
-		int ntline, nbreak;
-		float textwidth, ftline, w1, baseskip;
-		float lwidth;
-
-		baseskip = cfmt.textfont.size * cfmt.lineskipfac;
-		lwidth = ((cfmt.landscape ? cfmt.pageheight : cfmt.pagewidth)
-			- cfmt.leftmargin - cfmt.rightmargin) / cfmt.scale;
-
-		/* estimate text widths.. ok for T-R, wild guess for other fonts */
-		w1 = cfmt.textfont.swfac * cfmt.textfont.size;
-
-		PUT2(") %.1f P%d\n",
-		     lwidth, job == T_FILL ? 1 : 2);
-
-		/* estimate the skip:
-		 * 1- (total textwidth)/(available width) */
-		textwidth = twidth * w1;
-		ftline = textwidth / lwidth;
-		/* 2- assume some chars lost at each line end */
-		nbreak = ftline;
-		textwidth += 5 * nbreak * cwid('a') * w1;
-		ftline = textwidth / lwidth;
-		ntline = ftline + 1.0;
-		bskip((ntline - 1) * baseskip);
+	if (strtx) {
+		PUT1(")%s", strop);
+		strtx = 0;
 	}
+	if (job == T_JUSTIFY)
+		PUT0("}def\n"
+			"/strop/show load def str\n");
+	else if (job == T_FILL)
+		PUT0("\n");
 	bskip(cfmt.textfont.size * cfmt.parskipfac);
 	buffer_eob();
 
@@ -664,17 +743,68 @@ void write_text_block(int job,
 	if (!epsf && abc_state != ABC_S_TUNE && multicol_start == 0)
 		write_buffer();
 	font_init = 0;
+	strtw = -1;
+}
 
-	twidth = -1;
+/* -- output a line of words after tune -- */
+static int put_wline(unsigned char *p,
+		     float x,
+		     int right)
+{
+	unsigned char *q, *r, sep;
+
+	while (isspace(*p))
+		p++;
+	if (*p == '$' && isdigit(p[1])
+	    && (unsigned) (p[1] - '0') < DFONT_MIN) {
+		if (strcf != p[1] - '0') {
+			strcf = p[1] - '0';
+			strfc = 1;
+		}
+		p += 2;
+	}
+	r = 0;
+	q = p;
+	if (isdigit(*p) || p[1] == '.') {
+		while (*p != '\0') {
+			p++;
+			if (*p == ' '
+			    || p[-1] == ':'
+			    || p[-1] == '.')
+				break;
+		}
+		r = p;
+		while (*p == ' ')
+			p++;
+	}
+
+	/* on the left side, permit page break at empty lines or stanza start */
+	if (!right
+	   && (*p == '\0' || r != 0)) {
+		set_font_init();
+		buffer_eob();
+	}
+
+	if (r != 0) {
+		sep = *r;
+		*r = '\0';
+		PUT1("%.1f 0 M ", x);
+		put_str(q,  A_RIGHT);
+		*r = sep;
+	}
+	if (*p != '\0') {
+		PUT1("%.1f 0 M ", x + 5);
+		put_str(p, A_LEFT);
+	}
+	return *p == '\0' && r == 0;
 }
 
 /* -- output the words after tune -- */
 void put_words(struct SYMBOL *words)
 {
 	struct SYMBOL *s, *s_end, *s2;
-	char str[82];
-	unsigned char *p, *q;
-	int n, have_text, max2col;
+	unsigned char *p;
+	int i, n, have_text, max2col;
 	float middle;
 
 	str_font(&cfmt.wordsfont);
@@ -682,8 +812,7 @@ void put_words(struct SYMBOL *words)
 	/* see if we may have 2 columns */
 	middle = 0.5 * ((cfmt.landscape ? cfmt.pageheight : cfmt.pagewidth)
 		- cfmt.leftmargin - cfmt.rightmargin) / cfmt.scale;
-	max2col = (middle - 45.) / (cfmt.wordsfont.swfac * cfmt.wordsfont.size
-				    * cwid('a'));
+	max2col = (middle - 45.) / (cwid('a') * cfmt.wordsfont.swfac);
 	n = 0;
 	have_text = 0;
 	for (s = words; s != 0; s = s->next) {
@@ -702,8 +831,6 @@ void put_words(struct SYMBOL *words)
 		} else	have_text = 1;
 	}
 	if (n > 0) {
-		int i;
-
 		n++;
 		n /= 2;
 		i = n;
@@ -714,8 +841,7 @@ void put_words(struct SYMBOL *words)
 			while (isspace(*p))
 				p++;
 			if (*p == '\0') {
-				if (have_text
-				    && --i <= 0)
+				if (have_text && --i <= 0)
 					break;
 				have_text = 0;
 			} else	have_text = 1;
@@ -732,69 +858,13 @@ void put_words(struct SYMBOL *words)
 	for (s = words; s != 0 || s2 != 0; ) {
 		bskip(cfmt.lineskipfac * cfmt.wordsfont.size);
 		if (s != 0) {
-			p = &s->as.text[2];
-			while (isspace(*p))
-				p++;
-			q = str;
-			if (isdigit(*p) || p[1] == '.') {
-				while (*p != '\0') {
-					*q++ = *p++;
-					if (*p == ' '
-					    || p[-1] == ':'
-					    || p[-1] == '.')
-						break;
-				}
-				if (*p == ' ')
-					p++;
-			}
-			*q = '\0';
-
-			/* permit page break at empty lines or stanza start */
-			if (*p == '\0' || str[0] != '\0') {
-				set_font_init();
-				buffer_eob();
-			}
-			if (str[0] != '\0') {
-				PUT0("45 0 M ");
-				put_str(str,  A_RIGHT);
-			}
-			if (*p != '\0') {
-				PUT0("50 0 M ");
-				put_str(p, A_LEFT);
-				PUT0("\n");
-			}
+			put_wline(&s->as.text[2], 45., 0);
 			s = s->next;
 			if (s == s_end)
 				s = 0;
 		}
 		if (s2 != 0) {
-			p = &s2->as.text[2];
-			while (isspace(*p))
-				p++;
-			q = str;
-			if (isdigit(*p) || p[1] == '.') {
-				while (*p != '\0') {
-					*q++ = *p++;
-					if (*p == ' '
-					    || p[-1] == ':'
-					    || p[-1] == '.')
-						break;
-				}
-				if (*p == ' ')
-					p++;
-			}
-			*q = '\0';
-			if (str[0] != '\0') {
-				PUT1("%.2f 0 M ", 20. + middle);
-				put_str(str, A_RIGHT);
-				PUT0("\n");
-			}
-			if (*p != '\0') {
-				PUT1("%.2f 0 M ",
-				     25. + middle);
-				put_str(p, A_LEFT);
-				PUT0("\n");
-			} else if (str[0] == '\0') {
+			if (put_wline(&s2->as.text[2], 20. + middle, 1)) {
 				if (--n == 0) {
 					if (s != 0)
 						n++;
@@ -824,11 +894,8 @@ static void put_text(char *head,
 		return;
 	h = cfmt.textfont.size * cfmt.lineskipfac;
 	bskip(h * 1.2);
-	if (curfont != 0) {
-		set_font(&cfmt.textfont);
-		curfont = 0;
-	}
-	PUT1("0 0 M (%s) show ", head);
+	strcf = -1;
+	PUT1("0 0 M(%s)show ", head);
 	if (strlen(head) > 6)
 		bskip(h);
 	do {
@@ -899,14 +966,14 @@ void write_title(struct SYMBOL *s)
 		set_font(&cfmt.subtitlefont);
 	}
 	if (cfmt.titleleft)
-		PUT0("0 0 M (");
-	else	PUT1("%.1f 0 M (",
+		PUT0("0 0 M(");
+	else	PUT1("%.1f 0 M(",
 		     0.5 * ((cfmt.landscape ? cfmt.pageheight : cfmt.pagewidth)
 		     - cfmt.leftmargin - cfmt.rightmargin) / cfmt.scale);
 	if (s == info.title && cfmt.withxrefs)
 		PUT1("%s.  ", info.xref);
-	put_texstr(t);
-	PUT1(") show%s\n", cfmt.titleleft ? "" : "c");
+	tex_str(t);
+	PUT2("%s)show%s\n", tex_buf, cfmt.titleleft ? "" : "c");
 }
 
 /* -- write_heading -- */
@@ -971,9 +1038,7 @@ void write_heading(void)
 			str_font(&cfmt.infofont);
 			bskip(cfmt.infofont.size + cfmt.infospace);
 			PUT1("%.1f 0 M ", lwidth);
-			if (rhythm)
-				put_inf2r(rhythm, info.area, A_RIGHT);
-			else	put_inf2r(info.area, 0, A_RIGHT);
+			put_inf2r(rhythm, info.area, A_RIGHT);
 			PUT0("\n");
 			down1 += cfmt.infofont.size + cfmt.infospace;
 		}
