@@ -22,32 +22,31 @@
  */
 
 #include <stdio.h>
+#include <time.h>
 #include <string.h>
 
 #include "abcparse.h"
 #include "abc2ps.h"
 
-/*  subroutines to define postscript macros which draw symbols	*/
-
 static char ps_head[] =
-	"/xymove{/x 2 index def /y 1 index def M}!\n"
+	"/xymove{/x 2 index def/y 1 index def M}!\n"
 
 	/* str showc - center at current pt */
-	"/showc{dup stringwidth pop 2 div neg 0 RM show}!\n"
+	"/showc{dup stringwidth pop .5 mul neg 0 RM show}!\n"
 
-	/* str lshow - show left-aligned */
-	"/lshow{dup stringwidth pop neg 0 RM show}!\n"
+	/* str showr - show right-aligned */
+	"/showr{dup stringwidth pop neg 0 RM show}!\n"
 
 	/* str showb - show in box */
 	"/showb{	dup currentpoint 3 -1 roll show\n"
-	"	0.6 setlinewidth\n"
+	"	.6 SLW\n"
 	"	exch 2 sub exch 3 sub 3 -1 roll\n"
 	"	stringwidth pop 4 add fh 4 add rectstroke}!\n"
 
 #if 0
 	"/showcb{ % usage: str showcb - show centered in box\n"
-	"	dup stringwidth pop dup 2 div neg 0 RM currentpoint 4 -1 roll show\n"
-	"	0.6 setlinewidth\n"
+	"	dup stringwidth pop dup .5 mul neg 0 RM currentpoint 4 -1 roll show\n"
+	"	.6 SLW\n"
 	"	exch 2 sub exch 3 sub 3 -1 roll\n"
 	"	4 add fh 4 add rectstroke}!\n"
 #endif
@@ -78,9 +77,9 @@ static char ps_head[] =
 	"/stclef{exch 0.85 div exch 0.85 div gsave 0.85 dup scale tclef grestore}!\n"
 
 	/* x y octu - upper '8' */
-	"/octu{/Times-Roman 12 selectfont M -1.5 36 RM (8) show}!\n"
+	"/octu{/Times-Roman 12 selectfont M -1.5 36 RM(8)show}!\n"
 	/* x y octl - lower '8' */
-	"/octl{/Times-Roman 12 selectfont M -3.5 -19 RM (8) show}!\n"
+	"/octl{/Times-Roman 12 selectfont M -3.5 -19 RM(8)show}!\n"
 
 	/* x y bclef - bass clef */
 	"/bclef{	M\n"
@@ -115,15 +114,15 @@ static char ps_head[] =
 	/* x y cclef */
 	"/cclef{	gsave T\n"
 	"	cchalf 0 24 T 1 -1 scale cchalf\n"
-	"	-5.5 0 M 0 24 RL 3 0 RL 0 -24 RL fill\n"
-	"	-0.5 0 M 0 24 RL 0.7 setlinewidth stroke grestore}!\n"
+	"	-5.5 0 3 24 rectfill\n"
+	"	-0.5 0 M 0 24 RL 0.7 SLW stroke grestore}!\n"
 
-	"/scclef{exch 0.85 div exch 0.85 div gsave 0.85 dup scale\n"
+	"/scclef{exch .85 div exch .85 div gsave .85 dup scale\n"
 	"	2 add cclef grestore}!\n"
 
 	/* x y pclef */
-	"/pclef{	M 1.4 setlinewidth -2.7 2 RM\n"
-	"	0 20 RL 5.4 0 RL 0 -20 RL -5.4 0 RL stroke}!\n"
+	"/pclef{	exch 2.7 sub exch 2 add 5.4 20\n"
+	"	1.4 SLW rectstroke}!\n"
 	"/spclef{pclef}!\n"
 
 	/* t dx dy x y bm - beam, depth t */
@@ -131,9 +130,9 @@ static char ps_head[] =
 	"	neg exch neg exch RL 0 exch RL fill}!\n"
 
 	/* str x y bnum - tuplet number / ratio */
-	"/bnum{M /Times-Italic 12 selectfont showc}!\n"
+	"/bnum{M/Times-Italic 12 selectfont showc}!\n"
 	"/bnumb{	currentgray 3 index stringwidth pop 4 add\n"
-	"        dup 2 div neg 4 index add 3 index 3 -1 roll 8\n"
+	"        dup .5 mul neg 4 index add 3 index 3 -1 roll 8\n"
 	"        1.0 setgray rectfill setgray bnum}!\n"
 
 	/* dx dy x y tubr - tuplet bracket */
@@ -142,19 +141,19 @@ static char ps_head[] =
 
 	/* x y r00 - longa rest */
 	"/r00{	xymove\n"
-	"	-1 6 RM 0 -12 RL 3 0 RL 0 12 RL fill}!\n"
+	"	-1 -6 RM currentpoint 3 12 rectfill}!\n"
 
 	/* x y r0 - breve rest */
 	"/r0{	xymove\n"
-	"	-1 6 RM 0 -6 RL 3 0 RL 0 6 RL fill}!\n"
+	"	-1 0 RM currentpoint 3 6 rectfill}!\n"
 
 	/* x y r1 - rest */
 	"/r1{	xymove\n"
-	"	-3 6 RM 0 -3 RL 7 0 RL 0 3 RL fill}!\n"
+	"	-3 3 RM currentpoint 7 3 rectfill}!\n"
 
 	/* x y r2 - half rest */
 	"/r2{	xymove\n"
-	"	-3 0 RM 0 3 RL 7 0 RL 0 -3 RL fill}!\n"
+	"	-3 0 RM currentpoint 7 3 rectfill}!\n"
 
 	/* x y r4 - quarter rest */
 	"/r4{	xymove\n"
@@ -177,20 +176,20 @@ static char ps_head[] =
 
 	/* x y r8 - eighth rest */
 	"/r8{	xymove\n"
-	"	0.5 setlinewidth 3.3 4 RM\n"
+	"	.5 SLW 3.3 4 RM\n"
 	"	-3.4 -9.6 RL stroke\n"
 	"	x y M 3.4 4 RM r8e}!\n"
 
 	/* x y r16 - 16th rest */
 	"/r16{	xymove\n"
-	"	0.5 setlinewidth 3.3 4 RM\n"
+	"	.5 SLW 3.3 4 RM\n"
 	"	-4 -15.6 RL stroke\n"
 	"	x y M 3.4 4 RM r8e\n"
 	"	x y M 1.9 -2 RM r8e}!\n"
 
 	/* x y r32 - 32th rest */
 	"/r32{	xymove\n"
-	"	0.5 setlinewidth 4.8 10 RM\n"
+	"	.5 SLW 4.8 10 RM\n"
 	"	-5.5 -21.6 RL stroke\n"
 	"	x y M 4.9 10 RM r8e\n"
 	"	x y M 3.4 4 RM r8e\n"
@@ -198,7 +197,7 @@ static char ps_head[] =
 
 	/* x y r64 - 64th rest */
 	"/r64{	xymove\n"
-	"	0.5 setlinewidth 4.8 10 RM\n"
+	"	.5 SLW 4.8 10 RM\n"
 	"	-7 -27.6 RL stroke\n"
 	"	x y M 4.9 10 RM r8e\n"
 	"	x y M 3.4 4 RM r8e\n"
@@ -207,7 +206,7 @@ static char ps_head[] =
 
 	/* x y r128 - 128th rest */
 	"/r128{	xymove\n"
-	"	0.5 setlinewidth 5.8 16 RM\n"
+	"	.5 SLW 5.8 16 RM\n"
 	"	-8.5 -33.6 RL stroke\n"
 	"	x y M 5.9 16 RM r8e\n"
 	"	x y M 4.4 10 RM r8e\n"
@@ -253,7 +252,7 @@ static char ps_head[] =
 	"/stc{M currentpoint 1.2 0 360 arc fill}!\n"
 
 	/* x y emb - emphasis bar */
-	"/emb{	1.2 setlinewidth 1 setlinecap M\n"
+	"/emb{	1.2 SLW 1 setlinecap M\n"
 	"	-2.5 0 RM 5 0 RL stroke 0 setlinecap}!\n"
 
 	/* x y cpu - roll sign above head */
@@ -268,7 +267,7 @@ static char ps_head[] =
 
 	/* x y trl - trill sign */
 	"/trl{	/Times-BoldItalic 16 selectfont\n"
-	"	M -4 2 RM (tr) show}!\n"
+	"	M -4 2 RM(tr)show}!\n"
 
 	/* x y umrd - upper mordent */
 	"/umrd{	4 add M\n"
@@ -279,7 +278,7 @@ static char ps_head[] =
 
 	/* x y lmrd - lower mordent */
 	"/lmrd{	2 copy umrd 8 add M\n"
-	"	0.6 setlinewidth 0 -8 RL stroke}!\n"
+	"	.6 SLW 0 -8 RL stroke}!\n"
 
 	/* str x y fng - finger (0-5) */
 	"/fng{/Bookman-Demi 8 selectfont M -3 1 RM show}!\n"
@@ -288,21 +287,21 @@ static char ps_head[] =
 	"/dacs{/Times-Roman 16 selectfont 3 add M showc}!\n"
 
 	/* x y brth - breath */
-	"/brth{/Times-BoldItalic 30 selectfont 6 add M (,) show}!\n"
+	"/brth{/Times-BoldItalic 30 selectfont 6 add M(,)show}!\n"
 
 	/* str x y pf - p, f, pp, .. */
-	"/pf{/Times-BoldItalic 16 selectfont 5 add M showc}!\n"
+	"/pf{/Times-BoldItalic 16 selectfont 5 add M show}!\n"
 
 	/* str x y sfz */
-	"/sfz{	exch 4 sub exch 5 add M pop\n"
-	"	/Times-Italic 14 selectfont (s) show\n"
-	"	/Times-BoldItalic 16 selectfont (f) show\n"
-	"	/Times-Italic 14 selectfont (z) show}!\n"
+	"/sfz{	M -7 5 RM pop\n"
+	"	/Times-Italic 14 selectfont(s)show\n"
+	"	/Times-BoldItalic 16 selectfont(f)show\n"
+	"	/Times-Italic 14 selectfont(z)show}!\n"
 
 	/* x y coda - coda */
-	"/coda{	1 setlinewidth 2 add 2 copy M 0 20 RL\n"
-	"	2 copy 10 add exch -10 add exch M 20 0 RL stroke\n"
-	"	10 add 6 0 360 arc 1.7 setlinewidth stroke}!\n"
+	"/coda{	1 SLW 2 add 2 copy M 0 20 RL\n"
+	"	2 copy M -10 10 RM 20 0 RL stroke\n"
+	"	10 add 6 0 360 arc 1.7 SLW stroke}!\n"
 
 	/* x y sgno - segno */
 	"/sgno{	M 0 3 RM currentpoint currentpoint currentpoint\n"
@@ -315,24 +314,30 @@ static char ps_head[] =
 	"	-4 -1.9 -5.9 1.7 -4.2 2.6 RC\n"
 	"	1.3 0.7 2.9 -1.3 0.7 -2 RC\n"
 	"	fill\n"
-	"	M 0.8 setlinewidth -6 1.2 RM 12.6 12.6 RL stroke\n"
+	"	M 0.8 SLW -6 1.2 RM 12.6 12.6 RL stroke\n"
 	"	7 add exch -6 add exch 1.2 0 360 arc fill\n"
 	"	8 add exch 6 add exch 1.2 0 360 arc fill}!\n"
 
 	/* w x y cresc - crescendo */
-	"/cresc{	1.2 setlinewidth M dup 6 RM\n"
-	"	dup neg 4 RL 4 RL stroke}!\n"
+	"/cresc{	1 SLW M dup 5 RM\n"
+	"	defl 1 and 0 eq\n"
+	"	{dup neg 4 RL 4 RL}\n"
+	"	{dup neg 2.2 RL 0 3.6 RM 2.2 RL}\n"
+	"	ifelse stroke}!\n"
 
 	/* w x y dim - diminuendo */
-	"/dim{	1.2 setlinewidth 6 add M\n"
-	"	dup 4 RL neg 4 RL stroke}!\n"
+	"/dim{	1 SLW 5 add M\n"
+	"	defl 2 and 0 eq\n"
+	"	{dup 4 RL neg 4 RL}\n"
+	"	{dup 2.2 RL 0 3.6 RM neg 2.2 RL}\n"
+	"	ifelse stroke}!\n"
 
-	/* x y dplus - + decoration */
-	"/dplus{	1.2 setlinewidth 0.5 add M 0 6 RL\n"
+	/* x y dplus - plus */
+	"/dplus{	1.2 SLW 0.5 add M 0 6 RL\n"
 	"	-3 -3 RM 6 0 RL stroke}!\n"
 
 	/* x y accent - accent */
-	"/accent{1.2 setlinewidth M -4 2 RM\n"
+	"/accent{1.2 SLW M -4 2 RM\n"
 	"	8 2 RL -8 2 RL stroke}!\n"
 
 	/* x y turn - turn */
@@ -351,18 +356,18 @@ static char ps_head[] =
 
 	/* x y trnx - turn with line through it */
 	"/turnx{	2 copy turn M\n"
-	"	0.6 setlinewidth 0 1.5 RM 0 9 RL stroke}!\n"
+	"	.6 SLW 0 1.5 RM 0 9 RL stroke}!\n"
 
 	/* x y lphr - longphrase */
-	"/lphr{1.2 setlinewidth M 0 -18 RL stroke}!\n"
+	"/lphr{1.2 SLW M 0 -18 RL stroke}!\n"
 
 	/* x y mphr - mediumphrase */
-	"/mphr{1.2 setlinewidth M 0 -12 RL stroke}!\n"
+	"/mphr{1.2 SLW M 0 -12 RL stroke}!\n"
 
 	/* x y sphr - shortphrase */
-	"/sphr{1.2 setlinewidth M 0 -6 RL stroke}!\n"
+	"/sphr{1.2 SLW M 0 -6 RL stroke}!\n"
 
-	/* len x y ltr - long trill */
+	/* w x y ltr - long trill */
 	"/ltr{	gsave 4 add T\n"
 	"	0 6 3 -1 roll{\n"
 	/*		% first loop draws left half of squiggle; second draws right\n*/
@@ -373,13 +378,13 @@ static char ps_head[] =
 	"			1.9 0.8 1.4 0.7 0 -0.4 curveto\n"
 	"			fill\n"
 	"			pop 180 rotate -6 0 translate\n"
-	"		} for\n"
+	"		}for\n"
 	/*		% shift axes right one squiggle*/
 	"		pop 6 0 translate\n"
-	"	} for\n"
+	"	}for\n"
 	"	grestore}!\n"
 
-	/* len x ylow arp - arpeggio */
+	/* h x ylow arp - arpeggio */
 	"/arp{gsave 90 rotate exch neg ltr grestore}!\n"
 
 	/* x y wedge - wedge */
@@ -401,23 +406,25 @@ static char ps_head[] =
 	"	2 add M 0 -4 RL stroke}!\n"
 
 	/* y hl - helper line at height y */
-	"/hl{	0.8 setlinewidth x -6 add exch M\n"
+	"/hl{	.8 SLW x -6 add exch M\n"
 	"	12 0 RL stroke}!\n"
-
 	/* y hl1 - longer helper line */
-	"/hl1{	0.8 setlinewidth x -7 add exch M\n"
+	"/hl1{	.8 SLW x -7 add exch M\n"
 	"	14 0 RL stroke}!\n"
+	/* y hl2 - more longer helper line */
+	"/hl2{	.8 SLW x -8 add exch M\n"
+	"	16 0 RL stroke}!\n"
 
 	/* -- accidentals -- */
 	/* x y sh0 - sharp sign */
-	"/sh0{	gsave T 0.9 setlinewidth\n"
+	"/sh0{	gsave T .9 SLW\n"
 	"	-1.2 -8.4 M 0 15.4 RL\n"
 	"	1.4 -7.2 M 0 15.4 RL stroke\n"
 	"	-2.6 -3 M 5.4 1.6 RL 0 -2.2 RL -5.4 -1.6 RL 0 2.2 RL fill\n"
 	"	-2.6 3.4 M 5.4 1.6 RL 0 -2.2 RL -5.4 -1.6 RL 0 2.2 RL fill\n"
 	"	grestore}!\n"
 	/* x y ft0 - flat sign */
-	"/ft0{	gsave T 0.8 setlinewidth\n"
+	"/ft0{	gsave T .8 SLW\n"
 	"	-1.8 2.5 M\n"
 	"	6.4 3.3 6.5 -3.6 0 -6.6 RC\n"
 	"	4.6 3.9 4.5 7.6 0 5.7 RC\n"
@@ -425,18 +432,18 @@ static char ps_head[] =
 	"	0 7.1 RM 0 -12.6 RL stroke\n"
 	"	grestore}!\n"
 	/* x y nt0 - natural sign */
-	"/nt0{	gsave T 0.5 setlinewidth\n"
+	"/nt0{	gsave T .5 SLW\n"
 	"	-2 -4.3 M 0 12.2 RL\n"
 	"	1.3 -7.8 M 0 12.2 RL stroke\n"
-	"	2.1 setlinewidth\n"
+	"	2.1 SLW\n"
 	"	-2 -2.9 M 3.3 0.6 RL\n"
 	"	-2 2.4 M 3.3 0.6 RL stroke\n"
 	"	grestore}!\n"
 	/* x y dsh0 - double sharp */
-	"/dsh0{	2 copy M 0.7 setlinewidth\n"
+	"/dsh0{	2 copy M .7 SLW\n"
 	"	-2 -2 RM 4 4 RL\n"
 	"	-4 0 RM 4 -4 RL stroke\n"
-	"	0.5 setlinewidth 2 copy M 1.3 -1.3 RM\n"
+	"	.5 SLW 2 copy M 1.3 -1.3 RM\n"
 	"	2 -0.2 RL 0.2 -2 RL -2 0.2 RL -0.2 2 RL fill\n"
 	"	2 copy M 1.3 1.3 RM\n"
 	"	2 0.2 RL 0.2 2 RL -2 -0.2 RL -0.2 -2 RL fill\n"
@@ -457,13 +464,13 @@ static char ps_head[] =
 
 	/* some microtone accidentals */
 	/* 1/4 ton sharp */
-	"/sh1{	gsave T 0.9 setlinewidth\n"
+	"/sh1{	gsave T .9 SLW\n"
 	"	0 -7.8 M 0 15.4 RL stroke\n"
 	"	-1.8 -2.7 M 3.6 1.1 RL 0 -2.2 RL -3.6 -1.1 RL 0 2.2 RL fill\n"
 	"	-1.8 3.7 M 3.6 1.1 RL 0 -2.2 RL -3.6 -1.1 RL 0 2.2 RL fill\n"
 	"	grestore}!\n"
 	/* 3/4 ton sharp */
-	"/sh513{	gsave T 0.8 setlinewidth\n"
+	"/sh513{	gsave T .8 SLW\n"
 	"	-2.5 -8.7 M 0 15.4 RL\n"
 	"	0 -7.8 M 0 15.4 RL\n"
 	"	2.5 -6.9 M 0 15.4 RL stroke\n"
@@ -492,25 +499,30 @@ static char ps_head[] =
 	/* str gcshow - guitar chord */
 	"/gchshow{\n"
 	"	pop pop\n"
-	"	dup 129 eq {pop sharp_glyph}\n"
-	"	  {dup 130 eq {pop flat_glyph}\n"
-	"	    {dup 131 eq {pop nat_glyph}\n"
-	"		{currentfont /Encoding get exch get glyphshow}\n"
+	"	dup 129 eq{pop sharp_glyph}\n"
+	"	  {dup 130 eq{pop flat_glyph}\n"
+	"	    {dup 131 eq{pop nat_glyph}\n"
+	"		{currentfont/Encoding get exch get glyphshow}\n"
 	"		ifelse}\n"
 	"	    ifelse}\n"
 	"	  ifelse}!\n"
 	"/gcshow{/gchshow load cshow}!\n"
 	/* x y w h box - draw a box */
-	"/box{0.6 setlinewidth rectstroke}!\n"
+	"/box{.6 SLW rectstroke}!\n"
 
 	/* str anshow - annotation */
 	"/anshow{gcshow}!\n"
 
 	/* -- lyrics under notes -- */
-	/* l x y wln - ligne */
-	"/wln{M 0.8 setlinewidth 0 RL stroke}!\n"
-	/* x y whf - hyphen */
-	"/whf{3 add 3 3 1 roll wln}!\n"
+	/* l x y wln - underscore line */
+	"/wln{M .8 SLW 0 RL stroke}!\n"
+	/* l x y hyph - hyphen */
+	"/hyph{	.8 SLW 3 add M\n"
+	"	dup cvi 40 idiv 1 add 1 index exch div\n"	/* l dx */
+	"	dup 3 sub 3 1 roll\n"		/* (dx-3) l dx */
+	"	dup .5 mul dup 1.5 sub 0 RM\n"	/* (dx-3) l dx (dx/2) */
+	"	exch 3 -1 roll\n"		/* (dx-3) (dx/2) dx l */
+	"	{pop 3 0 RL dup 0 RM} for stroke pop}!\n"
 	/* str lyshow - lyrics */
 	"/lyshow{show}!\n"
 
@@ -520,7 +532,7 @@ static char ps_head[] =
 	/* h x y dotbar - dotted bar */
 	"/dotbar{[5] 0 setdash bar [] 0 setdash}!\n"
 	/* h x y thbar - thick bar */
-	"/thbar{M dup 0 exch RL 3 0 RL 0 exch neg RL fill}!\n"
+	"/thbar{3 -1 roll 3 exch rectfill}!\n"
 	/* x y rdots - repeat dots */
 	"/rdots{	9 add M currentpoint 2 copy 1.2 0 360 arc\n"
 	"	6 add M currentpoint 1.2 0 360 arc fill}!\n"
@@ -541,12 +553,12 @@ static char ps_head[] =
 	"/ctsig{dlw 2 copy csig 4 add M 0 16 RL stroke}!\n"
 
 	/* (top) (bot) x y tsig - time signature */
-	"/tsig{	M gsave /Times-Bold 16 selectfont 1.2 1 scale\n"
+	"/tsig{	M gsave/Times-Bold 16 selectfont 1.2 1 scale\n"
 	"	0 1 RM currentpoint 3 -1 roll showc\n"
 	"	M 0 12 RM showc grestore}!\n"
 
 	/* (meter) x y stsig - single time signature */
-	"/stsig{	M gsave /Times-Bold 18 selectfont 1.2 1 scale\n"
+	"/stsig{	M gsave/Times-Bold 18 selectfont 1.2 1 scale\n"
 	"	0 6 RM showc grestore}!\n"
 
 	/* l x y staff - 5 lines staff */
@@ -573,16 +585,16 @@ static char ps_head[] =
 	"/bracket{M dlw -5 2 RM currentpoint\n"
 	"	-1.7 2 RM 10.5 -1 12 4.5 12 3.5 RC\n"
 	"	0 -1 -3.5 -5.5 -8.5 -5.5 RC fill\n"
-	"	3 setlinewidth M 0 2 RM\n"
+	"	3 SLW M 0 2 RM\n"
 	"	0 exch neg -8 add RL currentpoint stroke\n"
 	"	dlw M -1.7 0 RM\n"
 	"	10.5 1 12 -4.5 12 -3.5 RC\n"
 	"	0 1 -3.5 5.5 -8.5 5.5 RC fill}!\n"
 
 	/* nb_measures x y mrest */
-	"/mrest{	gsave T 1 setlinewidth\n"
+	"/mrest{	gsave T 1 SLW\n"
 	"	-20 6 M 0 12 RL 20 6 M 0 12 RL stroke\n"
-	"	5 setlinewidth -20 12 M 40 0 RL stroke\n"
+	"	5 SLW -20 12 M 40 0 RL stroke\n"
 	"	/Times-Bold 15 selectfont 0 28 M showc grestore}!\n"
 
 	/* x y mrep - measure repeat */
@@ -596,46 +608,26 @@ static char ps_head[] =
 	"/mrep2{	2 copy 2 copy\n"
 	"	M -5 18 RM currentpoint 1.4 0 360 arc\n"
 	"	M 5 6 RM currentpoint 1.4 0 360 arc fill\n"
-	"	M 1.8 setlinewidth\n"
+	"	M 1.8 SLW\n"
 	"	-7 4 RM 14 10 RL -14 -4 RM 14 10 RL\n"
 	"	stroke}!\n"
 
 	/* str bracket_type dx x y repbra - repeat bracket */
 	"/repbra{gsave dlw T 0 -20 M\n"
-	"	0 20 3 index 1 ne {RL} {RM} ifelse 0 RL 0 ne {0 -20 RL} if stroke\n"
+	"	0 20 3 index 1 ne{RL}{RM}ifelse 0 RL 0 ne{0 -20 RL}if stroke\n"
 	"	4 -13 M show grestore}!\n"
 
 	/* pp2x pp1x p1 pp1 pp2 p2 p1 SL - slur / tie */
-	"/SL{M curveto RL curveto fill}!\n"
+	"/SL{M curveto RL curveto closepath fill}!\n"
 
 	/* pp2x pp1x p1 pp1 pp2 p2 p1 dSL - dotted slur / tie */
-	"/dSL{	M [4] 0 setdash 0.8 setlinewidth\n"
+	"/dSL{	M [4] 0 setdash .8 SLW\n"
 	"	curveto stroke [] 0 setdash\n"
 	"	pop pop pop pop pop pop pop pop}!\n"
 
 	/* -- text -- */
-	"/dsp{dup stringwidth pop}!\n"
-	"/glue{	2 copy length exch length add string dup 4 2 roll 2 index 0 3 index\n"
-	"	putinterval exch length exch putinterval}!\n"
-	"/TXT{/txt exch def}!\n"
-	"/rejoin{( ) search pop exch glue}!\n"
-	"/measure{dsp txt stringwidth pop add textwidth 2 add gt}!\n"
-	"/join{txt exch glue TXT}!\n"
-	"/find{search {pop 3 -1 roll 1 add 3 1 roll}{pop exit} ifelse}!\n"
-	"/spacecount{0 exch ( ) {find} loop}!\n"
-	"/jproc{dsp textwidth exch sub exch dup spacecount}!\n"
-	"/popzero{dup 0 eq {pop}{div} ifelse}!\n"
-	"/justify{jproc 1 sub 3 2 roll exch popzero 0 32 4 3 roll widthshow} def\n"
-	/* str lwidth P1 */
-	"/P1{	/textwidth exch def () TXT\n"
-	"	dup spacecount{\n"
-	"		rejoin measure {gsave txt show grestore LF () TXT join}{join} ifelse\n"
-	"	} repeat gsave txt show grestore LF () TXT pop}!\n"
-	/* str lwidth P2 */
-	"/P2{	/textwidth exch def () TXT\n"
-	"	dup spacecount{\n"
-	"		rejoin measure {gsave txt justify grestore LF () TXT join}{join} ifelse\n"
-	"	} repeat gsave txt show grestore LF () TXT pop}!\n"
+	"/strw{stringwidth pop w add/w exch def}!\n"
+	"/jshow{w 0 32 4 -1 roll widthshow}!\n"
 
 	/* -- note heads -- */
 	/* x y hd - full head */
@@ -666,32 +658,32 @@ static char ps_head[] =
 	/* x y HDD - round breve */
 	"/HDD{	dlw HD\n"
 	"	x y M -6 -4 RM 0 8 RL\n"
-	"	x y M 6 -4 RM 0 8 RL stroke}!\n"
+	"	12 0 RM 0 -8 RL stroke}!\n"
 	/* x y breve - square breve */
 	"/breve{	xymove\n"
-	"	2.5 setlinewidth -6 -2.7 RM 12 0 RL\n"
+	"	2.5 SLW -6 -2.7 RM 12 0 RL\n"
 	"	0 5.4 RM -12 0 RL stroke\n"
 	"	dlw x y M -6 -5 RM 0 10 RL\n"
-	"	x y M 6 -5 RM 0 10 RL stroke}!\n"
+	"	12 0 RM 0 -10 RL stroke}!\n"
 	/* x y longa */
 	"/longa{	xymove\n"
-	"	2.5 setlinewidth -6 -2.7 RM 12 0 RL\n"
+	"	2.5 SLW -6 -2.7 RM 12 0 RL\n"
 	"	0 5.4 RM -12 0 RL stroke\n"
 	"	dlw x y M -6 -5 RM 0 10 RL\n"
-	"	x y M 6 -10 RM 0 15 RL stroke}!\n"
+	"	12 0 RM 0 -16 RL stroke}!\n"
 
 	/* -- default percussion heads -- */
 	/* x y pshhd - percussion sharp head */
-	"/pshhd{/x 2 index def /y 1 index def dsh0}!\n"
+	"/pshhd{/x 2 index def/y 1 index def dsh0}!\n"
 	/* x y pflhd - percussion flat head */
-	"/pflhd{/x 2 index def /y 1 index def dsh0\n"
-	"	0.7 setlinewidth x y 4 0 360 arc stroke}!\n"
+	"/pflhd{/x 2 index def/y 1 index def dsh0\n"
+	"	.7 SLW x y 4 0 360 arc stroke}!\n"
 
 	/* tin whistle */
 	"/tw_head{/Helvetica 8 selectfont\n"
-	"	0 -45 M 90 rotate (WHISTLE) show -90 rotate\n"
+	"	0 -45 M 90 rotate(WHISTLE)show -90 rotate\n"
 	"	/Helvetica-Bold 36 selectfont\n"
-	"	0 -45 M show .5 setlinewidth newpath}!\n"
+	"	0 -45 M show .5 SLW newpath}!\n"
 	"/tw_under{\n"
 	"	1 index 2.5 sub -4 M 2.5 -2.5 RL 2.5 2.5 RL\n"
 	"	-2.5 -2.5 RM 0 6 RL stroke}!\n"
@@ -704,95 +696,96 @@ static char ps_head[] =
 	"/tw_p{pop -55 M 0 6 RL -3 -3 RM 6 0 RL stroke}!\n"
 	"/tw_pp{	pop 3 sub -53.5 M 6 0 RL\n"
 	"	-1.5 -1.5 RM 0 3 RL\n"
-	"	-3 0 RM 0 -3 RL stroke}!\n";
+	"	-3 0 RM 0 -3 RL stroke}!\n"
+	"0 setlinecap 0 setlinejoin\n";
 
 static char *enc_tb[MAXENC] = {
 	/* 1 */
-	"/space /exclamdown /cent /sterling /currency /yen /brokenbar /section\n"
-	"/dieresis /copyright /ordfeminine /guillemotleft /logicalnot /hyphen /registered /macron\n"
-/*	"/degree /plusminus /twosuperior /threesuperior /acute /mu /paragraph /bullet\n" */
-	"/degree /plusminus /twosuperior /threesuperior /acute /mu /paragraph /periodcentered\n"
-/*	"/cedilla /dotlessi /ordmasculine /guillemotright /onequarter /onehalf /threequarters /questiondown\n" */
-	"/cedilla /onesuperior /ordmasculine /guillemotright /onequarter /onehalf /threequarters /questiondown\n"
+	"/space/exclamdown/cent/sterling/currency/yen/brokenbar/section\n"
+	"/dieresis/copyright/ordfeminine/guillemotleft/logicalnot/hyphen/registered/macron\n"
+/*	"/degree/plusminus/twosuperior/threesuperior/acute/mu/paragraph/bullet\n" */
+	"/degree/plusminus/twosuperior/threesuperior/acute/mu/paragraph/periodcentered\n"
+/*	"/cedilla/dotlessi/ordmasculine/guillemotright/onequarter/onehalf/threequarters/questiondown\n" */
+	"/cedilla/onesuperior/ordmasculine/guillemotright/onequarter/onehalf/threequarters/questiondown\n"
 	/* (300) */
-	"/Agrave /Aacute /Acircumflex /Atilde /Adieresis /Aring /AE /Ccedilla\n"
-	"/Egrave /Eacute /Ecircumflex /Edieresis /Igrave /Iacute /Icircumflex /Idieresis\n"
-	"/Eth /Ntilde /Ograve /Oacute /Ocircumflex /Otilde /Odieresis /multiply\n"
-	"/Oslash /Ugrave /Uacute /Ucircumflex /Udieresis /Yacute /Thorn /germandbls\n"
-	"/agrave /aacute /acircumflex /atilde /adieresis /aring /ae /ccedilla\n"
-	"/egrave /eacute /ecircumflex /edieresis /igrave /iacute /icircumflex /idieresis\n"
-	"/eth /ntilde /ograve /oacute /ocircumflex /otilde /odieresis /divide\n"
-	"/oslash /ugrave /uacute /ucircumflex /udieresis /yacute /thorn /ydieresis",
+	"/Agrave/Aacute/Acircumflex/Atilde/Adieresis/Aring/AE/Ccedilla\n"
+	"/Egrave/Eacute/Ecircumflex/Edieresis/Igrave/Iacute/Icircumflex/Idieresis\n"
+	"/Eth/Ntilde/Ograve/Oacute/Ocircumflex/Otilde/Odieresis/multiply\n"
+	"/Oslash/Ugrave/Uacute/Ucircumflex/Udieresis/Yacute/Thorn/germandbls\n"
+	"/agrave/aacute/acircumflex/atilde/adieresis/aring/ae/ccedilla\n"
+	"/egrave/eacute/ecircumflex/edieresis/igrave/iacute/icircumflex/idieresis\n"
+	"/eth/ntilde/ograve/oacute/ocircumflex/otilde/odieresis/divide\n"
+	"/oslash/ugrave/uacute/ucircumflex/udieresis/yacute/thorn/ydieresis",
 	/* 2 */
-	"/space /Aogonek /breve /Lslash /currency /Lcaron /Sacute /section\n"
-	"/dieresis /Scaron /Scedilla /Tcaron /Zacute /hyphen /Zcaron /Zdotaccent\n"
-	"/degree /aogonek /ogonek /lslash /acute /lcaron /sacute /caron\n"
-	"/cedilla /scaron /scedilla /tcaron /zacute /hungarumlaut /zcaron /zdotaccent\n"
+	"/space/Aogonek/breve/Lslash/currency/Lcaron/Sacute/section\n"
+	"/dieresis/Scaron/Scedilla/Tcaron/Zacute/hyphen/Zcaron/Zdotaccent\n"
+	"/degree/aogonek/ogonek/lslash/acute/lcaron/sacute/caron\n"
+	"/cedilla/scaron/scedilla/tcaron/zacute/hungarumlaut/zcaron/zdotaccent\n"
 	/* (300) */
-	"/Racute /Aacute /Acircumflex /Abreve /Adieresis /Lacute /Cacute /Ccedilla\n"
-	"/Ccaron /Eacute /Eogonek /Edieresis /Ecaron /Iacute /Icircumflex /Dcaron\n"
-	"/Dbar /Nacute /Ncaron /Oacute /Ocircumflex /Ohungarumlaut /Odieresis /multiply\n"
-	"/Rcaron /Uring /Uacute /Uhungarumlaut /Udieresis /Yacute /Tcedilla /germandbls\n"
-	"/racute /aacute /acircumflex /abreve /adieresis /lacute /cacute /ccedilla\n"
-	"/ccaron /eacute /eogonek /edieresis /ecaron /iacute /icircumflex /dcaron\n"
-	"/dbar /nacute /ncaron /oacute /ocircumflex /ohungarumlaut /odieresis /divide\n"
-	"/rcaron /uring /uacute /uhungarumlaut /udieresis /yacute /tcedilla /dotaccent",
+	"/Racute/Aacute/Acircumflex/Abreve/Adieresis/Lacute/Cacute/Ccedilla\n"
+	"/Ccaron/Eacute/Eogonek/Edieresis/Ecaron/Iacute/Icircumflex/Dcaron\n"
+	"/Dbar/Nacute/Ncaron/Oacute/Ocircumflex/Ohungarumlaut/Odieresis/multiply\n"
+	"/Rcaron/Uring/Uacute/Uhungarumlaut/Udieresis/Yacute/Tcedilla/germandbls\n"
+	"/racute/aacute/acircumflex/abreve/adieresis/lacute/cacute/ccedilla\n"
+	"/ccaron/eacute/eogonek/edieresis/ecaron/iacute/icircumflex/dcaron\n"
+	"/dbar/nacute/ncaron/oacute/ocircumflex/ohungarumlaut/odieresis/divide\n"
+	"/rcaron/uring/uacute/uhungarumlaut/udieresis/yacute/tcedilla/dotaccent",
 	/* 3 */
-	"/space /Hstroke /breve /sterling /currency /yen /Hcircumflex /section\n"
-	"/dieresis /Idotaccent /Scedilla /Gbreve /Jcircumflex /hyphen /registered /Zdotaccent\n"
-	"/degree /hstroke /twosuperior /threesuperior /acute /mu /hcircumflex /bullet\n"
-	"/cedilla /dotlessi /scedilla /gbreve /jcircumflex /onehalf /threequarters /zdotaccent\n"
+	"/space/Hstroke/breve/sterling/currency/yen/Hcircumflex/section\n"
+	"/dieresis/Idotaccent/Scedilla/Gbreve/Jcircumflex/hyphen/registered/Zdotaccent\n"
+	"/degree/hstroke/twosuperior/threesuperior/acute/mu/hcircumflex/bullet\n"
+	"/cedilla/dotlessi/scedilla/gbreve/jcircumflex/onehalf/threequarters/zdotaccent\n"
 	/* (300) */
-	"/Agrave /Aacute /Acircumflex /Atilde /Adieresis /Cdotaccent /Ccircumflex /Ccedilla\n"
-	"/Egrave /Eacute /Ecircumflex /Edieresis /Igrave /Iacute /Icircumflex /Idieresis\n"
-	"/Eth /Ntilde /Ograve /Oacute /Ocircumflex /Gdotaccent /Odieresis /multiply\n"
-	"/Gcircumflex /Ugrave /Uacute /Ucircumflex /Udieresis /Ubreve /Scircumflex /germandbls\n"
-	"/agrave /aacute /acircumflex /atilde /adieresis /cdotaccent /ccircumflex /ccedilla\n"
-	"/egrave /eacute /ecircumflex /edieresis /igrave /iacute /icircumflex /idieresis\n"
-	"/eth /ntilde /ograve /oacute /ocircumflex /gdotaccent /odieresis /divide\n"
-	"/gcircumflex /ugrave /uacute /ucircumflex /udieresis /ubreve /scircumflex /dotaccent",
+	"/Agrave/Aacute/Acircumflex/Atilde/Adieresis/Cdotaccent/Ccircumflex/Ccedilla\n"
+	"/Egrave/Eacute/Ecircumflex/Edieresis/Igrave/Iacute/Icircumflex/Idieresis\n"
+	"/Eth/Ntilde/Ograve/Oacute/Ocircumflex/Gdotaccent/Odieresis/multiply\n"
+	"/Gcircumflex/Ugrave/Uacute/Ucircumflex/Udieresis/Ubreve/Scircumflex/germandbls\n"
+	"/agrave/aacute/acircumflex/atilde/adieresis/cdotaccent/ccircumflex/ccedilla\n"
+	"/egrave/eacute/ecircumflex/edieresis/igrave/iacute/icircumflex/idieresis\n"
+	"/eth/ntilde/ograve/oacute/ocircumflex/gdotaccent/odieresis/divide\n"
+	"/gcircumflex/ugrave/uacute/ucircumflex/udieresis/ubreve/scircumflex/dotaccent",
 	/* 4 */
-	"/space /Aogonek /kra /Rcedilla /currency /Itilde /Lcedilla /section\n"
-	"/dieresis /Scaron /Emacron /Gcedilla /Tbar /hyphen /Zcaron /macron\n"
-	"/degree /aogonek /ogonek /rcedilla /acute /itilde /lcedilla /caron\n"
-	"/cedilla /scaron /emacron /gcedilla /tbar /Eng /zcaron /eng\n"
+	"/space/Aogonek/kra/Rcedilla/currency/Itilde/Lcedilla/section\n"
+	"/dieresis/Scaron/Emacron/Gcedilla/Tbar/hyphen/Zcaron/macron\n"
+	"/degree/aogonek/ogonek/rcedilla/acute/itilde/lcedilla/caron\n"
+	"/cedilla/scaron/emacron/gcedilla/tbar/Eng/zcaron/eng\n"
 	/* (300) */
-	"/Amacron /Aacute /Acircumflex /Atilde /Adieresis /Aring /AE /Iogonek\n"
-	"/Ccaron /Eacute /Eogonek /Edieresis /Edotaccent /Iacute /Icircumflex /Imacron\n"
-	"/Eth /Ncedilla /Omacron /Kcedilla /Ocircumflex /Otilde /Odieresis /multiply\n"
-	"/Oslash /Uogonek /Uacute /Ucircumflex /Udieresis /Utilde /Umacron /germandbls\n"
-	"/amacron /aacute /acircumflex /atilde /adieresis /aring /ae /iogonek\n"
-	"/ccaron /eacute /eogonek /edieresis /edotaccent /iacute /icircumflex /imacron\n"
-	"/dbar /ncedilla /omacron /kcedilla /ocircumflex /otilde /odieresis /divide\n"
-	"/oslash /uogonek /uacute /ucircumflex /udieresis /utilde /umacron /dotaccent",
+	"/Amacron/Aacute/Acircumflex/Atilde/Adieresis/Aring/AE/Iogonek\n"
+	"/Ccaron/Eacute/Eogonek/Edieresis/Edotaccent/Iacute/Icircumflex/Imacron\n"
+	"/Eth/Ncedilla/Omacron/Kcedilla/Ocircumflex/Otilde/Odieresis/multiply\n"
+	"/Oslash/Uogonek/Uacute/Ucircumflex/Udieresis/Utilde/Umacron/germandbls\n"
+	"/amacron/aacute/acircumflex/atilde/adieresis/aring/ae/iogonek\n"
+	"/ccaron/eacute/eogonek/edieresis/edotaccent/iacute/icircumflex/imacron\n"
+	"/dbar/ncedilla/omacron/kcedilla/ocircumflex/otilde/odieresis/divide\n"
+	"/oslash/uogonek/uacute/ucircumflex/udieresis/utilde/umacron/dotaccent",
 	/* 5 */
-	"/space /exclamdown /cent /sterling /currency /yen /brokenbar /section\n"
-	"/dieresis /copyright /ordfeminine /guillemotleft /logicalnot /hyphen /registered /macron\n"
-	"/degree /plusminus /twosuperior /threesuperior /acute /mu /paragraph /bullet\n"
-	"/cedilla /dotlessi /ordmasculine /guillemotright /onequarter /onehalf /threequarters /questiondown\n"
+	"/space/exclamdown/cent/sterling/currency/yen/brokenbar/section\n"
+	"/dieresis/copyright/ordfeminine/guillemotleft/logicalnot/hyphen/registered/macron\n"
+	"/degree/plusminus/twosuperior/threesuperior/acute/mu/paragraph/bullet\n"
+	"/cedilla/dotlessi/ordmasculine/guillemotright/onequarter/onehalf/threequarters/questiondown\n"
 	/* (300) */
-	"/Agrave /Aacute /Acircumflex /Atilde /Adieresis /Aring /AE /Ccedilla\n"
-	"/Egrave /Eacute /Ecircumflex /Edieresis /Igrave /Iacute /Icircumflex /Idieresis\n"
-	"/Gbreve /Ntilde /Ograve /Oacute /Ocircumflex /Otilde /Odieresis /multiply\n"
-	"/Oslash /Ugrave /Uacute /Ucircumflex /Udieresis /Idotaccent /Scedilla /germandbls\n"
-	"/agrave /aacute /acircumflex /atilde /adieresis /aring /ae /ccedilla\n"
-	"/egrave /eacute /ecircumflex /edieresis /igrave /iacute /icircumflex /idieresis\n"
-	"/gbreve /ntilde /ograve /oacute /ocircumflex /otilde /odieresis /divide\n"
-	"/oslash /ugrave /uacute /ucircumflex /udieresis /dotlessi /scedilla /ydieresis",
+	"/Agrave/Aacute/Acircumflex/Atilde/Adieresis/Aring/AE/Ccedilla\n"
+	"/Egrave/Eacute/Ecircumflex/Edieresis/Igrave/Iacute/Icircumflex/Idieresis\n"
+	"/Gbreve/Ntilde/Ograve/Oacute/Ocircumflex/Otilde/Odieresis/multiply\n"
+	"/Oslash/Ugrave/Uacute/Ucircumflex/Udieresis/Idotaccent/Scedilla/germandbls\n"
+	"/agrave/aacute/acircumflex/atilde/adieresis/aring/ae/ccedilla\n"
+	"/egrave/eacute/ecircumflex/edieresis/igrave/iacute/icircumflex/idieresis\n"
+	"/gbreve/ntilde/ograve/oacute/ocircumflex/otilde/odieresis/divide\n"
+	"/oslash/ugrave/uacute/ucircumflex/udieresis/dotlessi/scedilla/ydieresis",
 	/* 6 */
-	"/space /Aogonek /Emacron /Gcedilla /Imacron /Itilde /Kcedilla /Lcedilla\n"
-	"/acute /Rcedilla /Scaron /Tbar /Zcaron /hyphen /kra /Eng\n"
-	"/dbar /aogonek /emacron /gcedilla /imacron /itilde /kcedilla /lcedilla\n"
-	"/nacute /rcedilla /scaron /tbar /zcaron /section /germandbls /eng\n"
+	"/space/Aogonek/Emacron/Gcedilla/Imacron/Itilde/Kcedilla/Lcedilla\n"
+	"/acute/Rcedilla/Scaron/Tbar/Zcaron/hyphen/kra/Eng\n"
+	"/dbar/aogonek/emacron/gcedilla/imacron/itilde/kcedilla/lcedilla\n"
+	"/nacute/rcedilla/scaron/tbar/zcaron/section/germandbls/eng\n"
 	/* (300) */
-	"/Amacron /Aacute /Acircumflex /Atilde /Adieresis /Aring /AE /Iogonek\n"
-	"/Ccaron /Eacute /Eogonek /Edieresis /Edotaccent /Iacute /Icircumflex /Idieresis\n"
-	"/Dbar /Ncedilla /Omacron /Oacute /Ocircumflex /Otilde /Odieresis /Utilde\n"
-	"/Oslash /Uogonek /Uacute /Ucircumflex /Udieresis /Yacute /Thorn /Umacron\n"
-	"/amacron /aacute /acircumflex /atilde /adieresis /aring /ae /iogonek\n"
-	"/ccaron /eacute /eogonek /edieresis /edotaccent /iacute /icircumflex /idieresis\n"
-	"/eth /ncedilla /omacron /oacute /ocircumflex /otilde /odieresis /utilde\n"
-	"/oslash /uogonek /uacute /ucircumflex /udieresis /yacute /thorn /umacron"
+	"/Amacron/Aacute/Acircumflex/Atilde/Adieresis/Aring/AE/Iogonek\n"
+	"/Ccaron/Eacute/Eogonek/Edieresis/Edotaccent/Iacute/Icircumflex/Idieresis\n"
+	"/Dbar/Ncedilla/Omacron/Oacute/Ocircumflex/Otilde/Odieresis/Utilde\n"
+	"/Oslash/Uogonek/Uacute/Ucircumflex/Udieresis/Yacute/Thorn/Umacron\n"
+	"/amacron/aacute/acircumflex/atilde/adieresis/aring/ae/iogonek\n"
+	"/ccaron/eacute/eogonek/edieresis/edotaccent/iacute/icircumflex/idieresis\n"
+	"/eth/ncedilla/omacron/oacute/ocircumflex/otilde/odieresis/utilde\n"
+	"/oslash/uogonek/uacute/ucircumflex/udieresis/yacute/thorn/umacron"
 };
 
 /* -- define an encoding -- */
@@ -808,35 +801,35 @@ void define_encoding(int enc,		/* index */
 		ename = enc_txt;
 		if (enc > 0)
 		    fprintf(fout, "/%s [\n"
-			"/.notdef /.notdef /.notdef /.notdef /.notdef /.notdef /.notdef /.notdef\n"
-			"/.notdef /.notdef /.notdef /.notdef /.notdef /.notdef /.notdef /.notdef\n"
-			"/.notdef /.notdef /.notdef /.notdef /.notdef /.notdef /.notdef /.notdef\n"
-			"/.notdef /.notdef /.notdef /.notdef /.notdef /.notdef /.notdef /.notdef\n"
-			"/space /exclam /quotedbl /numbersign /dollar /percent /ampersand /quoteright\n"
-			"/parenleft /parenright /asterisk /plus /comma /minus /period /slash\n"
-			"/zero /one /two /three /four /five /six /seven\n"
-			"/eight /nine /colon /semicolon /less /equal /greater /question\n"
+			"/.notdef/.notdef/.notdef/.notdef/.notdef/.notdef/.notdef/.notdef\n"
+			"/.notdef/.notdef/.notdef/.notdef/.notdef/.notdef/.notdef/.notdef\n"
+			"/.notdef/.notdef/.notdef/.notdef/.notdef/.notdef/.notdef/.notdef\n"
+			"/.notdef/.notdef/.notdef/.notdef/.notdef/.notdef/.notdef/.notdef\n"
+			"/space/exclam/quotedbl/numbersign/dollar/percent/ampersand/quoteright\n"
+			"/parenleft/parenright/asterisk/plus/comma/minus/period/slash\n"
+			"/zero/one/two/three/four/five/six/seven\n"
+			"/eight/nine/colon/semicolon/less/equal/greater/question\n"
 			/* (100) */
-			"/at /A /B /C /D /E /F /G\n"
-			"/H /I /J /K /L /M /N /O\n"
-			"/P /Q /R /S /T /U /V /W\n"
-			"/X /Y /Z /bracketleft /backslash /bracketright /asciicircum /underscore\n"
-			"/quoteleft /a /b /c /d /e /f /g\n"
-			"/h /i /j /k /l /m /n /o\n"
-			"/p /q /r /s /t /u /v /w\n"
-			"/x /y /z /braceleft /bar /braceright /asciitilde /.notdef\n"
+			"/at/A/B/C/D/E/F/G\n"
+			"/H/I/J/K/L/M/N/O\n"
+			"/P/Q/R/S/T/U/V/W\n"
+			"/X/Y/Z/bracketleft/backslash/bracketright/asciicircum/underscore\n"
+			"/quoteleft/a/b/c/d/e/f/g\n"
+			"/h/i/j/k/l/m/n/o\n"
+			"/p/q/r/s/t/u/v/w\n"
+			"/x/y/z/braceleft/bar/braceright/asciitilde/.notdef\n"
 			/* (200) */
-			"/.notdef /.notdef /.notdef /.notdef /.notdef /.notdef /.notdef /.notdef\n"
-			"/.notdef /.notdef /.notdef /.notdef /.notdef /.notdef /.notdef /.notdef\n"
-			"/dotlessi /grave /acute /circumflex /tilde /macron /breve /dotaccent\n"
-			"/dieresis /.notdef /ring /cedilla /.notdef /hungarumlaut /ogonek /caron\n"
+			"/.notdef/.notdef/.notdef/.notdef/.notdef/.notdef/.notdef/.notdef\n"
+			"/.notdef/.notdef/.notdef/.notdef/.notdef/.notdef/.notdef/.notdef\n"
+			"/dotlessi/grave/acute/circumflex/tilde/macron/breve/dotaccent\n"
+			"/dieresis/.notdef/ring/cedilla/.notdef/hungarumlaut/ogonek/caron\n"
 			"%s\n"
 			"] def\n",
 			ename, enc_tb[enc - 1]);
 	}
 	fprintf(fout, "/mkfontext%d{\n"
 		"	findfont dup length dict begin\n"
-                "		{1 index /FID ne {def}{pop pop} ifelse} forall\n"
+                "		{1 index/FID ne{def}{pop pop}ifelse}forall\n"
 		"		/Encoding %s def\n"
 		"		currentdict\n"
 		"	end\n"
@@ -850,13 +843,13 @@ void define_font(char name[],
 		 int enc)
 {
 	if (enc == ENC_NATIVE || strcmp(name, "Symbol") == 0) {
-		fprintf(fout, "/F%d{dup 0.8 mul /fh exch def"
-			" /%s exch selectfont}!\n",
+		fprintf(fout, "/F%d{dup 0.8 mul/fh exch def"
+			"/%s exch selectfont}!\n",
 			num, name);
 		return;
 	}
-	fprintf(fout, "/%s%d /%s mkfontext%d\n"
-		"/F%d{dup 0.8 mul /fh exch def /%s%d exch selectfont}!\n",
+	fprintf(fout, "/%s%d/%s mkfontext%d\n"
+		"/F%d{dup 0.8 mul/fh exch def/%s%d exch selectfont}!\n",
 		name, enc, name, enc, num, name, enc);
 }
 
@@ -875,7 +868,7 @@ static void def_stems(FILE *fp)
 /* -- stem and flags -- */
 static void def_flags(FILE *fp)
 {
-	/* n len sfu - stem and n flag up */
+	/* n len sfu - stem and n flags up */
 	fprintf(fp, "/sfu{	dlw x y M %.1f %.1f RM\n"
 		"	%.1f sub 0 exch RL currentpoint stroke\n"
 		"	M dup 1 eq\n"
@@ -885,12 +878,12 @@ static void def_flags(FILE *fp)
 		"		1.6 6 -1.3 11.6 -5.6 12.8 RC\n"
 		"		fill\n"
 		"	  }{\n"
-		"		2 1 3 -1 roll {\n"
+		"		2 1 3 -1 roll{\n"
 		"			pop currentpoint\n"
 		"			0.9 -3.7 9.1 -6.4 6 -12.4 RC\n"
 		"			1 5.4 -4.2 8.4 -6 8.4 RC\n"
 		"			fill -5.4 add M\n"
-		"		} for\n"
+		"		}for\n"
 		"		1.2 -3.2 9.6 -5.7 5.6 -14.6 RC\n"
 		"		1.6 5.4 -1 10.2 -5.6 11.4 RC\n"
 		"		fill\n"
@@ -898,7 +891,7 @@ static void def_flags(FILE *fp)
 		"	ifelse}!\n",
 		STEM_XOFF, STEM_YOFF, STEM_YOFF);
 
-	/* n len sfd - stem and n flag down */
+	/* n len sfd - stem and n flags down */
 	fprintf(fp, "/sfd{	dlw x y M %.1f %.1f RM\n"
 		"	%.1f add 0 exch RL currentpoint stroke\n"
 		"	M dup 1 eq\n"
@@ -908,12 +901,12 @@ static void def_flags(FILE *fp)
 		"		1.6 -6 -1.3 -11.6 -5.6 -12.8 RC\n"
 		"		fill\n"
 		"	  }{\n"
-		"		2 1 3 -1 roll {\n"
+		"		2 1 3 -1 roll{\n"
 		"			pop currentpoint\n"
 		"			0.9 3.7 9.1 6.4 6 12.4 RC\n"
 		"			1 -5.4 -4.2 -8.4 -6 -8.4 RC\n"
 		"			fill 5.4 add M\n"
-		"		} for\n"
+		"		}for\n"
 		"		1.2 3.2 9.6 5.7 5.6 14.6 RC\n"
 		"		1.6 -5.4 -1 -10.2 -5.6 -11.4 RC\n"
 		"		fill\n"
@@ -924,13 +917,13 @@ static void def_flags(FILE *fp)
 	/* n len sfs - stem and n straight flag down */
 	fprintf(fp, "/sfs{	dlw x y M %.1f %.1f RM\n"
 		"	%.1f add 0 exch RL currentpoint stroke\n"
-		"	M 1 1 3 -1 roll {\n"
+		"	M 1 1 3 -1 roll{\n"
 		"		pop currentpoint\n"
 		"		7 %.1f RL\n"
 		"		0 %.1f RL\n"
 		"		-7 -%.1f RL\n"
 		"		fill 5.4 add M\n"
-		"	} for}!\n",
+		"	}for}!\n",
 		-STEM_XOFF, -STEM_YOFF, STEM_YOFF,
 		BEAM_DEPTH, BEAM_DEPTH, BEAM_DEPTH);
 }
@@ -940,21 +933,21 @@ static void def_gnote(FILE *fp)
 {
 	/* x y ghd - grace note head */
 	fprintf(fp, "/ghd{	xymove\n"
-		"	-1.3 1.5 RM\n"
-		"	2.4 2 5 -1 2.6 -3 RC\n"
-		"	-2.4 -2 -5 1 -2.6 3 RC fill}!\n"
+		"	1.7 1 RM\n"
+		"	-1 1.7 -4.5 -0.2 -3.4 -2 RC\n"
+		"	1 -1.7 4.5 0.2 3.4 2 RC fill}!\n"
 
 	/* len gu - grace note stem up */
-		"/gu{	0.6 setlinewidth x y M\n"
+		"/gu{	.6 SLW x y M\n"
 		"	%.1f 0 RM 0 exch RL stroke}!\n"
 
 	/* len gd - grace note stem down */
-		"/gd{	0.6 setlinewidth x y M\n"
+		"/gd{	.6 SLW x y M\n"
 		"	%.1f 0 RM 0 exch RL stroke}!\n",
 		GSTEM_XOFF, -GSTEM_XOFF);
 
 	/* n len sgu - gnote stem and n flag up */
-	fprintf(fp, "/sgu{	0.6 setlinewidth x y M %.1f 0 RM\n"
+	fprintf(fp, "/sgu{	.6 SLW x y M %.1f 0 RM\n"
 		"	0 exch RL currentpoint stroke\n"
 		"	M dup 1 eq\n"
 		"	  {\n"
@@ -963,18 +956,18 @@ static void def_gnote(FILE *fp)
 		"		1.2 4.4 -1.4 7 -3 7 RC\n"
 		"		fill\n"
 		"	  }{\n"
-		"		1 1 3 -1 roll {\n"
+		"		1 1 3 -1 roll{\n"
 		"			pop currentpoint\n"
 		"			1 -3.2 5.6 -2.8 3.2 -8 RC\n"
 		"			1.4 4.8 -2.4 5.4 -3.2 5.2 RC\n"
 		"			fill -3.5 add M\n"
-		"		} for\n"
+		"		}for\n"
 		"	  }\n"
 		"	ifelse}!\n",
 		GSTEM_XOFF);
 
 	/* n len sgd - gnote stem and n flag down */
-	fprintf(fp, "/sgd{	0.6 setlinewidth x y M %.1f 0 RM\n"
+	fprintf(fp, "/sgd{	.6 SLW x y M %.1f 0 RM\n"
 		"	0 exch RL currentpoint stroke\n"
 		"	M dup 1 eq\n"
 		"	  {\n"
@@ -983,24 +976,24 @@ static void def_gnote(FILE *fp)
 		"		1.2 -4.4 -1.4 -7 -3 -7 RC\n"
 		"		fill\n"
 		"	  }{\n"
-		"		1 1 3 -1 roll {\n"
+		"		1 1 3 -1 roll{\n"
 		"			pop currentpoint\n"
 		"			1 3.2 5.6 2.8 3.2 8 RC\n"
 		"			1.4 -4.8 -2.4 -5.4 -3.2 -5.2 RC\n"
 		"			fill 3.5 add M\n"
-		"		} for\n"
+		"		}for\n"
 		"	  }\n"
 		"	ifelse}!\n",
 		-GSTEM_XOFF);
 
 	/* n len sgs - gnote stem and n straight flag up */
-	fprintf(fp, "/sgs{	0.6 setlinewidth x y M %.1f 0 RM\n"
+	fprintf(fp, "/sgs{	.6 SLW x y M %.1f 0 RM\n"
 		"	0 exch RL currentpoint stroke\n"
-		"	M 1 1 3 -1 roll {\n"
+		"	M 1 1 3 -1 roll{\n"
 		"		pop currentpoint\n"
 		"		3 -1.5 RL 0 -2 RL -3 1.5 RL\n"
 		"		closepath fill -3 add M\n"
-		"	} for}!\n",
+		"	}for}!\n",
 		GSTEM_XOFF);
 
 	fprintf(fp,
@@ -1009,7 +1002,7 @@ static void def_gnote(FILE *fp)
 		"/gda{x y M -5 -4 RM RL stroke}!\n"
 
 		/* y ghl - grace note helper line */
-		"/ghl{	0.6 setlinewidth x -3 add exch M\n"
+		"/ghl{	.6 SLW x -3 add exch M\n"
 		"	6 0 RL stroke}!\n"
 
 		/* x1 y2 x2 y2 x3 y3 x0 y0 gsl */
